@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -93,9 +93,15 @@
 */
 #define GKRELLMD_VERSION_MAJOR   2
 #define GKRELLMD_VERSION_MINOR   2
-#define GKRELLMD_VERSION_REV     4
+#define GKRELLMD_VERSION_REV     7
 #define GKRELLMD_EXTRAVERSION    ""
 
+#define GKRELLMD_CHECK_VERSION(major,minor,rev)    \
+(GKRELLMD_VERSION_MAJOR > (major) || \
+(GKRELLMD_VERSION_MAJOR == (major) && GKRELLMD_VERSION_MINOR > (minor)) || \
+(GKRELLMD_VERSION_MAJOR == (major) && GKRELLMD_VERSION_MINOR == (minor) && \
+GKRELLMD_VERSION_REV >= (rev)))
+
 #define GKRELLMD_CONFIG     "gkrellmd.conf"
 
 #define GKRELLMD_PLUGINS_DIR		".gkrellm2/plugins-gkrellmd"
@@ -115,13 +121,14 @@ typedef struct
 	gint		major_version,
 				minor_version,
 				rev_version;
-	gint		fd;
 	gchar		*hostname;
+
+	gint		fd;
 	gboolean	served,
 				alive,
 				last_client;
 	gboolean	feature_subdisk;
-	gchar		buffer[2048];
+	GString		*input_gstring;
 	}
 	GkrellmdClient;
 
@@ -148,9 +155,12 @@ typedef struct
 	GString			*serve_gstring;
 	GkrellmdClient	*client;
 
+	GList			*config_list;
+
 	gboolean		is_plugin;
 	void			*handle;
 	gchar			*path;
+	void			(*client_input_func)(GkrellmdClient *, gchar *);
 	}
 	GkrellmdMonitorPrivate;
 
@@ -182,12 +192,28 @@ void		gkrellmd_add_serveflag_done(gboolean *);
 gboolean	gkrellmd_check_client_version(GkrellmdMonitor *mon,
 						gint major, gint minor, gint rev);
 
-  /* Small set of useful functions.
+const gchar	*gkrellmd_config_getline(GkrellmdMonitor *mon);
+
+void		gkrellmd_client_input_connect(GkrellmdMonitor *mon,
+						void (*func)(GkrellmdClient *, gchar *));
+
+
+  /* Small set of useful functions duplicated from src/utils.c.
+  |  These really should just be in the gkrellm_ namespace for sysdep code
+  |  common to gkrellm and gkrellmd, but for convenience, offer them in
+  |  both gkrellm_ and gkrellmd_ namespaces.
   */
 void		gkrellmd_free_glist_and_data(GList **list_head);
+gboolean	gkrellmd_getline_from_gstring(GString **, gchar *, gint);
 gchar		*gkrellmd_dup_token(gchar **string, gchar *delimeters);
 gboolean	gkrellmd_dup_string(gchar **dst, gchar *src);
 
+void		gkrellm_free_glist_and_data(GList **list_head);
+gboolean	gkrellm_getline_from_gstring(GString **, gchar *, gint);
+gchar		*gkrellm_dup_token(gchar **string, gchar *delimeters);
+gboolean	gkrellm_dup_string(gchar **dst, gchar *src);
+
+
   /* Plugins should use above data serve functions instead of this.
   */
 gint		gkrellmd_send_to_client(GkrellmdClient *client, gchar *buf);
@@ -201,7 +227,7 @@ gint		gkrellmd_send_to_client(GkrellmdClient *client, gchar *buf);
 */
 #define G_FILE_TEST_EXISTS		1
 #define G_FILE_TEST_IS_DIR		2
-#define	G_FILE_TEST_IS_REGULAR	3
+#define	G_FILE_TEST_IS_REGULAR	4
 
 #include <dirent.h>
 
