@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -317,16 +317,32 @@ check_maildir(Mailbox *mbox)
   |    where S is bitwise or of status flags:
   |    1: read  2: replied  4: marked  8: deleted
   |  
-  |  Evolution uses status with X-Evolution: 00000000-00x0 where x status is
-  |    0: unread   1: read   2: unread + attachments  3: read + attachments
+  |  Evolution uses status with X-Evolution: 00000000-xxxx where xxxx status is
+  |  a bitfield in hexadecimal (see enum _CamelMessageFlags in evolution/camel
+  |  source) and most importantly CAMEL_MESSAGE_SEEN = 1<<4.
   */
+  /* test if buf is a status for standard mail, mozilla or evolution 
+  */
+static gboolean
+is_status(gchar *buf)
+	{
+	if (buf[0] != 'S' && buf[0] != 'X')
+		return FALSE;
+
+	if (   !strncmp(buf, "Status:", 7)  /* Standard mail clients */
+	    || !strncmp(buf, "X-Mozilla-Status:", 17) /* Netscape */
+	    || !strncmp(buf, "X-Evolution:", 12)      /* Mozilla */
+	   )
+	    return TRUE;
+	else
+	    return FALSE;
+	}
+
 static gboolean
 status_is_old(gchar *buf)
 	{
 	gchar	c;
-
-	if (buf[0] != 'S' && buf[0] != 'X')
-		return FALSE;
+	int tmp;
 
 	/* Standard mail clients
 	*/
@@ -348,10 +364,45 @@ status_is_old(gchar *buf)
 
 	/* Evolution
 	*/
-	if (   !strncmp(buf, "X-Evolution:", 12)
-		&& (!strncmp(buf+24, "1", 1) || !strncmp(buf+24, "3", 1))
-	   )
-		return TRUE;
+	if (!strncmp(buf, "X-Evolution:", 12))
+		{
+		sscanf(buf+22, "%04x", &tmp);
+		if (tmp & (1<<4))
+			return TRUE;
+		}
+
+	return FALSE;
+	}
+
+  /* test if a mail is marked as deleted  
+  |  Evolution uses status with X-Evolution: 00000000-xxxx where xxxx status is
+  |  a bitfield in hexadecimal (see enum _CamelMessageFlags in evolution/camel source)
+  |  and most importantly CAMEL_MESSAGE_DELETED = 1<<1.
+  */
+static gboolean
+status_is_deleted(gchar *buf)
+	{
+	gint	tmp;
+
+	/* Standard mail clients
+	if (   !strncmp(buf, "Status:", 7) )
+	*/
+	/* Netscape
+	if (!strncmp(buf, "X-Mozilla-Status:", 17))
+	*/
+	/* Evolution
+	*/
+	if (!strncmp(buf, "X-Evolution:", 12))
+		{
+		sscanf(buf+22, "%04x", &tmp);
+		if (tmp & (1<<1))
+			return TRUE;
+		/* Junk is not explicitly marked as deleted but is shown as if
+		|  where in evolution
+		*/
+		if (tmp & (1<<7))
+			return TRUE;
+		}
 
 	return FALSE;
 	}
@@ -373,6 +424,9 @@ check_mbox(Mailbox *mbox)
 		mbox->mail_count = mbox->old_mail_count = mbox->new_mail_count = 0;
 		mbox->last_mtime = 0;
 		mbox->last_size = 0;
+		if (_GK.debug_level & DEBUG_MAIL)
+			printf("check_mbox can't stat(%s): %s\n", mbox->path,
+						g_strerror(errno));
 		return FALSE;
 		}
 
@@ -384,7 +438,12 @@ check_mbox(Mailbox *mbox)
 	   )
 		{
 		if ((f = fopen(mbox->path, "r")) == NULL)
+			{
+			if (_GK.debug_level & DEBUG_MAIL)
+				printf("check_mbox can't fopen(%s): %s\n", mbox->path,
+							g_strerror(errno));
 			return FALSE;
+			}
 		mbox->mail_count = 0;
 		mbox->old_mail_count = 0;
 		while(fgets(buf, sizeof(buf), f))
@@ -406,10 +465,19 @@ check_mbox(Mailbox *mbox)
 				in_header = TRUE;
 				marked_read = FALSE;
 				}
-			else if (in_header && status_is_old(buf) && !marked_read)
+			else if (in_header && is_status(buf))
 				{
-				mbox->old_mail_count += 1;
-				marked_read = TRUE;
+				if (status_is_old(buf) && !marked_read)
+					{
+					mbox->old_mail_count += 1;
+					marked_read = TRUE;
+				    }
+				if (status_is_deleted(buf))
+					{
+					if (marked_read)
+						mbox->old_mail_count -= 1;
+					mbox->mail_count -= 1;
+				    }
 				}
 			else if (in_header && mbox->is_internal)
 				{
@@ -441,9 +509,9 @@ check_mbox(Mailbox *mbox)
 					mbox->path,
 					mbox->mail_count, mbox->old_mail_count);
 		}
-	/* Set the animation state when new mail count changes, and override
-	|  the animation to false if mbox has been accessed since last modify
-	|  (A MUA has probably read the mbox).
+
+	/* If mbox has been accessed since last modify a MUA has probably read
+	|  the mbox.
 	*/
 	mbox->new_mail_count = mbox->mail_count - mbox->old_mail_count;
 	if (s.st_atime > s.st_mtime)
@@ -478,6 +546,7 @@ update_mail(GkrellmdMonitor *mon, gboolean force)
 			{
 			mbox->changed = TRUE;
 			mail_need_serve = TRUE;
+			gkrellmd_need_serve(mon);
 			}
 		mbox->prev_mail_count = mbox->mail_count;
 		mbox->prev_new_mail_count = mbox->new_mail_count;
