@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -22,6 +22,8 @@
 #include "gkrellmd.h"
 #include "gkrellmd-private.h"
 
+#include "../src/inet.h"
+
 #ifndef HAVE_GETADDRINFO
 struct addrinfo
 	{
@@ -43,7 +45,9 @@ struct addrinfo
 struct GkrellmdConfig	_GK;
 GkrellmdTicks			GK;
 
-GList			*gkrellmd_client_list;
+GList			*gkrellmd_client_list,
+				*gkrellmd_plugin_config_list;
+
 static GList	*allow_host_list;
 
 static gboolean	detach_flag;
@@ -187,6 +191,102 @@ is_valid_reverse(char *addr, char *host, sa_family_t family)
 	}
 #endif
 
+/* Check for CIDR match.
+ */
+static gboolean
+cidr_match(struct sockaddr *sa, socklen_t salen, char *allowed)
+{
+#ifdef HAVE_GETADDRINFO
+	struct addrinfo hints, *res;
+	struct sockaddr_storage	ss;
+	char		*buf;
+	char		*p, *ep;
+	guchar		*addr, *pat;
+	u_int32_t	mask;
+	int			plen, i;
+	gboolean	result;
+
+	buf = g_strdup(allowed);
+	plen = -1;
+	if ((p = strchr(buf, '/')) != NULL)
+		{
+		plen = strtoul(p + 1, &ep, 10);
+		if (errno != 0 || ep == NULL || *ep != '\0' || plen < 0)
+			{
+			g_free(buf);
+			return FALSE;
+			}
+		*p = '\0';
+		allowed = buf;
+		}
+
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_family = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
+	result = getaddrinfo(allowed, NULL, &hints, &res);
+	g_free(buf);
+	if (result != 0)
+		return FALSE;
+	memcpy(&ss, res->ai_addr, res->ai_addrlen);
+	freeaddrinfo(res);
+
+	if (sa->sa_family != ((struct sockaddr *)&ss)->sa_family)
+		return FALSE;
+	switch (sa->sa_family)
+		{
+#if defined(INET6)
+		case AF_INET6:
+			if (plen < 0)
+				plen = 128;
+			if (plen > 128)
+				return FALSE;
+			if (((struct sockaddr_in6 *)&ss)->sin6_scope_id != 0 &&
+			    ((struct sockaddr_in6 *)&ss)->sin6_scope_id !=
+			    ((struct sockaddr_in6 *)sa)->sin6_scope_id)
+				return FALSE;
+			addr = (guchar *)&((struct sockaddr_in6 *)sa)->sin6_addr;
+			pat = (guchar *)&((struct sockaddr_in6 *)&ss)->sin6_addr;
+			i = 0;
+			while (plen > 0)
+				{
+				if (plen < 32)
+					{
+					mask = htonl(~(0xffffffff >> plen));
+					if ((*(u_int32_t *)&addr[i] & mask) !=
+					    (*(u_int32_t *)&pat[i] & mask))
+						return FALSE;
+					break;
+					}
+				if (*(u_int32_t *)&addr[i] !=
+				    *(u_int32_t *)&pat[i])
+					return FALSE;
+				i += 4;
+				plen -= 32;
+				}
+			break;
+#endif
+		case AF_INET:
+			if (plen < 0)
+				plen = 32;
+			if (plen > 32)
+				return FALSE;
+			addr = (guchar *)&((struct sockaddr_in *)sa)->sin_addr;
+			pat = (guchar *)&((struct sockaddr_in *)&ss)->sin_addr;
+			mask = htonl(~(0xffffffff >> plen));
+			if ((*(u_int32_t *)addr & mask) !=
+			    (*(u_int32_t *)pat & mask))
+				return FALSE;
+			break;
+		default:
+			return FALSE;
+		}
+	return TRUE;
+#else
+	return FALSE;
+#endif
+}
+
 static gboolean
 allow_host(GkrellmdClient *client, struct sockaddr *sa, socklen_t salen)
 	{
@@ -236,6 +336,9 @@ allow_host(GkrellmdClient *client, struct sockaddr *sa, socklen_t salen)
 		   )
 			return TRUE;
 
+		if (addr && cidr_match(sa, salen, allowed))
+			return TRUE;
+
 		/* Check for simple IPv4 subnet match.  Worry later about ranges and
 		|  other hosts_access type patterns.
 		*/
@@ -337,7 +440,7 @@ parse_config(gchar *config, gchar *arg)
 	{
 	if (!strcmp(config, "clear-hosts") || !strcmp(config, "c"))
 		{
-		gkrellmd_free_glist_and_data(&allow_host_list);
+		gkrellm_free_glist_and_data(&allow_host_list);
 		return 0;
 		}
 	if (!strcmp(config, "detach") || !strcmp(config, "d"))
@@ -370,6 +473,12 @@ parse_config(gchar *config, gchar *arg)
 		_GK.nfs_interval = atoi(arg);
 	else if (!strcmp(config, "inet-interval"))
 		_GK.inet_interval = atoi(arg);
+	else if (!strcmp(config, "mbmon-port"))
+		_GK.mbmon_port = atoi(arg);
+	else if (!strcmp(config, "net-timer"))
+		_GK.net_timer = g_strdup(arg);
+	else if (!strcmp(config, "debug-level") || !strcmp(config, "debug"))
+		_GK.debug_level = (gint) strtoul(arg, NULL, 0);
 #if !defined(WIN32)
 	else if (!strcmp(config, "pidfile"))
 		_GK.pidfile = g_strdup(arg);
@@ -402,8 +511,10 @@ parse_config(gchar *config, gchar *arg)
 static void
 load_config(gchar *path)
 	{
-	FILE	*f;
-	gchar	buf[128], config[32], arg[128];
+	FILE			*f;
+	PluginConfigRec	*cfg;
+	gchar			buf[128], config[32], arg[128];
+	gchar			*s, *plugin_config_block = NULL;
 
 	f = fopen(path, "r");
 	if (!f)
@@ -412,13 +523,67 @@ load_config(gchar *path)
 		{
 		if (!buf[0] || buf[0] == '#')
 			continue;
-		arg[0] = '\0';
-		sscanf(buf, "%31s %127s", config, arg);
-		parse_config(config, arg);
+		if (buf[0] == '[' || buf[0] == '<')
+			{
+			if (buf[1] == '/')
+				{
+				g_free(plugin_config_block);
+				plugin_config_block = NULL;
+				}
+			else
+				{
+				if (   (s = strchr(buf, ']')) != NULL
+					|| (s = strchr(buf, '>')) != NULL
+				   )
+					*s = '\0';
+				plugin_config_block = g_strdup(&buf[1]);
+				}
+			continue;
+			}
+		if (plugin_config_block)
+			{
+			cfg = g_new0(PluginConfigRec, 1);
+			cfg->name = g_strdup(plugin_config_block);
+			if ((s = strchr(buf, '\n')) != NULL)
+				*s = '\0';
+			cfg->line = g_strdup(buf);
+			gkrellmd_plugin_config_list
+						= g_list_append(gkrellmd_plugin_config_list, cfg);
+			}
+		else	/* main gkrellmd config line */
+			{
+			arg[0] = '\0';
+			sscanf(buf, "%31s %127s", config, arg);
+			parse_config(config, arg);
+			}
 		}
 	fclose(f);
 	}
 
+const gchar *
+gkrellmd_config_getline(GkrellmdMonitor *mon)
+	{
+	GList			*list;
+	PluginConfigRec	*cfg;
+
+	if (!mon->privat)
+		{
+		mon->privat = g_new0(GkrellmdMonitorPrivate, 1);
+		mon->privat->config_list = gkrellmd_plugin_config_list;
+		}
+
+	for (list = mon->privat->config_list; list; list = list->next)
+		{
+		cfg = (PluginConfigRec *) list->data;
+		if (!strcmp(cfg->name, mon->name))
+			{
+			mon->privat->config_list = list->next;
+			return cfg->line;
+			}
+		}
+	return NULL;
+	}
+
 static void
 read_config(void)
 	{
@@ -779,7 +944,10 @@ main(gint argc, gchar **argv)
 
 	if (_GK.verbose)
 		printf("update_HZ=%d\n", _GK.update_HZ);
-	if (detach_flag && !_GK.log_plugins && !_GK.list_plugins)
+
+	if (   detach_flag
+	    && !_GK.log_plugins && !_GK.list_plugins && _GK.debug_level == 0
+	   )
 		detach_from_terminal();
 	else
 		{
