@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -323,6 +323,7 @@ typedef struct
 				changed;
 	gint		device_number,
 				unit_number;
+	gboolean	virtual;
 	guint64		rb,
 				wb;
 	}
@@ -394,7 +395,7 @@ add_subdisk(gchar *subdisk_name, gchar *disk_name, gint subdisk)
 	}
 
 static void
-disk_assign_data(DiskData *disk, guint64 rb, guint64 wb)
+disk_assign_data(DiskData *disk, guint64 rb, guint64 wb, gboolean virtual)
 	{
 	if (disk)
 		{
@@ -404,6 +405,7 @@ disk_assign_data(DiskData *disk, guint64 rb, guint64 wb)
 			disk->changed = FALSE;
 		disk->rb = rb;
 		disk->wb = wb;
+		disk->virtual = virtual;
 		}
 	}
 
@@ -421,7 +423,7 @@ gkrellm_disk_units_are_blocks(void)
 
 void
 gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
-			guint64 rb, guint64 wb)
+			guint64 rb, guint64 wb, gboolean virtual)
 	{
 	GList		*list;
 	DiskData	*disk = NULL;
@@ -444,11 +446,11 @@ gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
 		if (name)
 			disk = add_disk(name, order, device_number, unit_number);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
-gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb)
+gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb, gboolean virtual)
 	{
 	DiskData	*disk;
 	gchar		name[32];
@@ -460,11 +462,12 @@ gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb)
 		sprintf(name, "%s%c", _("Disk"), 'A' + n);
 		disk = add_disk(name, n, 0, 0);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
-gkrellm_disk_assign_data_by_name(gchar *name, guint64 rb, guint64 wb)
+gkrellm_disk_assign_data_by_name(gchar *name,
+			guint64 rb, guint64 wb, gboolean virtual)
 	{
 	GList		*list;
 	DiskData	*disk = NULL;
@@ -482,7 +485,7 @@ gkrellm_disk_assign_data_by_name(gchar *name, guint64 rb, guint64 wb)
 		order = gkrellm_sys_disk_order_from_name(name);
 		disk = add_disk(name, order, 0, 0);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
@@ -514,7 +517,7 @@ gkrellm_disk_subdisk_assign_data_by_name(gchar *subdisk_name, gchar *disk_name,
 			return;
 		disk = add_subdisk(subdisk_name, disk_name, subdisk);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, FALSE);
 	}
 
 static void
@@ -550,8 +553,14 @@ serve_disk_data(GkrellmdMonitor *mon, gboolean first_serve)
 		if (!disk->changed && !first_serve)
 			continue;
 		if (!disk->subdisk_parent)
-			snprintf(buf, sizeof(buf), "%s %llu %llu\n",
-						disk->name, disk->rb, disk->wb);
+			{
+			if (gkrellmd_check_client_version(mon, 2, 2, 7) && disk->virtual)
+				snprintf(buf, sizeof(buf), "%s virtual %llu %llu\n",
+							disk->name, disk->rb, disk->wb);
+			else
+				snprintf(buf, sizeof(buf), "%s %llu %llu\n",
+							disk->name, disk->rb, disk->wb);
+			}
 		else if (mon->privat->client->feature_subdisk)
 			snprintf(buf, sizeof(buf), "%s %s %llu %llu\n",
 						disk->name, disk->subdisk_parent, disk->rb, disk->wb);
@@ -666,7 +675,7 @@ update_inet(GkrellmdMonitor *mon, gboolean first_update)
 
 	if (first_update || check_tcp == 0)
 		{
-		gkrellmd_free_glist_and_data(&inet_dead_list);
+		gkrellm_free_glist_and_data(&inet_dead_list);
 		inet_new = FALSE;
 		for (list = inet_list; list; list = list->next)
 			{
@@ -823,6 +832,11 @@ init_inet_monitor(void)
 	}
 
 /* ======================================================= */
+
+#define TIMER_TYPE_NONE	0
+#define TIMER_TYPE_PPP	1
+#define TIMER_TYPE_IPPP	2
+
 typedef struct
 	{
 	gchar		*name;
@@ -831,16 +845,26 @@ typedef struct
 				up_prev,
 				up_event,
 				down_event;
+
+	gboolean	timed_changed;
+	time_t		up_time;
+
 	gulong		rx,
 				tx;
 	}
 	NetData;
 
-GList	*net_list,
-		*net_sys_list;
+static NetData	*net_timer;
+
+static GList	*net_list,
+				*net_sys_list;
+
+static time_t	net_timer0;
+static gint		net_timer_type;
 
 static gboolean	net_use_routed;
 
+
 gchar *
 gkrellm_net_mon_first(void)
 	{
@@ -874,6 +898,21 @@ gkrellm_net_use_routed(gboolean real_routed /* not applicable in server */)
 	net_use_routed = TRUE;
 	}
 
+static NetData *
+net_new(gchar *name)
+	{
+	NetData	*net;
+
+	net = g_new0(NetData, 1);
+	net->name = g_strdup(name);
+	net_list = g_list_append(net_list, net);
+
+	if (net_timer_type != TIMER_TYPE_NONE && !strcmp(_GK.net_timer, net->name))
+		net_timer = net;
+
+	return net;
+	}
+
 void
 gkrellm_net_assign_data(gchar *name, gulong rx, gulong tx)
 	{
@@ -893,11 +932,8 @@ gkrellm_net_assign_data(gchar *name, gulong rx, gulong tx)
 			}
 		}
 	if (!list)
-		{
-		net = g_new0(NetData, 1);
-		net->name = g_strdup(name);
-		net_list = g_list_append(net_list, net);
-		}
+		net = net_new(name);
+
 	if (GK.second_tick && !net_use_routed)
 		net->up = TRUE;
 	net->rx = rx;
@@ -917,11 +953,8 @@ gkrellm_net_routed_event(gchar *name, gboolean routed)
 			break;
 		}
 	if (!list)
-		{
-		net = g_new0(NetData, 1);
-		net->name = g_strdup(name);
-		net_list = g_list_append(net_list, net);
-		}
+		net = net_new(name);
+
 	if (routed)
 		net->up_event = TRUE;
 	else
@@ -932,13 +965,19 @@ gkrellm_net_routed_event(gchar *name, gboolean routed)
 void
 gkrellm_net_add_timer_type_ppp(gchar *name)
 	{
-	/* Not supported remotely */
+	if (!_GK.net_timer)
+		return;
+	if (name && !strncmp(_GK.net_timer, name, strlen(name) - 1))
+		net_timer_type = TIMER_TYPE_PPP;
 	}
 
 void
 gkrellm_net_add_timer_type_ippp(gchar *name)
 	{
-	/* Not supported remotely */
+	if (!_GK.net_timer)
+		return;
+	if (name && !strncmp(_GK.net_timer, name, strlen(name) - 1))
+		net_timer_type = TIMER_TYPE_IPPP;
 	}
 
 void
@@ -950,8 +989,9 @@ gkrellm_net_set_lock_directory(gchar *dir)
 static void
 update_net(GkrellmdMonitor *mon, gboolean first_update)
 	{
-	GList	*list;
-	NetData	*net;
+	GList		*list;
+	NetData		*net;
+	gint		up_time = 0;
 
 	if (GK.second_tick)
 		{
@@ -980,6 +1020,46 @@ update_net(GkrellmdMonitor *mon, gboolean first_update)
 				net->down_event = TRUE;
 			}
 		}
+
+	if (net_timer && GK.second_tick)
+		{
+		if (net_timer_type == TIMER_TYPE_PPP)
+			{
+			struct stat		st;
+			gchar			buf[256];
+
+			if (net_timer->up_event)
+				{
+				snprintf(buf, sizeof(buf), "/var/run/%s.pid", net_timer->name);
+				if (stat(buf, &st) == 0)
+					net_timer0 = st.st_mtime;
+				else
+					time(&net_timer0);
+				}
+			if (net_timer->up)
+				up_time = (int) (time(0) - net_timer0);
+			}
+		else if (net_timer_type == TIMER_TYPE_IPPP)
+			{
+			/* get all isdn status from its connect state because the
+			|  net_timer->up can be UP even with isdn line not connected.
+			|  Can't get time history if gkrellmd started after connects.
+			*/
+			static gboolean		old_connected;
+			gboolean			connected;
+
+			connected = gkrellm_sys_net_isdn_online();
+			if (connected && !old_connected)
+				time(&net_timer0);			/* New session just started */
+			old_connected = connected;
+
+			up_time = (int) (time(0) - net_timer0);
+			}
+		if (up_time != net_timer->up_time)
+			net_timer->timed_changed = TRUE;
+		net_timer->up_time = up_time;
+		}
+
 	gkrellmd_need_serve(mon);	/* serve func checks for changed */
 	}
 
@@ -1022,18 +1102,36 @@ serve_net_data(GkrellmdMonitor *mon, gboolean first_serve)
 				net->up_event = net->down_event = FALSE;
 			}
 		}
+
+	if (net_timer && GK.second_tick)
+		{
+		if (net_timer->timed_changed || first_serve)
+			{
+			gkrellmd_set_serve_name(mon, "net_timer");
+			sprintf(buf, "%s %d\n", net_timer->name, (gint)net_timer->up_time);
+			gkrellmd_serve_data(mon, buf);
+			}
+		}
 	}
 
 static void
 serve_net_setup(GkrellmdMonitor *mon)
 	{
 	GkrellmdClient	*client = mon->privat->client;
+	gchar			buf[128];
 
 	/* The client <-> server link always uses routed mode, but the client
 	|  needs to know if server sysdep uses routed for config purposes.
 	*/
 	if (net_use_routed)
 		gkrellmd_send_to_client(client, "<net_setup>\nnet_use_routed\n");
+
+	if (net_timer_type != TIMER_TYPE_NONE)
+		{
+		snprintf(buf, sizeof(buf),
+				"<net_setup>\nnet_timer %s\n", _GK.net_timer);
+		gkrellmd_send_to_client(client, buf);
+		}
 	}
 
 static GkrellmdMonitor net_monitor =
@@ -1048,6 +1146,7 @@ static GkrellmdMonitor net_monitor =
 static GkrellmdMonitor *
 init_net_monitor(void)
 	{
+	net_timer_type = TIMER_TYPE_NONE;
 	if (gkrellm_sys_net_init())
 		return &net_monitor;
 	return NULL;
@@ -2114,6 +2213,12 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 
 	gkrellmd_send_to_client(client, "<gkrellmd_setup>\n");
 
+	s = g_strdup_printf("<version>\ngkrellmd %d.%d.%d%s\n",
+				GKRELLMD_VERSION_MAJOR, GKRELLMD_VERSION_MINOR,
+				GKRELLMD_VERSION_REV, GKRELLMD_EXTRAVERSION);
+	gkrellmd_send_to_client(client, s);
+	g_free(s);
+
 	lc = localeconv();
 	sprintf(buf, "%c\n", *lc->decimal_point);
 	s = g_strconcat("<decimal_point>\n", buf, "\n", NULL);
@@ -2160,13 +2265,23 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 	gkrellmd_send_to_client(client, "</gkrellmd_setup>\n");
 	}
 
+void
+gkrellmd_client_input_connect(GkrellmdMonitor *mon,
+			void (*func)(GkrellmdClient *, gchar *))
+	{
+	mon->privat->client_input_func = func;
+	}
+
 void
 gkrellmd_client_read(gint fd, gint nbytes)
 	{
-	GList			*list;
-	GkrellmdClient	*client;
-	gchar			buf[128];
-	gint			n, buflen;
+	GList					*list;
+	GkrellmdClient			*client;
+	GkrellmdMonitor			*mon;
+	GkrellmdMonitorPrivate	*mp;
+	gchar					buf[513], *s, *e;
+	gint					n, buflen;
+	static void				(*func)(GkrellmdClient *, gchar *);
 
 	for (list = gkrellmd_client_list; list; list = list->next)
 		{
@@ -2174,9 +2289,10 @@ gkrellmd_client_read(gint fd, gint nbytes)
 		if (client->fd != fd)
 			continue;
 
-		/* Just drain it.  Don't use client sent data yet.
-		*/
-		buflen = sizeof(buf);
+		if (!client->input_gstring)
+			client->input_gstring = g_string_new("");
+
+		buflen = sizeof(buf) - 1;
 		while (nbytes > 0)
 			{
 			n = (nbytes > buflen) ? buflen : nbytes;
@@ -2184,8 +2300,36 @@ gkrellmd_client_read(gint fd, gint nbytes)
 			if (n <= 0)
 				break;
 			nbytes -= n;
-			printf("received unexpected %d bytes from %s\n",
-						n, client->hostname);
+			buf[n] = '\0';
+			client->input_gstring =
+						g_string_append(client->input_gstring, buf);
 			}
+		break;
+		}
+	if (!list)
+		return;
+	while (gkrellmd_getline_from_gstring(&client->input_gstring,
+				buf, sizeof(buf) - 1))
+		{
+		if (*buf == '<')
+			{
+			func = NULL;
+			s = buf + 1;
+			for (list = gkrellmd_monitor_list; list; list = list->next)
+				{
+				mon = (GkrellmdMonitor *) list->data;
+				mp = mon->privat;
+				n = strlen(mp->serve_name);
+				e = s + n;
+				if (*e == '>' && !strncmp(mp->serve_name, s, n))
+					{
+					func = mp->client_input_func;
+					break;
+					}
+				}
+			}
+		else if (func)
+			(*func)(client, buf);
+//		printf("%s: %s", client->hostname, buf);
 		}
 	}
