@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -436,7 +436,7 @@ destroy_alert(GkrellmAlert *alert)
 		if (MONITOR_ENABLED(gap->mon) && gap->destroy_func)
 			(*gap->destroy_func)(alert, data);
 		}
-	free_glist_and_data(&alert->plugin_list);	/* should all be detached */
+	gkrellm_free_glist_and_data(&alert->plugin_list);	/* should all be detached */
 	g_free(alert->name);
 	g_free(alert->unit_string);
 	g_free(alert->alarm_command);
@@ -633,6 +633,11 @@ gkrellm_reset_panel_alerts(GkrellmPanel *p)
 			done = FALSE;
 			reset_alert(alert, FALSE);
 			gkrellm_alert_list = g_list_remove(gkrellm_alert_list, alert);
+			alert->panel = NULL;
+			alert->cb_trigger = NULL;
+			alert->cb_trigger_data = NULL;
+			alert->cb_stop = NULL;
+			alert->cb_stop_data = NULL;
 			break;
 			}
 		}
@@ -643,7 +648,7 @@ gkrellm_reset_panel_alerts(GkrellmPanel *p)
   |  retriggered.  Surely an alert going off is a good time to change themes.
   */
 void
-reset_all_alerts(void)
+gkrellm_alert_reset_all(void)
 	{
 	GList	*list;
 
@@ -654,7 +659,7 @@ reset_all_alerts(void)
 	}
 
 void
-update_alerts(void)
+gkrellm_alert_update(void)
 	{
 	GList					*list, *plist;
 	GkrellmAlert			*alert;
