@@ -1,5 +1,5 @@
 /*
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -428,7 +428,10 @@ draw_time_left_decal(Battery *bat, gboolean force)
 		case BATTERYDISPLAY_PERCENT:
 		default:
 			t = bat->percent;
-			snprintf(buf, sizeof(buf), "%d%%", t);
+			if (t == -1)	/* APM battery flags should cause hide... but */
+				snprintf(buf, sizeof(buf), "no bat");
+			else
+				snprintf(buf, sizeof(buf), "%d%%", t);
 			break;
 		}
 
@@ -456,7 +459,7 @@ update_battery_panel(Battery *bat)
 	gkrellm_panel_show(p);
 
 	if (bat->time_left > 0 && bat->charging)
-		bat->charge_rate = (gfloat) bat->percent / (gfloat) bat->time_left;
+		bat->charge_rate = (gfloat) (100 - bat->percent) / (gfloat) bat->time_left;
 	else
 		bat->charge_rate = 0.0;
 
@@ -629,14 +632,13 @@ create_battery_panel(Battery *bat, gboolean first_create)
 		g_signal_connect(G_OBJECT(p->drawing_area), "leave_notify_event",
                 G_CALLBACK(cb_panel_leave), bat);
 		 }
+
+	gkrellm_setup_decal_launcher(p, &launch, bat->time_decal);
 	if (   (bat == composite_battery && enable_composite)
 		|| (bat->id == 0 && composite_battery && !enable_composite)
 		|| (bat->id == 0 && !composite_battery)
 		)
-		{
-		gkrellm_setup_decal_launcher(p, &launch, bat->time_decal);
 		launch_battery = bat;
-		}
 
 	if (bat == composite_battery)
 		bat->enabled = enable_composite;
