@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -24,6 +24,10 @@
 
 #include <math.h>
 
+
+static void	set_grid_resolution_spin_button(GkrellmChart *, gint);
+
+
   /* For grid images of height 2 pixels, make room at bottom of chartdata
   |  window so both pixel lines will show.
   */
@@ -513,9 +517,8 @@ draw_chartdata_lines(GkrellmChart *cp, GkrellmChartdata *cd, gint i0)
 static void
 set_auto_grid_resolution(GkrellmChart *cp, gint maxval)
 	{
-	static void	set_grid_resolution_spin_button(GkrellmChart *, gint);
 	GkrellmChartconfig	*cf = cp->config;
-	gint		grids, grid_res, maxval_base;
+	gint				grids, grid_res, maxval_base;
 
 	if (maxval <= cp->maxval_auto_base)
 		maxval = cp->maxval_auto_base;
@@ -994,6 +997,13 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 	   )
 		cp->text_format_reuse = TRUE;
 
+	if (_GK.debug_level & DEBUG_CHART_TEXT)
+		{
+		printf("\n");
+		if (!cp->text_format_reuse)
+			printf("draw_chart_text: [%s]\n", str);
+		}
+
 	if (   !cp->text_format_reuse
 		|| cp->text_run_sequence_id != cp->bg_sequence_id
 	   )
@@ -1134,29 +1144,43 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 				tr->cache_valid = FALSE;
 			tr->x = x;
 			tr->y = y;
-#if 0
-gchar buf[128];
-strncpy(buf, s, text_length);
-buf[text_length] = '\0';
-if (!tr->cache_valid)
-printf("pango draw chart text: %s\n", buf);
-#endif
+
+			if (_GK.debug_level & DEBUG_CHART_TEXT)
+				{
+				gchar	buf[128];
+
+				strncpy(buf, s, text_length);
+				buf[text_length] = '\0';
+				printf("draw_chart_text: [%s] ", buf);
+				}
 			
 			offset = cp->baseline_ref - tr->baseline;	/* align baselines */
-			if (!tr->cache_valid)
+			if (_GK.chart_text_no_fill)
+	    		gkrellm_draw_text(cp->pixmap, ts, x, y + offset, s,
+						text_length);
+			else /* Default text draw effect is fill solid and can use cache */
 				{
-				gdk_draw_drawable(cp->bg_text_pixmap, _GK.draw1_GC,
-						cp->bg_pixmap,
+				if (!tr->cache_valid)
+					{
+					if (_GK.debug_level & DEBUG_CHART_TEXT)
+						printf("pango ");
+					gdk_draw_drawable(cp->bg_text_pixmap, _GK.draw1_GC,
+							cp->bg_pixmap,
+							x - 1, y + tr->y_ink + offset - 1,
+							x - 1, y + tr->y_ink + offset - 1,
+							tr->w + shadow + 2, tr->h + shadow + 1);
+		    		gkrellm_draw_text(cp->bg_text_pixmap, ts, x, y + offset, s,
+							text_length);
+					}
+				if (_GK.debug_level & DEBUG_CHART_TEXT)
+					printf("x=%d y=%d w=%d h=%d\n",
+							x - 1, y + tr->y_ink + offset - 1,
+							tr->w + shadow + 2, tr->h + shadow + 1);
+				gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_text_pixmap,
 						x - 1, y + tr->y_ink + offset - 1,
 						x - 1, y + tr->y_ink + offset - 1,
 						tr->w + shadow + 2, tr->h + shadow + 1);
-	    		gkrellm_draw_text(cp->bg_text_pixmap, ts, x, y + offset, s,
-						text_length);
 				}
-			gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_text_pixmap,
-					x - 1, y + tr->y_ink + offset - 1,
-					x - 1, y + tr->y_ink + offset - 1,
-					tr->w + shadow + 2, tr->h + shadow + 1);
 			}
 		if (field_width && !fw_right)
 			x += (field_width > tr->w) ? field_width : tr->w;
