@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -57,10 +57,11 @@
 #endif
 
 #if defined(__NetBSD__)
-#define HAVE_GETADDRINFO    1
+#define HAVE_GETADDRINFO	1
 #endif
 
 #if defined(__solaris__)
+/* getaddrinfo is related to IPv6 stuff */
 # include <netconfig.h>
 # if defined(NC_INET6)
 #  define HAVE_GETADDRINFO	1
@@ -75,8 +76,13 @@ typedef struct
 	}
 	KeyTable;
 
+static gint		server_major_version,
+				server_minor_version,
+				server_rev_version;
+
 static gint		client_input_id,
 				client_fd;
+static gboolean	server_alive;
 
 static gchar	server_buf[4097];
 static gint		buf_index;
@@ -86,6 +92,20 @@ static gchar	server_decimal_point = '.';
 static gboolean	need_locale_fix = FALSE;
 
 
+gboolean
+gkrellm_client_check_server_version(gint major, gint minor, gint rev)
+    {
+    if (   server_major_version > major
+        || (server_major_version == major && server_minor_version > minor)
+        || (   server_major_version == major && server_minor_version == minor
+            && server_rev_version >= rev
+           )
+       )
+        return TRUE;
+    return FALSE;
+    }
+
+
   /* There can be a locale decimal point mismatch with the server.
   */
 static void
@@ -248,10 +268,11 @@ client_sys_proc_init(void)
 /* ================================================================= */
 typedef struct
 	{
-	gchar	*name;
-	gchar	*subdisk_parent;
-	guint64	rblk,
-			wblk;
+	gchar		*name;
+	gchar		*subdisk_parent;
+	guint64		rblk,
+				wblk;
+	gboolean	virtual;
 	}
 	DiskData;
 
@@ -267,10 +288,18 @@ client_disk_line_from_server(gchar *line)
 	gchar		name[16], s1[32], s2[32], s3[32];
 	guint64		rblk, wblk;
 	gint		n;
+	gboolean	virtual = FALSE;
 
 	n = sscanf(line, "%15s %31s %31s %31s", name, s1, s2, s3);
 	if (n == 4)
 		{
+		if (   gkrellm_client_check_server_version(2, 2, 7)
+			&& !strcmp(s1, "virtual")
+		   )
+			{
+			virtual = TRUE;
+			s1[0] = '\0';
+			}
 		rblk = strtoull(s2, NULL, 0);
 		wblk = strtoull(s3, NULL, 0);
 		}
@@ -300,6 +329,7 @@ client_disk_line_from_server(gchar *line)
 		{
 		disk->rblk = rblk;
 		disk->wblk = wblk;
+		disk->virtual = virtual;
 		}
 	}
 
@@ -317,7 +347,7 @@ read_disk_data(void)
 					disk->subdisk_parent, disk->rblk, disk->wblk);
 		else
 			gkrellm_disk_assign_data_by_name(disk->name,
-					disk->rblk, disk->wblk);
+					disk->rblk, disk->wblk, disk->virtual);
 		}
 	}
 
@@ -336,7 +366,7 @@ client_sys_disk_init(void)
 
 	/* Disk monitor config needs to know will be using assign by name
 	*/
-	gkrellm_disk_assign_data_by_name(NULL, 0, 0);
+	gkrellm_disk_assign_data_by_name(NULL, 0, 0, FALSE);
 	}
 
 static void
@@ -352,12 +382,19 @@ typedef struct
 	gchar		*name;
 	gboolean	up_event,
 				down_event;
+
+	gint		up_time;
+
 	gulong		rx,
 				tx;
 	}
 	NetData;
 
-GList	*net_list;
+static NetData	*net_timer;
+
+static GList	*net_list;
+
+static gchar	*net_timer_name;
 
 static gboolean	net_server_use_routed;
 
@@ -387,6 +424,9 @@ client_net_line_from_server(gchar *line)
 		net_list = g_list_append(net_list, net);
 		net->rx = rx;
 		net->tx = tx;
+
+		if (net_timer_name && !strcmp(net_timer_name, net->name))
+			net_timer = net;
 		}
 	}
 
@@ -420,6 +460,22 @@ client_net_routed_line_from_server(gchar *line)
 		}
 	}
 
+gint
+gkrellm_client_server_get_net_timer(void)
+	{
+	return net_timer ? net_timer->up_time : 0;
+	}
+
+static void
+client_net_timer_line_from_server(gchar *line)
+	{
+	gchar		name[32];
+
+	if (!net_timer)
+		return;
+	sscanf(line, "%s %d", name, &net_timer->up_time);
+	}
+
 static void
 check_net_routes(void)
 	{
@@ -456,16 +512,27 @@ client_sys_net_init(void)
 	{
 	gkrellm_net_client_divert(read_net_data, check_net_routes, NULL);
 	gkrellm_net_use_routed(net_server_use_routed);
+
+	if (net_timer_name)
+		gkrellm_net_server_has_timer();
 	}
 
 static void
 client_net_setup(gchar *line)
 	{
+	gchar	buf[32];
+
 	/* This is the server sysdep net_use_routed value.  The client <-> server
 	|  link always uses routed mode.
 	*/
 	if (!strcmp(line, "net_use_routed"))
 		net_server_use_routed = TRUE;
+
+	if (!strncmp(line, "net_timer", 9))
+		{
+		sscanf(line, "%*s %31s\n", buf);
+		net_timer_name = g_strdup(buf);
+		}
 	}
 
 
@@ -1204,6 +1271,13 @@ client_time_line_from_server(gchar *s)
 	}
 
 /* ================================================================= */
+static void
+client_server_version_setup(gchar *line)
+	{
+	sscanf(line, "%*s %d.%d.%d", &server_major_version,
+                    &server_minor_version, &server_rev_version);
+	}
+
 static void
 client_hostname_setup(gchar *s)
 	{
@@ -1353,6 +1427,7 @@ client_server_reconnect_timeout(gchar *line)
 /* ================================================================= */
 KeyTable	setup_table[] =
 	{
+	{"<version>",		client_server_version_setup },
 	{"<sensors_setup>",	client_sensors_setup },
 	{"<hostname>",		client_hostname_setup },
 	{"<sysname>",		client_sysname_setup },
@@ -1424,6 +1499,50 @@ gkrellm_client_plugin_serve_data_connect(GkrellmMonitor *mon,
 				= g_list_append(client_plugin_serve_data_list, plug);
 	}
 
+static gboolean
+client_send_to_server(gchar *buf)
+	{
+	gint	n;
+
+	if (!server_alive || client_fd < 0)
+		return FALSE;
+#if defined(MSG_NOSIGNAL)
+	n = send(client_fd, buf, strlen(buf), MSG_NOSIGNAL);
+#else
+	n = send(client_fd, buf, strlen(buf), 0);
+#endif
+	if (n < 0 && errno == EPIPE)
+		{
+		if (_GK.debug_level & DEBUG_CLIENT)
+			printf("Write on closed pipe to gkrellmd server.\n");
+		server_alive = FALSE;
+		return FALSE;
+		}
+	return TRUE;
+	}
+
+gboolean
+gkrellm_client_send_to_server(gchar *key_name, gchar *line)
+	{
+	gchar		*str;
+	gboolean	result;
+
+	if (!key_name || !line || !*line)
+		return FALSE;
+	str = g_strdup_printf("<%s>\n", key_name);
+	client_send_to_server(str);
+	g_free(str);
+	if (line[strlen(line) - 1] != '\n')
+		{
+		str = g_strdup_printf("%s\n", line);
+		result = client_send_to_server(str);
+		g_free(str);
+		}
+	else
+		result = client_send_to_server(line);
+
+	return result;
+	}
 
 KeyTable	update_table[] =
 	{
@@ -1432,6 +1551,7 @@ KeyTable	update_table[] =
 	{"<disk>",			client_disk_line_from_server},
 	{"<net>",			client_net_line_from_server},
 	{"<net_routed>",	client_net_routed_line_from_server},
+	{"<net_timer>",		client_net_timer_line_from_server},
 	{"<mem>",			client_mem_line_from_server},
 	{"<swap>",			client_swap_line_from_server},
 	{"<fs>",			client_fs_line_from_server},
@@ -1570,6 +1690,7 @@ gkrellm_client_mode_disconnect(void)
 	if (client_fd >= 0)
 		close(client_fd);
 	client_fd = -1;
+	server_alive = FALSE;
 	gdk_input_remove(client_input_id);
 	client_input_id = 0;
 	}
@@ -1605,25 +1726,21 @@ read_server_input(gpointer data, gint fd, GdkInputCondition condition)
 		buf_index = strlen(line);
 		memmove(server_buf, line, buf_index);
 		}
-	else
+	else if (!*line)
 		buf_index = 0;
+	else
+		{
+		buf_index += count;
+		if (buf_index >= sizeof(server_buf) - 2)
+			buf_index = 0;
+		}
 
 	server_buf[buf_index] = '\0';
 	}
 
-static gint
-unable_to_connect(void)
-	{
-	static gboolean	warned;
 
-	if (!warned)
-		printf("%s\n", _("Unable to connect."));
-	warned = TRUE;
-	return -1;
-	}
-
-static gint
-connect_to_server(void)
+gint
+gkrellm_connect_to(gchar *server, gint server_port)
 	{
 	gint		fd	= -1;
 #ifdef HAVE_GETADDRINFO
@@ -1636,13 +1753,13 @@ connect_to_server(void)
 #endif
 
 #ifdef HAVE_GETADDRINFO
-	snprintf (portnumber, sizeof(portnumber), "%d", _GK.server_port);
+	snprintf (portnumber, sizeof(portnumber), "%d", server_port);
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = PF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_protocol = 0;
-	if ((rv = getaddrinfo(_GK.server, portnumber, &hints, &res0)) != 0)
-		return unable_to_connect();
+	if ((rv = getaddrinfo(server, portnumber, &hints, &res0)) != 0)
+		return -1;
 
 	for (res = res0; res; res = res->ai_next)
 		{
@@ -1650,39 +1767,46 @@ connect_to_server(void)
 				res->ai_protocol)) < 0)
 			continue;
 		if (_GK.debug_level & DEBUG_CLIENT)
-			printf("\t[connect_to_server: (%d,%d,%d) %s:%d]\n",
-					res->ai_family, res->ai_socktype, res->ai_protocol,
-					_GK.server, _GK.server_port);
+			printf("\t[gkrellm_connect_to: (%d,%d,%d) %s:%d]\n",
+			       res->ai_family, res->ai_socktype,
+			       res->ai_protocol, server, server_port);
 		if (connect(fd, res->ai_addr, res->ai_addrlen) >= 0)
 			break;
+#ifdef WIN32
+		closesocket(fd);
+#else
 		close(fd);
+#endif
 		fd = -1;
 		}
 	freeaddrinfo(res0);
-	if (fd == -1)
-		return unable_to_connect();
 #else
 	if (_GK.debug_level & DEBUG_CLIENT)
-		printf("\t[connect_to_server: %s:%d]\n", _GK.server, _GK.server_port);
-	addr = gethostbyname(_GK.server);
+		printf("\t[gkrellm_connect_to: %s:%d]\n", server, server_port);
+	addr = gethostbyname(server);
 	if (addr)
 		{
-		memcpy(&s.sin_addr, addr->h_addr, sizeof(struct in_addr));  
 		fd = socket(AF_INET, SOCK_STREAM, 0);
 		if (fd >= 0)
 			{
+			memset(&s, 0, sizeof(s));
+			memcpy(&s.sin_addr.s_addr, addr->h_addr, addr->h_length);
 			s.sin_family = AF_INET;
-			s.sin_port = htons(_GK.server_port);
+			s.sin_port = htons(server_port);
 			if (connect(fd, (struct sockaddr *)&s, sizeof (s)) < 0)
 				{
+#ifdef WIN32
+				closesocket(fd);
+#else
 				close(fd);
+#endif
 				fd = -1;
 				}
 			}
 		}
-	if (fd < 0) 
-		return unable_to_connect();
 #endif
+	if (fd < 0)
+		return -1;
 
 	return fd;
 	}
@@ -1695,8 +1819,11 @@ gkrellm_client_mode_connect(void)
 	if (_GK.server_port == 0)
 		_GK.server_port = GKRELLMD_SERVER_PORT;
 
-	if ((client_fd = connect_to_server()) < 0)
+	client_fd = gkrellm_connect_to(_GK.server, _GK.server_port);
+	if (client_fd < 0) {
+		printf("%s\n", _("Unable to connect."));
 		return FALSE;
+	}
 
 	snprintf(buf, sizeof(buf), "gkrellm %d.%d.%d%s\n",
 			GKRELLM_VERSION_MAJOR, GKRELLM_VERSION_MINOR,
@@ -1717,6 +1844,8 @@ gkrellm_client_mode_connect(void)
 	client_input_id = gdk_input_add(client_fd, GDK_INPUT_READ,
 					(GdkInputFunction) read_server_input, NULL);
 
+	server_alive = TRUE;
+
 	return TRUE;
 	}
 
