@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -23,6 +23,14 @@
 #include "gkrellm-private.h"
 #include "gkrellm-sysdeps.h"
 
+
+static void	cb_alert_config(GkrellmAlert *ap, gpointer data);
+#if !defined(WIN32)
+static void	cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox,
+					gpointer data);
+#endif
+
+
 #ifdef CLK_TCK
 #define	CPU_TICKS_PER_SECOND	CLK_TCK
 #else
@@ -862,8 +870,7 @@ create_cpu(GtkWidget *vbox, gint first_create)
 		gkrellm_chartconfig_height_connect(cp->config, cb_height, cpu);
 		setup_cpu_scaling(cp->config);
 
-		if (! (!strcmp(cpu->name, "cpu") && cpu->is_composite))
-			gkrellm_sensors_create_decals(p, style_id,
+		gkrellm_sensors_create_decals(p, style_id,
 					&cpu->sensor_decal, &cpu->fan_decal);
 
 		gkrellm_panel_configure(p, cpu->panel_label, style);
@@ -949,11 +956,6 @@ create_alert(void)
 	{
 	GList		*list;
 	CpuMon		*cpu;
-	static void	cb_alert_config(GkrellmAlert *ap, gpointer data);
-#if !defined(WIN32)
-	static void	cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox,
-						gpointer data);
-#endif
 
 	list = g_list_nth(cpu_mon_list, 0);
 	if (!list)
@@ -1131,8 +1133,6 @@ gkrellm_cpu_set_sensor(gpointer sr, gint type, gint n)
 	{
 	CpuMon	*cpu;
 
-	if (n_smp_cpus > 0)
-		++n;
 	if (   (cpu = (CpuMon *) g_list_nth_data(cpu_mon_list, n)) == NULL
 		|| !cpu->enabled
 	   )
@@ -1205,20 +1205,40 @@ cb_enable(GtkWidget *button, gpointer data)
 static void
 cb_smp_mode(GtkWidget *button, gpointer data)
 	{
-	GList	*list;
-	CpuMon	*cpu;
-	gint	i = GPOINTER_TO_INT(data);
+	GList		*list;
+	CpuMon		*cpu;
+	gint		i = GPOINTER_TO_INT(data);
+	gboolean	prev_enabled, rebuild_temps = FALSE, rebuild_fans = FALSE;
 
 	if (GTK_TOGGLE_BUTTON(button)->active)
 		smp_mode = i;
 	for (list = cpu_mon_list; list; list = list->next)
 		{
 		cpu = (CpuMon *) list->data;
+		prev_enabled = cpu->enabled;
 		if (enable_cpu_visibility(cpu) && cpu->enabled)
 			gkrellm_reset_and_draw_chart(cpu->chart);
+		if (prev_enabled && !cpu->enabled)
+			{
+			if (cpu->sensor_temp)
+				{
+				gkrellm_sensor_reset_location(cpu->sensor_temp);
+				rebuild_temps |= TRUE;
+				}
+			if (cpu->sensor_fan)
+				{
+				gkrellm_sensor_reset_location(cpu->sensor_fan);
+				rebuild_fans |= TRUE;
+				}
+			cpu->sensor_temp = NULL;
+			cpu->sensor_fan = NULL;
+			fix_panel(cpu);
+			}
 		gkrellm_apply_launcher(&cpu->launch_entry, &cpu->tooltip_entry,
 					cpu->chart->panel, &cpu->launch, gkrellm_launch_button_cb);
 		}
+	if (rebuild_temps || rebuild_fans)
+		gkrellm_sensors_rebuild(rebuild_temps, rebuild_fans, FALSE);
 	}
 
 static void
@@ -1387,8 +1407,6 @@ gkrellm_cpu_get_sensor_panel_label(gint n)
 	CpuMon	*cpu;
 	gchar	*s = "cpu?";
 
-	if (n_smp_cpus > 0)
-		++n;
 	cpu = (CpuMon *) g_list_nth_data(cpu_mon_list, n);
 	if (cpu)
 		s = cpu->panel_label;
