@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -206,13 +206,17 @@ add_subdisk(gchar *subdisk_name, gchar *disk_name, gint subdisk)
 	}
 
 static void
-disk_assign_data(DiskMon *disk, guint64 rb, guint64 wb)
+disk_assign_data(DiskMon *disk, guint64 rb, guint64 wb, gboolean virtual)
 	{
 	if (!disk)
 		return;
 	disk->rb = rb;
 	disk->wb = wb;
-	if (disk->subdisk == -1)	/* not a subdisk (partition) */
+
+	/* Add data to composite disk if this is not a subdisk (partition) and
+	|  not a virtual disk (eg /dev/mdX software RAID multi-disk).
+	*/
+	if (disk->subdisk == -1 && !virtual)
 		{
 		composite_disk->rb += rb;
 		composite_disk->wb += wb;
@@ -261,7 +265,7 @@ gkrellm_disk_units_are_blocks(void)
 	
 void
 gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
-			guint64 rb, guint64 wb)
+			guint64 rb, guint64 wb, gboolean virtual)
 	{
 	DiskMon	*disk;
 	gchar	*name;
@@ -275,11 +279,11 @@ gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
 		if (name)
 			disk = add_disk(name, name, device_number, unit_number, order);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
-gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb)
+gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb, gboolean virtual)
 	{
 	DiskMon	*disk;
 	gchar	name[32], label[32];
@@ -293,11 +297,12 @@ gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb)
 		sprintf(label, "%s%c", _("Disk"), 'A' + n);
 		disk = add_disk(name, label, 0, 0, n);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
-gkrellm_disk_assign_data_by_name(gchar *name, guint64 rb, guint64 wb)
+gkrellm_disk_assign_data_by_name(gchar *name, guint64 rb, guint64 wb,
+			gboolean virtual)
 	{
 	DiskMon	*disk;
 	gint	order = -1;
@@ -312,7 +317,7 @@ gkrellm_disk_assign_data_by_name(gchar *name, guint64 rb, guint64 wb)
 			order = (*order_from_name)(name);
 		disk = add_disk(name, name, 0, 0, order);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
@@ -338,7 +343,7 @@ gkrellm_disk_subdisk_assign_data_by_name(gchar *subdisk_name, gchar *disk_name,
 			return;
 		disk = add_subdisk(subdisk_name, disk_name, subdisk);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, FALSE);
 	}
 
 
@@ -490,9 +495,9 @@ gkrellm_disk_temperature_display(gpointer sr, gchar *id_name, gfloat t,
 			|| (disk->name[len] != '\0' && !isdigit(disk->name[len]))
 		   )
 			continue;
-		if (!disk->enabled || !disk->chart)
+		if (!disk->enabled || !disk->chart || !disk->chart->panel->decal_list)
 			{
-			display_possible |= disk->enabled;	/* Not created yet */
+			display_possible |= disk->enabled;	/* Not created or rebuilding */
 			continue;
 			}
 		decal = disk->temp_decal;
@@ -710,6 +715,7 @@ create_disk_monitor(DiskMon *disk, gint first_create)
 	{
 	GkrellmChart		*cp;
 	GkrellmPanel		*p;
+	GkrellmDecal		*d;
 	GkrellmStyle		*style;
 	GkrellmTextstyle	*ts;
 	GkrellmMargin		*m;
@@ -746,11 +752,8 @@ create_disk_monitor(DiskMon *disk, gint first_create)
 	setup_disk_scaling(cp->config, cp);
 
 	ts = gkrellm_panel_alt_textstyle(style_id);
-	m = gkrellm_get_style_margins(style);
 	disk->temp_decal = gkrellm_create_decal_text(p, "188.8F",
 					ts, style, -1, -1, 0);
-	disk->temp_decal->x =
-				gkrellm_chart_width() - m->right - disk->temp_decal->w;
 	gkrellm_make_decal_invisible(p, disk->temp_decal);
 
 	gkrellm_panel_configure(p, disk->label, style);
@@ -758,6 +761,15 @@ create_disk_monitor(DiskMon *disk, gint first_create)
 	disk->enabled = TRUE;
 	disk->new_text_format = TRUE;
 
+	/* Position the temp decal to right edge and vertically align it
+	|  wrt the label.
+	*/
+	d = disk->temp_decal;
+	m = gkrellm_get_style_margins(style);
+	gkrellm_move_decal(p, d,
+				gkrellm_chart_width() - m->right - d->w,
+ 				m->top + (p->label->height - d->h + 1) / 2);
+
 	if (first_create)
 		{
 		g_signal_connect(G_OBJECT(cp->drawing_area), "expose_event",
