@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -32,6 +32,7 @@
 #define DEBUG_GUI		0x2000
 #define	DEBUG_POSITION	0x4000
 #define	DEBUG_BATTERY	0x8000
+#define	DEBUG_CHART_TEXT 0x10000
 
 #define	GKRELLM_FLOAT_FACTOR	1000.0
 
@@ -128,6 +129,7 @@ struct	GkrellmConfig
 	gint		chart_height_min;
 	gint		chart_height_max;
 	gint		allow_scaling;
+	gboolean	chart_text_no_fill;
 	gboolean	config_modified;
 	gboolean	any_transparency;
 
@@ -157,6 +159,8 @@ struct	GkrellmConfig
 
 	gfloat		float_factor;			/* avoid config locale breakage */
 
+	gint		mbmon_port;
+
 	gint		rx_led_x;				/* Move these to net monitor */
 	gint		rx_led_y;
 	gint		tx_led_x;
@@ -380,12 +384,13 @@ gchar	*gkrellm_get_large_font_string(void);
 gchar	*gkrellm_get_normal_font_string(void);
 gchar	*gkrellm_get_small_font_string(void);
 
-gint	gkrellm_label_position(gint, gint, gint, gint);
+gint	gkrellm_label_x_position(gint, gint, gint, gint);
 
 void	gkrellm_inet_load_data(void);
 void	gkrellm_inet_save_data(void);
 
 void	gkrellm_net_save_data(void);
+void	gkrellm_net_server_has_timer(void);
 
 gint	gkrellm_effect_string_value(gchar *);
 
@@ -435,6 +440,11 @@ gint		gkrellm_sensor_read_voltage(gpointer sr, gfloat *);
 GkrellmAlert *gkrellm_sensor_alert(gpointer sr);
 void		gkrellm_sensor_alert_connect(gpointer sr,
 						void (*fn)(), gpointer data);
+void		gkrellm_sensor_reset_location(gpointer sr);
+void		gkrellm_sensors_rebuild(gboolean do_temp, gboolean do_fan,
+						gboolean do_volt);
+void		gkrellm_sensors_model_update(void);
+void		gkrellm_sensors_interface_remove(gint interface);
 
 /* pixops */
 void		gkrellm_border_adjust(GkrellmBorder *border,
@@ -458,9 +468,12 @@ void	gkrellm_winop_state_above(gboolean);
 void	gkrellm_winop_state_below(gboolean);
 
 /* client */
+gint		gkrellm_connect_to(gchar *, gint);
 gboolean	gkrellm_client_mode_connect(void);
 void		gkrellm_client_mode_disconnect(void);
 struct tm	*gkrellm_client_server_time(void);
 gint		gkrellm_client_server_connect_state(void);
 void		gkrellm_client_mode_connect_thread(void);
+gint		gkrellm_client_server_get_net_timer(void);
+
 
