@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -149,6 +149,8 @@ void		gkrellm_draw_panel_layers(GkrellmPanel *);
 void		gkrellm_draw_panel_layers_force(GkrellmPanel *);
 void		gkrellm_panel_bg_piximage_override(GkrellmPanel *,
 						GkrellmPiximage *);
+void		gkrellm_panel_label_get_position(GkrellmStyle *style,
+						gint *x_position, gint *y_off);
 
 
   /* Krell functions
@@ -181,6 +183,9 @@ void		gkrellm_insert_krell_nth(GkrellmPanel *, GkrellmKrell *, gint);
 GkrellmDecal *gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
 						GkrellmTextstyle *ts, GkrellmStyle *style,
 						gint x, gint y, gint w);
+GkrellmDecal *gkrellm_create_decal_text_markup(GkrellmPanel *p, gchar *string,
+						GkrellmTextstyle *ts, GkrellmStyle *style,
+						gint x, gint y, gint w);
 GkrellmDecal *gkrellm_create_decal_text_with_height(GkrellmPanel *p,
 						GkrellmTextstyle *ts, GkrellmStyle *style,
 						gint x, gint y, gint w, gint h, gint y_pad);
@@ -194,11 +199,17 @@ void		gkrellm_draw_decal_pixmap(GkrellmPanel *, GkrellmDecal *, gint);
 void		gkrellm_draw_decal_text(GkrellmPanel *, GkrellmDecal *, gchar *,
 						gint);
 void		gkrellm_draw_decal_markup(GkrellmPanel *, GkrellmDecal *, gchar *);
+void		gkrellm_decal_text_get_offset(GkrellmDecal *d, gint *x, gint *y);
+void		gkrellm_decal_text_set_offset(GkrellmDecal *d, gint x, gint y);
 void		gkrellm_decal_text_clear(GkrellmDecal *d);
 void		gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s,
 						GkrellmTextstyle *ts, gint x_off, gint y_off);
-void		gkrellm_decal_text_set_offset(GkrellmDecal *d, gint x, gint y);
-void		gkrellm_decal_text_get_offset(GkrellmDecal *d, gint *x, gint *y);
+void		gkrellm_decal_text_markup_insert(GkrellmDecal *d, gchar *s,
+						GkrellmTextstyle *ts, gint x_off, gint y_off);
+void		gkrellm_decal_text_nth_inserted_set_offset(GkrellmDecal *d,
+						gint n, gint x_off, gint y_off);
+void		gkrellm_decal_text_nth_inserted_get_offset(GkrellmDecal *d,
+						gint n, gint *x_off, gint *y_off);
 void		gkrellm_decal_scroll_text_align_center(GkrellmDecal *d,
 						gboolean center);
 void		gkrellm_decal_scroll_text_horizontal_loop(GkrellmDecal *d,
@@ -330,6 +341,7 @@ gfloat		gkrellm_get_theme_scale(void);
 void		gkrellm_open_config_window(GkrellmMonitor *);
 gboolean	gkrellm_config_window_shown(void);
 void		gkrellm_config_modified(void);
+void		gkrellm_config_instant_apply(GkrellmMonitor *mon);
 GkrellmMargin *gkrellm_get_style_margins(GkrellmStyle *);
 void		gkrellm_set_style_margins(GkrellmStyle *, GkrellmMargin *);
 void		gkrellm_get_top_bottom_margins(GkrellmStyle *, gint *, gint *);
@@ -502,6 +514,11 @@ GtkWidget	*gkrellm_gtk_scrolled_text_view(GtkWidget *, GtkWidget **,
 void		gkrellm_gtk_text_view_append_strings(GtkWidget *, gchar **, gint);
 void		gkrellm_gtk_text_view_append(GtkWidget *, gchar *);
 
+GtkTreeSelection *gkrellm_gtk_scrolled_selection(GtkTreeView *treeview,
+			GtkWidget *box, GtkSelectionMode s_mode,
+			GtkPolicyType h_policy, GtkPolicyType v_policy,
+			void (*func_cb)(), gpointer data);
+
 
   /* Some utility functions
   */
@@ -559,17 +576,33 @@ void		gkrellm_client_plugin_serve_data_connect(GkrellmMonitor *mon,
 					gchar *key_name, void (*func_cb)(gchar *line));
 void		gkrellm_client_plugin_reconnect_connect(gchar *key_name,
 					void (*func_cb)());
-
+gboolean	gkrellm_client_send_to_server(gchar *key_name, gchar *line);
+gboolean	gkrellm_client_check_server_version(gint major,
+					gint minor, gint rev);
 
 /* ------- Some aids for using PangoFontDescription -------*/
-void	gkrellm_text_extents(PangoFontDescription *font_desc, gchar *text,
-					gint len, gint *width, gint *height, gint *baseline,
-					gint *y_ink);
+void	gkrellm_text_extents(PangoFontDescription *font_desc,
+					gchar *text, gint len, gint *width, gint *height,
+					gint *baseline, gint *y_ink);
+void	gkrellm_text_markup_extents(PangoFontDescription *font_desc,
+					gchar *text, gint len, gint *width, gint *height,
+					gint *baseline, gint *y_ink);
 
 gint	gkrellm_gdk_string_width(PangoFontDescription *, gchar *);
+gint	gkrellm_gdk_string_markup_width(PangoFontDescription *, gchar *);
+gint	gkrellm_gdk_text_width(PangoFontDescription *font_desc,
+					const gchar *string, gint len);
+gint	gkrellm_gdk_text_markup_width(PangoFontDescription *font_desc,
+					const gchar *string, gint len);
 void	gkrellm_gdk_draw_string(GdkDrawable *drawable,
 					PangoFontDescription *font,
 					GdkGC *gc, gint x, gint y, gchar *string);
+void	gkrellm_gdk_draw_string_markup(GdkDrawable *drawable,
+					PangoFontDescription *font,
+					GdkGC *gc, gint x, gint y, gchar *string);
 void	gkrellm_gdk_draw_text(GdkDrawable *drawable,
 					PangoFontDescription *font,
 					GdkGC *gc, gint x, gint y, gchar *string, gint len);
+void	gkrellm_gdk_draw_text_markup(GdkDrawable *drawable,
+					PangoFontDescription *font,
+					GdkGC *gc, gint x, gint y, gchar *string, gint len);
