@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -141,10 +141,11 @@ gint	gkrellm_sys_disk_order_from_name(gchar *name);
 void	gkrellm_disk_client_divert(void (*read_func)(),
 			gchar *(*name_from_device_func)(), gint (*order_from_name_func)());
 void	gkrellm_disk_assign_data_by_device(gint device_number,
-				gint unit_number, guint64 r, guint64 w);
-void	gkrellm_disk_assign_data_nth(gint n, guint64 r, guint64 w);
+				gint unit_number, guint64 r, guint64 w, gboolean virtual);
+void	gkrellm_disk_assign_data_nth(gint n, guint64 r, guint64 w,
+				gboolean virtual);
 void	gkrellm_disk_assign_data_by_name(gchar *name,
-				guint64 r, guint64 w);
+				guint64 r, guint64 w, gboolean virtual);
 void	gkrellm_disk_subdisk_assign_data_by_name(gchar *subdisk_name,
 				gchar *disk_name, guint64 r, guint64 w);
 
@@ -195,7 +196,8 @@ gchar	*gkrellm_net_mon_first(void);
 gchar	*gkrellm_net_mon_next(void);
 void	gkrellm_net_routed_event(gchar *name, gboolean routed);
 
-  /* Routines optionally called from gkrellm_sys_net_init().
+  /* Routines optionally called from gkrellm_sys_net_init()
+  *  or client_sys_net_init().
   */
 void	gkrellm_net_use_routed(gboolean);
 void	gkrellm_net_add_timer_type_ppp(gchar *);
@@ -236,7 +238,7 @@ void	gkrellm_fs_client_divert(void (*get_fsusage_func)(),
 				void (*get_mounts_func)(), void (*get_fstab_func)(),
 				gboolean (*fstab_modified_func)());
 void	gkrellm_fs_assign_fsusage_data(gpointer fs,
-				gulong blocks, gulong avail, gulong free, gulong bsize);
+				glong blocks, glong avail, glong free, glong bsize);
 void	gkrellm_fs_add_to_fstab_list(gchar *dir, gchar *dev, gchar *type,
 				gchar *options);
 void	gkrellm_fs_add_to_mounts_list(gchar *dir, gchar *dev, gchar *type);
@@ -284,7 +286,25 @@ gboolean gkrellm_sys_sensors_get_voltage(gchar *name, gint id,
 
 void	gkrellm_sensors_client_divert(gboolean (*get_temp_func)(),
             gboolean (*get_fan_func)(), gboolean (*get_volt_func)());
-void	gkrellm_sensors_add_sensor(gint type, gchar *sensor_path,
+gpointer gkrellm_sensors_add_sensor(gint type, gchar *sensor_path,
 			gchar *id_name,
         	gint id, gint iodev, gint inter,
         	gfloat factor, gfloat offset, gchar *vref, gchar *default_label);
+void	gkrellm_sensors_set_group(gpointer sr, gint group);
+
+  /* Common sensor reading from daemons.  Interfaces from 0 - 20 are available
+  |  for system dependent interfaces.  Interfaces from 20 up are used here
+  |  for the common daemon interfaces.
+  */
+#define HDDTEMP_INTERFACE	20
+#define MBMON_INTERFACE		21
+
+  /* Common hddtemp interface*/
+void	gkrellm_sys_sensors_hddtemp_check(void);
+gboolean gkrellm_sys_sensors_hddtemp_get_value(gchar *name, gfloat *value);
+
+  /* Common mbmon interface*/
+gboolean	gkrellm_sys_sensors_mbmon_supported(void);
+gboolean	gkrellm_sys_sensors_mbmon_check(gboolean force);
+gboolean	gkrellm_sys_sensors_mbmon_get_value(gchar *name, gfloat *value);
+gboolean	gkrellm_sys_sensors_mbmon_port_change(gint port);
