@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -89,7 +89,7 @@
 */
 #define	GKRELLM_VERSION_MAJOR	2
 #define	GKRELLM_VERSION_MINOR	2
-#define	GKRELLM_VERSION_REV		4
+#define	GKRELLM_VERSION_REV		7
 #define	GKRELLM_EXTRAVERSION	""
 
 #define GKRELLM_CHECK_VERSION(major,minor,rev)    \
@@ -1021,7 +1021,8 @@ typedef struct
 					from_command_line,
 					spacers_shown,
 					cap_images_off,
-					spacer_overlap_off;
+					spacer_overlap_off,
+					instant_apply;
 	GkrellmStyle	*panel_style,
 					*chart_style;
 	gchar			*style_name;
