@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -248,6 +248,31 @@ gkrellm_gtk_scrolled_text_view(GtkWidget *box, GtkWidget **scr,
 	return view;
 	}
 
+GtkTreeSelection *
+gkrellm_gtk_scrolled_selection(GtkTreeView *treeview, GtkWidget *box,
+			GtkSelectionMode s_mode,
+			GtkPolicyType h_policy, GtkPolicyType v_policy,
+			void (*func_cb)(), gpointer data)
+	{
+	GtkTreeSelection	*selection;
+	GtkWidget			*scrolled;
+
+	if (!box || !treeview)
+		return NULL;
+
+	scrolled = gtk_scrolled_window_new(NULL, NULL);
+	gtk_box_pack_start(GTK_BOX(box), scrolled, TRUE, TRUE, 0);
+	gtk_container_add(GTK_CONTAINER(scrolled), GTK_WIDGET(treeview));
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+				h_policy, v_policy);
+	selection = gtk_tree_view_get_selection(treeview);
+	gtk_tree_selection_set_mode(selection, s_mode);
+	if (func_cb)
+		g_signal_connect(G_OBJECT(selection), "changed",
+					G_CALLBACK(func_cb), data);
+	return selection;
+	}
+
 void
 gkrellm_launch_button_cb(GkrellmDecalbutton *button)
 	{
@@ -473,6 +498,7 @@ gkrellm_gtk_check_button_connected(GtkWidget *box, GtkWidget **button,
 	if (!string)
 		return;
 	b = gtk_check_button_new_with_label(string);
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), active);
 	if (box)
 		{
@@ -682,6 +708,10 @@ gkrellm_gtk_category_vbox(GtkWidget *box, gchar *category_header,
 	vbox1 = gtk_vbox_new(FALSE, box_pad);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, FALSE, 0);
 
+	/* Add some bottom pad */
+	label = gtk_label_new("");
+	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+
 	return vbox1;
 	}
 
@@ -721,7 +751,7 @@ create_about_tab(GtkWidget *vbox)
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 
 	buf = g_strdup_printf(_("GKrellM %d.%d.%d%s\nGNU Krell Monitors\n\n"
-				"Copyright (c) 1999-2004 by Bill Wilson\n"
+				"Copyright (c) 1999-2005 by Bill Wilson\n"
 				"bill@gkrellm.net\n"
 				"http://gkrellm.net\n\n"
 				"Released under the GNU General Public License"),
@@ -1975,10 +2005,8 @@ close_config(gpointer data)
 	gkrellm_plugins_config_close();
 	close_theme_config(from_close);
 	if (_GK.config_modified)
-		{
-		gkrellm_save_theme_config();
 		gkrellm_save_user_config();
-		}
+	gkrellm_save_theme_config();
 	}
 
 
@@ -2159,7 +2187,8 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 			real_create_config(gkrellm_get_cpu_mon());
 			real_create_config(gkrellm_get_proc_mon());
 			}
-		instant = (selected_monitor->apply_config == NULL);
+		instant = (   selected_monitor->apply_config == NULL
+				   || selected_monitor->privat->instant_apply);
 		}
 	else
 		instant = TRUE;
@@ -2172,6 +2201,15 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 					page, selected_monitor ? selected_monitor->name : "--");
 	}
 
+  /* Monitors may want to present as instant apply monitors, but still need
+  |  their apply function called when changing notebook pages or on OK button.
+  */
+void
+gkrellm_config_instant_apply(GkrellmMonitor *mon)
+	{
+	mon->privat->instant_apply = TRUE;
+	}
+
 gboolean
 gkrellm_config_window_shown(void)
 	{
