@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -54,6 +54,7 @@ typedef struct
 						xoff,
 						w;			/* Pixel width of string using ts */
 	gboolean			reduced;	/* TRUE if default string/font won't fit */
+	gboolean			no_fit;
 	}
 	DecalText;
 
@@ -65,6 +66,8 @@ static DecalText	host_dtext,
 static gchar		*hostname,
 					*system_name;
 
+static void create_hostname(GtkWidget *vbox, gint first_create);
+
 gboolean
 gkrellm_hostname_can_shorten(void)
 	{
@@ -141,6 +144,7 @@ setup_sysname(void)
 	g_free(system_dtext.string);
 	system_dtext.string = g_strdup(system_name);
 	system_dtext.reduced = FALSE;
+	system_dtext.no_fit = FALSE;
 	pick_textstyle(decal_sysname, &system_dtext);
 	if (system_dtext.w > decal_sysname->w)
 		{
@@ -155,6 +159,8 @@ setup_sysname(void)
 				{
 				prune(&system_dtext.string, system_name, TRUE, FALSE);
 				pick_textstyle(decal_sysname, &system_dtext);
+				if (system_dtext.w > decal_sysname->w)
+					system_dtext.no_fit = TRUE;
 				}
 			}
 		}
@@ -194,45 +200,67 @@ draw_hostname(void)
 static void
 draw_sysname(void)
 	{
-	DecalText	*dtext0, *dtext1 = NULL;
+	GkrellmTextstyle	ts_save;
+	gchar				buf[128];
+	gint				x, y, w;
 
 	if (!decal_sysname)
 		return;
 
+	ts_save = decal_sysname->text_style;
 	if (system_name_mode == 0 || !system_dtext.reduced)
-		dtext0 = &system_dtext;
+		{
+		decal_sysname->text_style = *system_dtext.ts;
+		snprintf(buf, sizeof(buf), "%s", system_dtext.string);
+		y = system_dtext.yoff;
+		}
 	else
 		{
-		dtext0 = &kname_dtext;
-		dtext1 = &kversion_dtext;
+		decal_sysname->text_style = *kversion_dtext.ts;
+		if (system_dtext.no_fit)
+			snprintf(buf, sizeof(buf), "%s\n<small><small>%s</small></small>",
+					kname_dtext.string, kversion_dtext.string);
+		else
+			snprintf(buf, sizeof(buf), "%s\n%s",
+					kname_dtext.string, kversion_dtext.string);
+
+		y = kname_dtext.yoff;
 		}
-			
-	gkrellm_decal_text_clear(decal_sysname);
-	gkrellm_decal_text_insert(decal_sysname, dtext0->string, dtext0->ts,
-				dtext0->xoff, dtext0->yoff);
-	if (dtext1)
-		gkrellm_decal_text_insert(decal_sysname, dtext1->string, dtext1->ts,
-					dtext1->xoff, dtext1->yoff);
+	if (system_dtext.no_fit)
+		gkrellm_decal_scroll_text_set_markup(host, decal_sysname, buf);
+	else
+		gkrellm_decal_scroll_text_set_text(host, decal_sysname, buf);
+
+	gkrellm_decal_scroll_text_get_size(decal_sysname, &w, NULL);
+	x = (decal_sysname->w - w) / 2;
+	gkrellm_decal_text_set_offset(decal_sysname, x, y);
+
+	decal_sysname->text_style = ts_save;
 	}
 
+
 void
 update_host(void)
 	{
-	gint		connect_state, delta, step;
-	gint		hz = gkrellm_update_HZ();
-	static gint	reconnect_timeout, y_target, asym;
+	gchar			**parts;
+	gint			connect_state, delta, step, h_scroll;
+	gint			hz = gkrellm_update_HZ();
+	static gint		reconnect_timeout, y_target, asym;
+	static gboolean	recheck_sysname;
 
 	if (decal_sysname && system_name_mode == 1 && system_dtext.reduced)
 		{
-		if (GK.two_second_tick && y_target != 0)
+		if (GK.two_second_tick && y_target == 0)
 			{
-			y_target = 0;
+			gkrellm_decal_scroll_text_get_size(decal_sysname, NULL, &h_scroll);
+			y_target = decal_sysname->h - h_scroll;
 			asym = 3;		/* 2 secs name, 6 secs version */
 			}
-		else if (GK.two_second_tick && y_target == 0  && --asym <= 0)
-			y_target = decal_sysname->h;
-
-		delta = y_target - kversion_dtext.yoff;
+		else if (GK.two_second_tick && y_target != 0  && --asym <= 0)
+			{
+			y_target = 0;
+			}
+		delta = y_target - kname_dtext.yoff;
 		if (delta != 0)
 			{
 			step = (delta < 0) ? -1 : 1;
@@ -244,7 +272,6 @@ update_host(void)
 					step *= 2;
 				}
 			kname_dtext.yoff += step;
-			kversion_dtext.yoff += step;
 			draw_sysname();
 			gkrellm_draw_panel_layers(host);
 			}
@@ -259,7 +286,10 @@ update_host(void)
 	if (connect_state == 2)		/* thread is trying a reconnect */
 		{
 		if (_GK.client_server_reconnect_timeout <= 0)
+			{
 			gkrellm_check_alert(server_alert, 1.0);		/* Warning	*/
+			draw_sysname();
+			}
 		}
 	else if (connect_state == 0)	/* Lost connection			*/
 		{
@@ -271,6 +301,8 @@ update_host(void)
 			gkrellm_client_mode_connect_thread();
 			reconnect_timeout = 0;
 			}
+		draw_sysname();
+		recheck_sysname = TRUE;		/* Server may be down for kernel upgrade */
 		}
 	else
 		{
@@ -281,9 +313,29 @@ update_host(void)
 					_GK.client_server_read_time + _GK.client_server_io_timeout
 		   )
 			gkrellm_client_mode_disconnect();
+		if (recheck_sysname)
+			{
+			if (strcmp(system_name, _GK.server_sysname))
+				{
+				g_free(system_name);
+				g_free(kname_dtext.string);
+				g_free(kversion_dtext.string);
+				system_name = g_strdup(_GK.server_sysname);
+				parts = g_strsplit(system_name, " ", 2);
+				kname_dtext.string = parts[0];
+				kversion_dtext.string = parts[1];
+				gkrellm_panel_destroy(host);
+				gkrellm_alert_destroy(&server_alert);
+				create_hostname(host_vbox, TRUE);
+				}
+			else
+				draw_sysname();				
+			}
+		recheck_sysname = FALSE;
 		}
 	}
 
+
 static void
 cb_command_process(GkrellmAlert *alert, gchar *src, gchar *buf, gint size,
 			gpointer data)
@@ -358,8 +410,8 @@ cb_sysname_mode(GkrellmDecalbutton *button, gpointer data)
 static void
 create_hostname(GtkWidget *vbox, gint first_create)
 	{
-	GkrellmStyle			*style;
-	gint			yy, y, l, r, w, a, d;
+	GkrellmStyle	*style;
+	gint			y, l;
 
 	if (first_create)
 		host = gkrellm_panel_new0();
@@ -371,7 +423,7 @@ create_hostname(GtkWidget *vbox, gint first_create)
 	style = gkrellm_meter_style(style_id);
 
 	decal_host = decal_sysname = NULL;
-	y = yy = 0;
+	y = 0;
 	if (_GK.enable_hostname)
 		{
 		decal_host = gkrellm_create_decal_text(host, "8Hgk",
@@ -383,14 +435,11 @@ create_hostname(GtkWidget *vbox, gint first_create)
 		decal_sysname = gkrellm_create_decal_text(host, system_name,
 					gkrellm_meter_textstyle(style_id), style, -1, -1, -1);
 		decal_sysname->y += y;
-		gkrellm_winop_text_extents(decal_sysname->text_style.font, system_name,
-					strlen(system_name), &l, &r, &w, &a, &d);
-		yy = (d > 0) ? 0 : 1;
+		gkrellm_decal_scroll_text_align_center(decal_sysname, TRUE);
 		}
 
 	gkrellm_panel_configure(host, NULL, style);
-	if (yy > 0)
-		gkrellm_panel_configure_add_height(host, yy);
+	gkrellm_panel_configure_add_height(host, 1);
 
 	gkrellm_panel_create(vbox, mon_host, host);
 
@@ -406,12 +455,13 @@ create_hostname(GtkWidget *vbox, gint first_create)
 	draw_sysname();
 	if (decal_host || decal_sysname)
 		gkrellm_draw_panel_layers(host);
+
 	if (decal_sysname && system_dtext.reduced)
 		{
 		l = decal_sysname->h * 4 / 5;
 		sysname_mode_button = gkrellm_make_scaled_button(host, NULL,
 					cb_sysname_mode, NULL, TRUE, TRUE,
-					0, 0, 0,	/* Use builtin images */
+					0, 0, 0,    /* Use builtin images */
 					gkrellm_chart_width() - l,
 					decal_sysname->y + decal_sysname->h - l,
 					l, l);
@@ -457,7 +507,7 @@ gkrellm_apply_hostname_config(void)
 	}
 
 void
-cb_gkrellmd_disconnect_alert(GtkWidget *button, gpointer data)
+gkrellm_gkrellmd_disconnect_cb(GtkWidget *button, gpointer data)
 	{
 	gkrellm_alert_config_window(&server_alert);
 	}
@@ -513,7 +563,7 @@ static GkrellmMonitor	monitor_host =
 	};
 
 GkrellmMonitor *
-init_host_monitor(void)
+gkrellm_init_host_monitor(void)
 	{
 	gchar	**parts;
 
@@ -522,12 +572,12 @@ init_host_monitor(void)
 	if (_GK.client_mode)
 		{
 		hostname = _GK.server_hostname;
-		system_name = _GK.server_sysname;
+		system_name = g_strdup(_GK.server_sysname);
 		}
 	else
 		{
 		hostname = gkrellm_sys_get_host_name();
-		system_name = gkrellm_sys_get_system_name();
+		system_name = g_strdup(gkrellm_sys_get_system_name());
 		}
 	parts = g_strsplit(system_name, " ", 2);
 	kname_dtext.string = parts[0];
