@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -692,7 +692,7 @@ get_connection_string_thread(void *data)
 	if (in->connection_string->len == 0)
 		g_string_append(in->connection_string, _("No current connections."));
 	in->connection_string_event = TRUE;
-	free_glist_and_data(&in->tcp_save_list);
+	gkrellm_free_glist_and_data(&in->tcp_save_list);
 	return NULL;
 	}
 
@@ -1001,14 +1001,14 @@ create_inet(GtkWidget *vbox, gint first_create)
 	n_inet_monitors = 0;
 	if (!first_create && last_chart_width != gkrellm_chart_width())
 		{  /* Will be allocating new data arrays */
-		save_inet_data();
+		gkrellm_inet_save_data();
 		new_data = TRUE;
 		last_chart_width = gkrellm_chart_width();
 		}
 	for (i = 0, list = inet_mon_list; list; ++i, list = list->next)
 		create_inet_monitor(inet_vbox, (InetMon *)list->data, first_create);
 	if (first_create || new_data)
-		load_inet_data();
+		gkrellm_inet_load_data();
 	if (inet_mon_list)
 		gkrellm_spacers_show(mon_inet);
 	else
@@ -1285,7 +1285,7 @@ make_inet_data_fname(InetMon *in)
 	}
 
 void
-save_inet_data(void)
+gkrellm_inet_save_data(void)
 	{
 	FILE		*f;
 	struct tm	*tm;
@@ -1320,7 +1320,7 @@ save_inet_data(void)
 	}
 
 void
-load_inet_data(void)
+gkrellm_inet_load_data(void)
 	{
 	FILE		*f;
 	GList		*list;
@@ -1579,7 +1579,7 @@ sync_inet_list(void)
 	|  a inet down or up slots, etc.  Data is lost only if a port number
 	|  for a monitor is changed.
 	*/
-	save_inet_data();
+	gkrellm_inet_save_data();
 	new_inet_list = NULL;
 	n_inet_monitors = 0;
 
@@ -1644,7 +1644,7 @@ sync_inet_list(void)
 	for (list = inet_mon_list; list; list = list->next)
 		create_inet_monitor(inet_vbox, (InetMon *)list->data, TRUE);
 
-	load_inet_data();
+	gkrellm_inet_load_data();
 
 	for (list = inet_mon_list; list; list = list->next)
 		{
@@ -2001,6 +2001,7 @@ create_inet_tab(GtkWidget *tab_vbox)
 	list = g_list_append(list,
 				"\\r\\f $M\\D1$a\\f $l\\D2$A\\f $L");
 	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
 	g_list_free(list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
 			text_format);
@@ -2055,7 +2056,7 @@ static GkrellmMonitor	monitor_inet =
 	};
 
 GkrellmMonitor *
-init_inet_monitor(void)
+gkrellm_init_inet_monitor(void)
 	{
 	monitor_inet.name = _(monitor_inet.name);
 	style_id = gkrellm_add_chart_style(&monitor_inet, INET_STYLE_NAME);
