@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -292,6 +292,13 @@ decal_text_list_free(GList **text_list)
 	}
 
 
+  /* Draw to first GkrellmText in text_list and truncate text_list to make
+  |  gkrellm_draw_decal_text() compatible with plugin code written before
+  |  the text_list implementation.  For backwards compatibility, d->x_off
+  |  shadows the first tx->x_off.
+  |  Combining gkrellm_draw_decal_text() and gkrellm_decal_text_insert()
+  |  functions on the same decal may lead to unexpected results.
+  */
 static void
 gkrellm_draw_decal_internal(GkrellmPanel *p, GkrellmDecal *d, gchar *s,
 			gint value)
@@ -324,6 +331,16 @@ void
 gkrellm_draw_decal_text(GkrellmPanel *p, GkrellmDecal *d, gchar *s, gint value)
 	{
 	d->flags &= ~DF_TEXT_USE_MARKUP;
+
+	/* In case mixing scroll_text and draw_decal_text calls
+	*/
+	d->flags &= ~DF_SCROLL_TEXT_DIVERTED;
+	if (d->scroll_text)
+		{
+		g_free(d->scroll_text);
+		d->scroll_text = NULL;
+		}
+
 	gkrellm_draw_decal_internal(p, d, s, value);
 	}
 
@@ -331,9 +348,55 @@ void
 gkrellm_draw_decal_markup(GkrellmPanel *p, GkrellmDecal *d, gchar *s)
 	{
 	d->flags |= DF_TEXT_USE_MARKUP;
+
+	/* In case mixing scroll_text and draw_decal_text calls
+	*/
+	d->flags &= ~DF_SCROLL_TEXT_DIVERTED;
+	if (d->scroll_text)
+		{
+		g_free(d->scroll_text);
+		d->scroll_text = NULL;
+		}
+
 	gkrellm_draw_decal_internal(p, d, s, 0);
 	}
 
+void
+gkrellm_decal_text_get_offset(GkrellmDecal *d, gint *x, gint *y)
+	{
+	if (!d)
+		return;
+	if (x)
+		*x = d->x_off;
+	if (y)
+		*y = d->y_off;
+	}
+
+  /* Setting offset of text decal means setting offset of first GkrellmText
+  |  in text_list.  For backwards compatibility with plugin code that
+  |  directly accesses d->x_off, shadow d->x_off to first tx->x_off.
+  */
+void
+gkrellm_decal_text_set_offset(GkrellmDecal *d, gint x, gint y)
+	{
+	GList		*list;
+	GkrellmText	*tx;
+
+	if (!d || (d->x_off == x && d->y_off == y))
+		return;
+
+	d->x_off = x;
+	d->y_off = y;
+	if ((list = d->text_list) != NULL)
+		{
+		tx = (GkrellmText *) list->data;
+		tx->x_off = x;
+		tx->x_off_prev = x;
+		tx->y_off = y - d->y_ink;;
+		}
+	d->modified = TRUE;
+	}
+
 void
 gkrellm_decal_text_clear(GkrellmDecal *d)
 	{
@@ -357,6 +420,10 @@ gkrellm_decal_text_clear(GkrellmDecal *d)
 	d->modified = TRUE;
 	}
 
+  /* Insert text into a decal's text_list at a given offset.  Using text
+  |  insert functions makes d->x_off and d->y_off meaningless since offsets
+  |  are in each GkrellmText's tx->x_off and tx->y_off.
+  */
 void
 gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts,
 		gint x_off, gint y_off)
@@ -365,6 +432,7 @@ gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts,
 
 	if (!s || !d || d->state == DS_INVISIBLE)
 		return;
+	d->flags &= ~DF_TEXT_USE_MARKUP;
 	if (!ts)
 		ts = &d->text_style;
 	if (   d->text_list
@@ -388,35 +456,71 @@ gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts,
 	}
 
 void
-gkrellm_decal_text_get_offset(GkrellmDecal *d, gint *x, gint *y)
+gkrellm_decal_text_markup_insert(GkrellmDecal *d, gchar *s,
+		GkrellmTextstyle *ts, gint x_off, gint y_off)
 	{
-	if (!d)
+	GkrellmText	*tx = NULL;
+
+	if (!s || !d || d->state == DS_INVISIBLE)
 		return;
-	if (x)
-		*x = d->x_off;
-	if (y)
-		*y = d->y_off;
+	d->flags |= DF_TEXT_USE_MARKUP;
+	if (!ts)
+		ts = &d->text_style;
+	if (   d->text_list
+		&& !*((GkrellmText *) d->text_list->data)->text
+	   )
+		tx = (GkrellmText *) d->text_list->data;
+
+	if (!tx)
+		{
+		tx = g_new0(GkrellmText, 1);
+		d->text_list = g_list_append(d->text_list, tx);
+		}
+
+	gkrellm_dup_string(&tx->text, s);
+	tx->x_off = x_off;
+	tx->x_off_prev = x_off;
+	tx->y_off = y_off - d->y_ink;
+	tx->text_style = *ts;
+
+	d->modified = TRUE;
 	}
 
 void
-gkrellm_decal_text_set_offset(GkrellmDecal *d, gint x, gint y)
+gkrellm_decal_text_nth_inserted_set_offset(GkrellmDecal *d, gint n,
+			gint x, gint y)
 	{
 	GList		*list;
 	GkrellmText	*tx;
 
-	if (!d || (d->x_off == x && d->y_off == y))
+	if (!d || (list = g_list_nth(d->text_list, n)) == NULL)
 		return;
 
-	d->x_off = x;
-	d->y_off = y;
-	for (list = d->text_list; list; list = list->next)
+	tx = (GkrellmText *) list->data;
+	if (tx->x_off != x || tx->y_off != y)
 		{
-		tx = (GkrellmText *) list->data;
 		tx->x_off = x;
 		tx->x_off_prev = x;
 		tx->y_off = y - d->y_ink;;
+		d->modified = TRUE;
 		}
-	d->modified = TRUE;
+	}
+
+void
+gkrellm_decal_text_nth_inserted_get_offset(GkrellmDecal *d, gint n,
+			gint *x, gint *y)
+	{
+	GList		*list;
+	GkrellmText	*tx;
+
+	if (!d || (list = g_list_nth(d->text_list, n)) == NULL)
+		return;
+
+	tx = (GkrellmText *) list->data;
+	if (x)
+		*x = tx->x_off;
+	if (y)
+		*y = tx->y_off + d->y_ink;
 	}
 
 void
@@ -524,6 +628,24 @@ decal_scroll_text_layout(GkrellmDecal *d, gchar *text, gint *yink)
 	return layout;
 	}
 
+  /* If mixing draw_decal_text and scroll_text calls, must reset decal regions
+  |  in bg_text_layer_pixmap to bg_pixmap at first switch to scroll_text calls.
+  */
+static void
+reset_text_layer_pixmap_decal_region(GkrellmPanel *p, GkrellmDecal *d)
+	{
+	if (d->flags & DF_MOVED)
+		{
+		gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
+					p->bg_pixmap,
+					d->x_old, d->y_old, d->x_old, d->y_old, d->w, d->h);
+		d->flags &= ~DF_MOVED;
+		}
+	else
+		gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
+					p->bg_pixmap,
+					d->x, d->y,  d->x, d->y,   d->w, d->h);
+	}
 
   /* Draw Pango rendered text onto a pixmap which can be scrolled across
   |  a decal's extents.  So when scrolling, a slow Pango redraw at each scroll
@@ -552,11 +674,13 @@ gkrellm_decal_scroll_text_set_internal(GkrellmPanel *p, GkrellmDecal *d,
 	GkrellmBorder		*b;
 	GkrellmTextstyle	*ts;
 	gint				dx, y_ink;
-	gboolean			new_text, no_scroll_caching;
+	gboolean			new_text, no_scroll_caching, first_scroll;
 
 	if (!p || !text || !d || d->state == DS_INVISIBLE)
 		return;
 
+	first_scroll = (d->scroll_text == NULL);
+
 	new_text = gkrellm_dup_string(&d->scroll_text, text);
 	no_scroll_caching = (p->transparency || p->scroll_text_cache_off);
 
@@ -579,30 +703,22 @@ gkrellm_decal_scroll_text_set_internal(GkrellmPanel *p, GkrellmDecal *d,
 			   )
 		   )
 			{
+			if (first_scroll && d->text_list)
+				reset_text_layer_pixmap_decal_region(p, d);
+
 			d->flags |= (DF_TEXT_OVERLAPS | DF_SCROLL_TEXT_DIVERTED);
 			if (new_text)
 				{
 				layout = decal_scroll_text_layout(d, text, NULL);
 				g_object_unref(G_OBJECT(layout));
 				}
-			gkrellm_draw_decal_text(p, d, text, -1);
+			gkrellm_draw_decal_internal(p, d, text, -1);
 			return;
 			}
 		}
 	if (d->text_list)	/* Erase any previous gkrellm_draw_decal_text()	*/
 		{
-		if (d->flags & DF_MOVED)
-			{
-			gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
-						p->bg_pixmap,
-						d->x_old, d->y_old, d->x_old, d->y_old, d->w, d->h);
-			d->flags &= ~DF_MOVED;
-			}
-		else
-			gdk_draw_drawable(p->bg_text_layer_pixmap, _GK.draw1_GC,
-						p->bg_pixmap,
-						d->x, d->y,  d->x, d->y,   d->w, d->h);
-
+		reset_text_layer_pixmap_decal_region(p, d);
 		decal_text_list_free(&d->text_list);
 		d->value = -1;
 		}
@@ -643,8 +759,11 @@ gkrellm_decal_scroll_text_set_internal(GkrellmPanel *p, GkrellmDecal *d,
 		}
 
 	if (pixbuf)
+		{
 		gkrellm_paste_pixbuf(pixbuf, d->pixmap, 0, 0,
 					d->scroll_width, d->scroll_height);
+		g_object_unref(G_OBJECT(pixbuf));
+		}
 	else
 		{
 		b = &p->bg_piximage->border;
@@ -1154,7 +1273,7 @@ gkrellm_create_decal_pixmap(GkrellmPanel *p,
 static GkrellmDecal *
 _create_decal_text(GkrellmPanel *p, gchar *string,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
-				gint x, gint y, gint w, gint h, gint y_ink)
+				gint x, gint y, gint w, gint h, gint y_ink, gboolean markup)
 	{
 	GtkWidget		*top_win;
 	GkrellmMargin	*m;
@@ -1179,8 +1298,14 @@ _create_decal_text(GkrellmPanel *p, gchar *string,
 
 	if (string && *string)
 		{
-		gkrellm_text_extents(d->text_style.font, string, strlen(string),
-				&width, &height, &baseline, &d->y_ink);
+		if (markup)
+			gkrellm_text_markup_extents(d->text_style.font,
+					string, strlen(string),
+					&width, &height, &baseline, &d->y_ink);
+		else
+			gkrellm_text_extents(d->text_style.font,
+					string, strlen(string),
+					&width, &height, &baseline, &d->y_ink);
 		d->h = height + d->text_style.effect;
 		}
 	else
@@ -1240,7 +1365,15 @@ gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
 				gint x, gint y, gint w)
 	{
-	return _create_decal_text(p, string, ts, style, x, y, w, 0, 0);
+	return _create_decal_text(p, string, ts, style, x, y, w, 0, 0, FALSE);
+	}
+
+GkrellmDecal *
+gkrellm_create_decal_text_markup(GkrellmPanel *p, gchar *string,
+				GkrellmTextstyle *ts, GkrellmStyle *style,
+				gint x, gint y, gint w)
+	{
+	return _create_decal_text(p, string, ts, style, x, y, w, 0, 0, TRUE);
 	}
 
 GkrellmDecal *
@@ -1248,7 +1381,7 @@ gkrellm_create_decal_text_with_height(GkrellmPanel *p,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
 				gint x, gint y, gint w, gint h, gint y_ink)
 	{
-	return _create_decal_text(p, NULL, ts, style, x, y, w, h, y_ink);
+	return _create_decal_text(p, NULL, ts, style, x, y, w, h, y_ink, FALSE);
 	}
 
 void
