@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -23,6 +23,10 @@
 #include "gkrellm-private.h"
 #include "gkrellm-sysdeps.h"
 
+#if GTK_CHECK_VERSION(2,4,0)
+#include "icon.xpm"
+#endif
+
 #include <signal.h>
 
 
@@ -250,11 +254,13 @@ set_or_save_position(gint save)
 	path = gkrellm_make_data_file_name(NULL, "startup_position");
 	if (save)
 		{
-		if (!configure_position_lock)
+		if (!configure_position_lock && y_pack < 0)
 			{	/* Most recent configure event may have happened before
 				|  gkrellm has removed it's lock, so get gdk's cached values.
 				|  Eg. gkrellm is moved < 2 sec after startup by the window
 				|  manager.  See cb_configure_notify().
+				|  But don't update _GK.y_position if current gkrellm position
+				|  reflects a packed move (ie, not a user set position).
 				*/
 			gdk_window_get_position(top_window->window,
 					&_GK.x_position, &_GK.y_position);
@@ -512,7 +518,8 @@ gkrellm_draw_text(GdkDrawable *drawable, GkrellmTextstyle *ts, gint x, gint y,
 	}
 
 gint
-gkrellm_label_position(gint position, gint w_field, gint w_label, gint margin)
+gkrellm_label_x_position(gint position, gint w_field, gint w_label,
+				gint margin)
 	{
 	gint	x;
 
@@ -1473,11 +1480,19 @@ gkrellm_spacers_hide(GkrellmMonitor *mon)
 static void
 create_widget_tree()
 	{
-	gchar	*title;
+	gchar		*title;
+#if GTK_CHECK_VERSION(2,4,0)
+	GdkPixbuf	*icon;
+#endif
 
 	gtree.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_widget_set_name(gtree.window, PACKAGE);
 
+#if GTK_CHECK_VERSION(2,4,0)
+	icon = gdk_pixbuf_new_from_xpm_data((const gchar **) icon_xpm);
+	gtk_window_set_default_icon(icon);
+#endif
+
 	title = gkrellm_make_config_file_name(NULL, PACKAGE);
 	gtk_window_set_title(GTK_WINDOW(gtree.window), title);
 //	gtk_window_set_wmclass(GTK_WINDOW(gtree.window), title, "Gkrellm");
