@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -738,7 +738,7 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 	GkrellmTextstyle *ts;
 	GkrellmStyle	*s;
 	gchar			*expand, buf[64], *u;
-	gint			w_label;
+	gint			w_label, label_x_position, label_y_off;
 
 	mm = &mem;
 	if (first_create)
@@ -790,13 +790,15 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 		gkrellm_destroy_piximage(im);
 
 	s = gkrellm_meter_style(mm->style_id);
+	gkrellm_panel_label_get_position(s, &label_x_position, &label_y_off);
+
 	mm->krell_used = gkrellm_create_krell(p,
 						gkrellm_krell_meter_piximage(mm->style_id), s);
 	gkrellm_monotonic_krell_values(mm->krell_used, FALSE);
 
 	if (mem.label)
 		g_free(mem.label);
-	if (s->label_position == GKRELLM_LABEL_NONE)
+	if (label_x_position == GKRELLM_LABEL_NONE)
 		mem.label = g_strdup("");
 	else
 		mem.label = g_strdup(_("Mem"));
@@ -809,13 +811,16 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 
 	snprintf(buf, sizeof(buf), "%sMemfj8", mem.label);
 	mm->decal_label = gkrellm_create_decal_text(p, buf,
-				gkrellm_meter_textstyle(mm->style_id), s, -1, -1, -1);
+				gkrellm_meter_textstyle(mm->style_id), s, -1,
+				(label_y_off > 0) ? label_y_off : -1,
+				-1);
 	gkrellm_panel_configure(p, NULL, s);
 	gkrellm_panel_create(vbox, mon_mem, p);
 
 	ts = &mm->decal_label->text_style;
 	w_label = gkrellm_gdk_string_width(ts->font, mm->label) + ts->effect;
-	mm->x_label = gkrellm_label_position(p->label->position, mm->decal_label->w,
+	mm->x_label = gkrellm_label_x_position(label_x_position,
+				mm->decal_label->w,
 				w_label, 0);
 	draw_decal_label(mm, 0);
 
@@ -843,7 +848,7 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 	GkrellmStyle	*style, *panel_style;
 	GkrellmTextstyle *ts;
 	gchar			buf[64], *u;
-	gint			w_label;
+	gint			w_label, label_x_position, label_y_off;
 
 	mm = &swap;
 	if (first_create)
@@ -851,6 +856,7 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 	p = mm->panel;
 
 	style = gkrellm_meter_style(mm->style_id);
+	gkrellm_panel_label_get_position(style, &label_x_position, &label_y_off);
 
 	/* Need a chart styled krell on the swap meter panel, but want it to track
 	|  the meter krell margins.
@@ -868,7 +874,7 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 
 	if (swap.label)
 		g_free(swap.label);
-	if (style->label_position == GKRELLM_LABEL_NONE)
+	if (label_x_position == GKRELLM_LABEL_NONE)
 		swap.label = g_strdup("");
 	else
 		swap.label = g_strdup(_("Swap"));
@@ -881,14 +887,17 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 
 	snprintf(buf, sizeof(buf), "%sMemfj8", swap.label);
 	mm->decal_label = gkrellm_create_decal_text(p, buf,
-				gkrellm_meter_textstyle(mm->style_id), style, -1, -1, -1);
+				gkrellm_meter_textstyle(mm->style_id), style, -1,
+				(label_y_off > 0) ? label_y_off : -1,
+				-1);
 
 	gkrellm_panel_configure(p, NULL, style);
 	gkrellm_panel_create(vbox, mon_swap, p);
 
 	ts = &mm->decal_label->text_style;
 	w_label = gkrellm_gdk_string_width(ts->font, mm->label) + ts->effect;
-	mm->x_label = gkrellm_label_position(p->label->position, mm->decal_label->w,
+	mm->x_label = gkrellm_label_x_position(label_x_position,
+				mm->decal_label->w,
 				w_label, 0);
 	draw_decal_label(mm, 0);
 
