@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -26,9 +26,31 @@
 
 #define	PPP_LOCK_FILE	"LCK..modem"
 
-#define TIMER_TYPE_NONE	0
-#define TIMER_TYPE_PPP	1
-#define TIMER_TYPE_IPPP	2
+#define TIMER_TYPE_NONE		0
+#define TIMER_TYPE_PPP		1
+#define TIMER_TYPE_IPPP		2
+#define TIMER_TYPE_SERVER	3
+
+
+#define		N_DAY_STATS		31
+#define		N_WEEK_STATS	26
+#define		N_MONTH_STATS	12
+
+typedef struct
+	{
+	gchar		*date;
+	gdouble		rx,
+				tx;
+	gint		connect_time;
+	}
+	NetStat;
+
+enum StatType
+	{
+	DAY_STAT,
+	WEEK_STAT,
+	MONTH_STAT
+	};
 
 
 typedef struct
@@ -76,9 +98,11 @@ typedef struct
 						rx_totalB,
 						tx_totalA,
 						tx_totalB;
-	gdouble				rx_day, rx_week, rx_month;
-	gdouble				tx_day, tx_week, tx_month;
-	gint				time_day, time_week, time_month;
+
+	NetStat				day_stats[N_DAY_STATS],
+						week_stats[N_WEEK_STATS],
+						month_stats[N_MONTH_STATS];
+
 	gboolean			show_totalB,
 						totals_shown,
 						reset_button_in,
@@ -103,7 +127,7 @@ typedef struct
 static void cb_alert_config(GkrellmAlert *ap, NetMon *net);
 static void	cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox,
 					NetMon *net);
-
+static void	net_stat_init(NetMon *net);
 
 typedef struct
 	{
@@ -156,7 +180,7 @@ strcmp_net_name(NetMon *net1, NetMon *net2)
 static NetMon *
 new_net(gchar *name)
 	{
-	NetMon	*net;
+	NetMon		*net;
 
 	net = g_new0(NetMon, 1);
 	net->name = g_strdup(name);
@@ -170,6 +194,8 @@ new_net(gchar *name)
 	net_mon_list = g_list_insert_sorted(net_mon_list, net,
 				(GCompareFunc) strcmp_net_name);
 
+	net_stat_init(net);
+
 	return net;
 	}
 
@@ -383,7 +409,6 @@ gkrellm_net_led_positions(gint *x_rx_led, gint *y_rx_led,
 #define	TX_ON	3
 #define	N_LEDS	4
 
-
 static GkrellmMonitor	*mon_net;
 static GkrellmMonitor	*mon_timer;
 
@@ -428,14 +453,15 @@ static gint			timer_button_enabled;
 static gint			last_time = -1;
 
 
-static gint			timer_button_type,
+static gint			timer_button_type = TIMER_TYPE_NONE,
 					timer_button_state,
 					timer_button_old_state,
 					last_timer_command;
 
 static gboolean		timer_seconds;
 
-static gint			check_connect_state;
+static gint			check_connect_state,
+					net_stats_window_height;
 
 static gint			ascent,
 					ascent_alt,
@@ -462,7 +488,22 @@ static GkrellmSizeAbbrev    stats_bytes_abbrev[]    =
 	{ TB_SIZE(1000),    TB_SIZE(1),     "%.2fTB" }
 	};
 
+#define	NET_DATA_VERSION	2
 
+#define	SATURDAY	6
+#define	SUNDAY		0
+
+static gchar	days_in_month[12] =
+		{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31  };
+
+
+  /* Called from client.c
+  */
+void
+gkrellm_net_server_has_timer(void)
+	{
+	timer_button_type = TIMER_TYPE_SERVER;
+	}
 
 static void
 draw_led(NetMon *net, int rxtx, int led_index)
@@ -527,111 +568,175 @@ draw_timer(GkrellmPanel *p, gint seconds, gint force)
 	gkrellm_draw_panel_layers(timer_panel);
 	}
 
-static void
-add_net_stat(GtkWidget *table, gdouble v, gint c0, gint c1, gint r0, gint r1)
+  /* --------- net stats window -----------
+  */
+enum
 	{
-	GtkWidget	*label;
-	gchar		buf[64];
+	DATE_COLUMN,
+	RX_COLUMN,
+	TX_COLUMN,
+	TOTAL_COLUMN,
+	TIME_COLUMN,
+	N_STATS_COLUMNS
+	};
 
-	gkrellm_format_size_abbrev(buf, sizeof(buf), (gfloat) v,
-				&stats_bytes_abbrev[0],
-				sizeof(stats_bytes_abbrev) / sizeof(GkrellmSizeAbbrev));
-	label = gtk_label_new(buf);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, c0, c1, r0, r1);
-	}
-
-static void
-add_net_time(GtkWidget *table, time_t t, gint c0, gint c1, gint r0, gint r1)
+static GtkTreeModel *
+stats_model_create(NetMon *net, NetStat *ns, gint n_stats)
 	{
-	GtkWidget	*label;
-	gchar		buf[64];
-	gint		hours, minutes;
+	GtkListStore	*store;
+	GtkTreeIter		iter;
+	gchar			rx[64], tx[64], total[64], time[64];
+	gint			i, hours, minutes;
 
-	minutes = t / 60;
-	hours = minutes / 60;
-	minutes %= 60;
+	store = gtk_list_store_new(N_STATS_COLUMNS,
+				G_TYPE_STRING,
+				G_TYPE_STRING,
+				G_TYPE_STRING,
+				G_TYPE_STRING,
+				G_TYPE_STRING);
 
-	snprintf(buf, sizeof(buf), "%4d:%02d", hours, minutes);
-	label = gtk_label_new(buf);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, c0, c1, r0, r1);
-	}
-
-static void
-net_stats_dialog(NetMon *net)
-	{
-	GtkWidget	*top_win;
-	GtkWidget	*dialog;
-	GtkWidget	*table;
-	GtkWidget	*vbox, *label, *sep;
-	gchar		buf[64];
-
-	top_win = gkrellm_get_top_window();
-	snprintf(buf, sizeof(buf), "%s", net->name);
-	dialog = gtk_dialog_new_with_buttons(buf,
-				GTK_WINDOW(top_win),
-				GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_STOCK_OK, GTK_RESPONSE_NONE,
-				NULL);
-	gtk_window_set_wmclass(GTK_WINDOW(dialog),
-				"Gkrellm_dialog", "Gkrellm");
-	g_signal_connect_swapped(GTK_OBJECT(dialog), "response",
-				G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(dialog));
-
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox,
-				FALSE, FALSE, 0);
-
-	table = gtk_table_new(5, 7, FALSE /*homogeneous*/);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 12);
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
-
-	label = gtk_label_new(_("Receive"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 0, 1);
-	label = gtk_label_new(_("Transmit"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 3, 4, 0, 1);
-	label = gtk_label_new(_("Total"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 4, 5, 0, 1);
-	if (net == net_timed)
+	for (i = 0; i < n_stats; ++i, ++ns)
 		{
-		label = gtk_label_new(_("Connect Time"));
-		gtk_table_attach_defaults(GTK_TABLE(table), label, 6, 7, 0, 1);
-		sep = gtk_vseparator_new();
-		gtk_table_attach_defaults(GTK_TABLE(table), sep, 5, 6, 0, 5);
+		if (i > 0 && !strncmp(ns->date, "---", 3))
+			continue;
+		gkrellm_format_size_abbrev(rx, sizeof(rx), (gfloat) ns->rx,
+					&stats_bytes_abbrev[0],
+					sizeof(stats_bytes_abbrev) / sizeof(GkrellmSizeAbbrev));
+		gkrellm_format_size_abbrev(tx, sizeof(tx), (gfloat) ns->tx,
+					&stats_bytes_abbrev[0],
+					sizeof(stats_bytes_abbrev) / sizeof(GkrellmSizeAbbrev));
+		gkrellm_format_size_abbrev(total, sizeof(total),
+					(gfloat) (ns->rx + ns->tx),
+					&stats_bytes_abbrev[0],
+					sizeof(stats_bytes_abbrev) / sizeof(GkrellmSizeAbbrev));
+		minutes = ns->connect_time / 60;
+		hours = minutes / 60;
+		minutes %= 60;
+		snprintf(time, sizeof(time), "%4d:%02d", hours, minutes);
+
+		gtk_list_store_append(store, &iter);
+		gtk_list_store_set(store, &iter,
+				DATE_COLUMN, ns->date,
+				RX_COLUMN, rx,
+				TX_COLUMN, tx,
+				TOTAL_COLUMN, total,
+				TIME_COLUMN, time,
+				-1);
 		}
+	return GTK_TREE_MODEL(store);
+	}
 
-	label = gtk_label_new(_("Today"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
-	label = gtk_label_new(_("Week"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
-	label = gtk_label_new(_("Month"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 4, 5);
+static gint
+net_stats_window_configure_cb(GtkWidget *widget, GdkEventConfigure *ev,
+			gpointer data)
+	{
+	net_stats_window_height = widget->allocation.height;
+	gkrellm_config_modified();
+	return FALSE;
+	}
+
+static void
+net_stats_close_cb(GtkWidget *widget, GtkWidget *stats_window)
+	{
+	gtk_widget_destroy(stats_window);
+	}
+
+static void
+net_stats_page(GtkWidget *vbox, NetMon *net,
+			NetStat *ns, gint n_stats, gchar *period)
+	{
+	GtkTreeView			*treeview;
+	GtkTreeModel		*model;
+	GtkTreeSelection	*selection;
+	GtkCellRenderer		*renderer;
+	gint				i;
+
+	model = stats_model_create(net, ns, n_stats);
+	treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
+	g_object_unref(model);
+	gtk_tree_view_set_rules_hint(treeview, TRUE);
+
+	renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_insert_column_with_attributes(treeview, -1, period,
+			renderer, "text", DATE_COLUMN, NULL);
+
+	renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_insert_column_with_attributes(treeview, -1, _("Received"),
+			renderer, "text", RX_COLUMN, NULL);
+
+	renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_insert_column_with_attributes(treeview, -1, _("Transmitted"),
+			renderer, "text", TX_COLUMN, NULL);
+
+	renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_insert_column_with_attributes(treeview, -1, _("Total"),
+			renderer, "text", TOTAL_COLUMN, NULL);
+
+	for (i = 0; i < n_stats; ++i)
+		if (ns[i].connect_time > 0)
+			{
+			renderer = gtk_cell_renderer_text_new();
+			gtk_tree_view_insert_column_with_attributes(treeview, -1,
+					_("Connect Time"),
+					renderer, "text", TIME_COLUMN, NULL);
+			break;
+			}
+	selection = gkrellm_gtk_scrolled_selection(treeview, vbox,
+			GTK_SELECTION_NONE,
+			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC,
+			NULL, NULL);
+	}
+
+static void
+net_stats_window_show(NetMon *net)
+	{
+	GtkWidget			*stats_window, *tabs, *main_vbox;
+	GtkWidget			*vbox, *hbox, *button, *sep;
+	gchar				buf[64];
+
+	snprintf(buf, sizeof(buf), _("%s Statistics"), net->name);
+
+	stats_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_title(GTK_WINDOW(stats_window), buf);
+	gtk_window_set_wmclass(GTK_WINDOW(stats_window),
+				"Gkrellm_netstats", "Gkrellm");
+
+	g_signal_connect(G_OBJECT(stats_window), "configure_event",
+				G_CALLBACK(net_stats_window_configure_cb), NULL);
+	gtk_window_set_default_size(GTK_WINDOW(stats_window),
+				-1, net_stats_window_height);
+
+	main_vbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 4);
+	gtk_container_add(GTK_CONTAINER(stats_window), main_vbox);
+
+	tabs = gtk_notebook_new();
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
+	gtk_box_pack_start(GTK_BOX(main_vbox), tabs, TRUE, TRUE, 0);
+
+	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Daily"));
+	net_stats_page(vbox, net, &net->day_stats[0], N_DAY_STATS, _("Date"));
+
+	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Weekly"));
+	net_stats_page(vbox, net, &net->week_stats[0], N_WEEK_STATS,
+				_("Week Ending"));
+
+	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Monthly"));
+	net_stats_page(vbox, net, &net->month_stats[0], N_MONTH_STATS,
+				_("Month"));
 
-	sep = gtk_vseparator_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), sep, 1, 2, 0, 5);
 	sep = gtk_hseparator_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), sep,
-			0, net == net_timed ? 7 : 6, 1, 2);
+	gtk_box_pack_start(GTK_BOX(main_vbox), sep, FALSE, FALSE, 3);
 
-	add_net_stat(table, net->rx_day,   2, 3, 2, 3);
-	add_net_stat(table, net->rx_week,  2, 3, 3, 4);
-	add_net_stat(table, net->rx_month, 2, 3, 4, 5);
+	hbox = gtk_hbutton_box_new();
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, FALSE, 4);
+	button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+	g_signal_connect(G_OBJECT(button), "clicked",
+			G_CALLBACK(net_stats_close_cb), stats_window);
+	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 
-	add_net_stat(table, net->tx_day,   3, 4, 2, 3);
-	add_net_stat(table, net->tx_week,  3, 4, 3, 4);
-	add_net_stat(table, net->tx_month, 3, 4, 4, 5);
-
-	add_net_stat(table, net->rx_day   + net->tx_day,  4, 5, 2, 3);
-	add_net_stat(table, net->rx_week  + net->tx_week, 4, 5, 3, 4);
-	add_net_stat(table, net->rx_month + net->tx_month, 4, 5, 4, 5);
-
-	if (net == net_timed)
-		{
-		add_net_time(table, net->time_day, 6, 7, 2, 3);
-		add_net_time(table, net->time_week, 6, 7, 3, 4);
-		add_net_time(table, net->time_month, 6, 7, 4, 5);
-		}
-	gtk_widget_show_all(dialog);
+	gtk_widget_show_all(stats_window);
 	}
 
 
@@ -646,6 +751,7 @@ get_connect_state(void)
 	switch (timer_button_type)
 		{
 		case TIMER_TYPE_NONE:
+		case TIMER_TYPE_SERVER:
 			break;
 		case TIMER_TYPE_PPP:
 			if (net_timed->up)
@@ -690,6 +796,7 @@ get_connect_time(void)
 	switch (timer_button_type)
 		{
 		case TIMER_TYPE_NONE:
+		case TIMER_TYPE_SERVER:
 			break;
 		case TIMER_TYPE_PPP:
 			snprintf(buf, sizeof(buf), "/var/run/%s.pid", timer_button_iface);
@@ -767,12 +874,18 @@ update_timer_button_monitor(void)
 			if (timer_button_state != TB_NORMAL)
 				draw_timer(timer_panel, (int) (time(0) - net_timer0), 0);
 			break;
+
+		case TIMER_TYPE_SERVER:
+			/* Button state is independent of displayed timer
+			*/
+			draw_timer(timer_panel, gkrellm_client_server_get_net_timer(), 0);
+			break;
 		}
 	if (net_timed && net_timed->up)
 		{
-		net_timed->time_day += 1;
-		net_timed->time_week += 1;
-		net_timed->time_month += 1;
+		net_timed->day_stats[0].connect_time += 1;
+		net_timed->week_stats[0].connect_time += 1;
+		net_timed->month_stats[0].connect_time += 1;
 		}
 	}
 
@@ -822,6 +935,7 @@ cb_timer_button_release(GtkWidget *widget, GdkEventButton *ev)
 	switch (timer_button_type)
 		{
 		case TIMER_TYPE_NONE:
+		case TIMER_TYPE_SERVER:
 			if (timer_button_state == TB_NORMAL)
 				{
 				if (*timer_on_command != '\0')
@@ -904,12 +1018,18 @@ timed_net_visibility(void)
 	if ((net->force_up || net_is_up) && !chart_is_visible)
 		gkrellm_chart_show(net->chart, TRUE);	/* Make sure panel is shown */
 	else if (!net->force_up && !net_is_up && chart_is_visible)
+		{
 		gkrellm_chart_hide(net->chart, FALSE);  /* Don't hide the panel */
 
-	if (!net_is_up && chart_is_visible)
+		/* Save data whenever net has transitioned to a "not up" state.
+		|  Don't have the info to do this when there's a force_up and in
+		|  this case just rely on the every six hour tick data save.
+		*/
 		gkrellm_net_save_data();
+		}
 	}
 
+
 static gint
 net_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	{
@@ -1047,7 +1167,7 @@ static void
 format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 	{
 	GkrellmChart	*cp;
-	gchar			c, *s, *s1;
+	gchar			c, *s, *s1, *result;
 	gint			len, bytes;
 	gdouble			fbytes;
 	gboolean		month, day, week;
@@ -1056,10 +1176,16 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 		return;
 	--size;
 	*buf = '\0';
+	result = buf;
+
 	if (!src_string)
 		return;
 	cp = net->chart;
 	net->totals_shown = FALSE;
+
+	if ((_GK.debug_level & DEBUG_CHART_TEXT))
+		printf("net chart text: %s\n", src_string);
+
 	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
@@ -1086,11 +1212,11 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 			else if (c == 'o')
 				{
 				if (month)
-					fbytes = net->tx_month;
+					fbytes = net->month_stats[0].tx;
 				else if (week)
-					fbytes = net->tx_week;
+					fbytes = net->week_stats[0].tx;
 				else if (day)
-					fbytes = net->tx_day;
+					fbytes = net->day_stats[0].tx;
 				else
 					fbytes = net->show_totalB
 								? net->tx_totalB : net->tx_totalA;
@@ -1098,11 +1224,11 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 			else if (c == 'i')
 				{
 				if (month)
-					fbytes = net->rx_month;
+					fbytes = net->month_stats[0].rx;
 				else if (week)
-					fbytes = net->rx_week;
+					fbytes = net->week_stats[0].rx;
 				else if (day)
-					fbytes = net->rx_day;
+					fbytes = net->day_stats[0].rx;
 				else
 					fbytes = net->show_totalB
 								? net->rx_totalB : net->rx_totalA;
@@ -1110,11 +1236,11 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 			else if (c == 'O')
 				{
 				if (month)
-					fbytes = net->rx_month + net->tx_month;
+					fbytes = net->month_stats[0].rx + net->month_stats[0].tx;
 				else if (week)
-					fbytes = net->rx_week + net->tx_week;
+					fbytes = net->week_stats[0].rx + net->week_stats[0].tx;
 				else if (day)
-					fbytes = net->rx_day + net->tx_day;
+					fbytes = net->day_stats[0].rx + net->day_stats[0].tx;
 				else
 					fbytes = net->show_totalB
 							? (net->rx_totalB + net->tx_totalB)
@@ -1163,6 +1289,9 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 		buf += len;
 		}
 	*buf = '\0';	
+
+	if ((_GK.debug_level & DEBUG_CHART_TEXT))
+		printf("              : %s\n", result);
 	}
 
 static void
@@ -1282,7 +1411,7 @@ cb_chart_release(GtkWidget *widget, GdkEventButton *ev, NetMon *net)
 	if (!net->reset_button_in && !net->stats_button_in)
 		return FALSE;
 	if (net->stats_button_in && gkrellm_in_decal(decal_stats, ev))
-		net_stats_dialog(net);
+		net_stats_window_show(net);
 	if (net->reset_button_in && gkrellm_in_decal(decal_reset, ev))
 		{
 		if (net->show_totalB)
@@ -1455,11 +1584,40 @@ net_timer_visibility(void)
 	}
 
 
-#define	NET_DATA_VERSION	1
-
-static gchar	days_in_month[12] =
-		{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31  };
+static gint
+days_in_year(gint y)
+	{
+	if((y % 4) == 0)
+		return 366;
+	return 365;
+	}
 
+  /* Adjust a tm date structure to reference its next day.
+  */
+static void 
+next_day(struct tm *t)
+	{
+	if ((t->tm_wday += 1) > SATURDAY)
+		t->tm_wday = SUNDAY;
+	if ((t->tm_yday += 1) >= days_in_year(t->tm_year) )
+		{
+		t->tm_year += 1;
+		t->tm_yday = 0;
+		}
+	if ((t->tm_mday += 1) > days_in_month[t->tm_mon])
+		{
+		if (   t->tm_mon == 1
+		    && (((gint)t->tm_year - 80) % 4) == 0
+		    && t->tm_mday == 29)
+			;
+		else
+			{
+			if ((t->tm_mon += 1) > 11)
+				t->tm_mon = 0;
+			t->tm_mday = 1;
+			}
+		}
+	}
 
 static gboolean
 net_accounting_month_new(struct tm *tm)
@@ -1481,6 +1639,7 @@ gkrellm_net_save_data(void)
 	struct tm	*tm  = gkrellm_get_current_time();
 	NetMon		*net;
 	gchar		fname[256];
+	gint		i;
 
 	for (list = net_mon_list; list; list = list->next)
 		{
@@ -1496,22 +1655,132 @@ gkrellm_net_save_data(void)
 		fprintf(f, "%d %d %d %d %d\n", tm->tm_wday,
 					tm->tm_mday, tm->tm_mon, tm->tm_yday, tm->tm_year);
 
-		fputs("day\n", f);
-		fprintf(f, "%.0f %.0f\n", net->rx_day, net->tx_day);
-		fputs("week\n", f);
-		fprintf(f, "%.0f %.0f\n", net->rx_week, net->tx_week);
-		fputs("month\n", f);
-		fprintf(f, "%.0f %.0f\n", net->rx_month, net->tx_month);
-		if (net == net_timed)
-			{
-			fputs("connect time: day week month\n", f);
-			fprintf(f, "%d %d %d\n",
-						net->time_day, net->time_week, net->time_month);
-			}
+		fputs("[daily]\n", f);
+		for (i = 0; i < N_DAY_STATS; ++i)
+			fprintf(f, "\"%s\" %.0f %.0f %d\n", net->day_stats[i].date,
+					net->day_stats[i].rx, net->day_stats[i].tx,
+					net->day_stats[i].connect_time);
+		fputs("[weekly]\n", f);
+		for (i = 0; i < N_WEEK_STATS; ++i)
+			fprintf(f, "\"%s\" %.0f %.0f %d\n", net->week_stats[i].date,
+					net->week_stats[i].rx, net->week_stats[i].tx,
+					net->week_stats[i].connect_time);
+		fputs("[monthly]\n", f);
+			for (i = 0; i < N_MONTH_STATS; ++i)
+		fprintf(f, "\"%s\" %.0f %.0f %d\n", net->month_stats[i].date,
+					net->month_stats[i].rx, net->month_stats[i].tx,
+					net->month_stats[i].connect_time);
 		fclose(f);
 		}
 	}
 
+static gchar *
+utf8_string(gchar *string)
+	{
+	gchar	*utf8 = NULL;
+
+	if (   g_utf8_validate(string, -1, NULL)
+	    || (utf8 = g_locale_to_utf8(string, -1, NULL, NULL, NULL)) == NULL
+	   )
+		utf8 = g_strdup(string);
+
+	return utf8;
+	}
+
+static void
+net_stat_set_date_string(NetStat *ns, enum StatType stat_type, struct tm *t)
+	{
+	struct tm	tm  = *t,
+				tmx;
+	gchar		*utf8, buf[32], bufa[32], bufb[32];
+
+	if (stat_type == DAY_STAT)
+		{
+		strftime(buf, sizeof(buf), "%F", &tm);	/* %Y-%m-%d (the ISO 8601) */
+		}
+	else if (stat_type == WEEK_STAT)
+		{
+		while (tm.tm_wday != SATURDAY)
+			next_day(&tm);
+		strftime(buf, sizeof(buf), "%F", &tm);
+		}
+	else if (stat_type == MONTH_STAT)
+		{
+		if (reset_mday == 1)
+			strftime(buf, sizeof(buf), "%B", &tm);	/* full month name */
+		else
+			{
+			tmx = tm;
+			if (tm.tm_mday < reset_mday)
+				{
+				tmx.tm_mon -= 1;
+				if (tmx.tm_mon < 0)
+					{
+					tmx.tm_mon = 11;
+					tmx.tm_year -= 1;
+					}
+				}
+			else
+				{
+				tm.tm_mon += 1;
+				if (tm.tm_mon > 11)
+					{
+					tm.tm_mon = 0;
+					tm.tm_year += 1;
+					}
+				}
+			tmx.tm_mday = reset_mday;
+			tm.tm_mday = reset_mday - 1;
+			if (tmx.tm_mday > days_in_month[tmx.tm_mon])
+				tmx.tm_mday = days_in_month[tmx.tm_mon];
+			if (tm.tm_mday > days_in_month[tm.tm_mon])
+				tm.tm_mday = days_in_month[tm.tm_mon];
+			strftime(bufa, sizeof(bufa), "%b %e", &tmx);
+			strftime(bufb, sizeof(bufb), "%b %e", &tm);
+			snprintf(buf, sizeof(buf), "%s - %s", bufa, bufb);
+			}
+		}
+	else
+		return;
+
+	g_free(ns->date);
+	utf8 = utf8_string(buf);
+	ns->date = utf8 ? utf8 : g_strdup("??");
+	}
+
+static void
+net_stat_init(NetMon *net)
+	{
+	struct tm	*tm = gkrellm_get_current_time();
+	gint		i;
+
+	for (i = 0; i < N_DAY_STATS; ++i)
+		net->day_stats[i].date = g_strdup("---");
+	for (i = 0; i < N_WEEK_STATS; ++i)
+		net->week_stats[i].date = g_strdup("---");
+	for (i = 0; i < N_MONTH_STATS; ++i)
+		net->month_stats[i].date = g_strdup("---");
+
+	net_stat_set_date_string(&net->day_stats[0], DAY_STAT, tm);
+	net_stat_set_date_string(&net->week_stats[0], WEEK_STAT, tm);
+	net_stat_set_date_string(&net->month_stats[0], MONTH_STAT, tm);
+	}
+
+static void
+net_stats_shift_down(NetStat *ns, gint n_stats, enum StatType stat_type)
+	{
+	gint	d;
+
+	g_free(ns[n_stats - 1].date);
+
+	for (d = n_stats - 1; d > 0; --d)
+		ns[d] = ns[d - 1];
+	ns->rx = ns->tx = 0.0;
+	ns->connect_time = 0;
+	ns->date = NULL;
+	net_stat_set_date_string(ns, stat_type, gkrellm_get_current_time());
+	}
+
 static void
 load_net_data(void)
 	{
@@ -1519,9 +1788,10 @@ load_net_data(void)
 	GList		*list;
 	struct tm	*tm  = gkrellm_get_current_time();
 	NetMon		*net;
-	gchar		buf[128], fname[256];
+	NetStat		*ns = NULL;
+	gchar		buf[128], fname[256], date[32];
 	gint		wday, mday, month, yday, year, version;
-	gint		day_delta, month_delta;
+	gint		day_delta, month_delta, n_stats = 0;
 
 	for (list = net_mon_list; list; list = list->next)
 		{
@@ -1531,34 +1801,91 @@ load_net_data(void)
 		if ((f = fopen(fname, "r")) == NULL)
 			continue;
 		fgets(buf, sizeof(buf), f);
-		if (   sscanf(buf, "%d\n", &version) != 1
-			|| version != NET_DATA_VERSION
-		   )
+		if (sscanf(buf, "%d\n", &version) != 1)
 			{
 			fclose(f);
 			continue;
 			}
+
 		fgets(buf, sizeof(buf), f);		/* Comment line */
 		fgets(buf, sizeof(buf), f);
 		sscanf(buf, "%d %d %d %d %d", &wday, &mday, &month, &yday, &year);
 
-		fgets(buf, sizeof(buf), f);		/* day */
-		fgets(buf, sizeof(buf), f);
-		sscanf(buf, "%lf %lf", &net->rx_day, &net->tx_day);
-		fgets(buf, sizeof(buf), f);		/* week */
-		fgets(buf, sizeof(buf), f);
-		sscanf(buf, "%lf %lf", &net->rx_week, &net->tx_week);
-		fgets(buf, sizeof(buf), f);		/* month */
-		fgets(buf, sizeof(buf), f);
-		sscanf(buf, "%lf %lf", &net->rx_month, &net->tx_month);
-		if (   net == net_timed
-			&& fgets(buf, sizeof(buf), f)	/* connect day*/
-		   )
+		if (version == 1)
 			{
+			struct tm	tmx = *gkrellm_get_current_time();
+
+			tmx.tm_wday = wday;
+			tmx.tm_mday = mday;
+			tmx.tm_mon = month;
+			tmx.tm_yday = yday;
+			tmx.tm_year = year;
+
+			fgets(buf, sizeof(buf), f);		/* day */
 			fgets(buf, sizeof(buf), f);
-			sscanf(buf, "%d %d %d",
-						&net->time_day, &net->time_week, &net->time_month);
+			sscanf(buf, "%lf %lf",
+						&net->day_stats[0].rx, &net->day_stats[0].tx);
+			fgets(buf, sizeof(buf), f);		/* week */
+			fgets(buf, sizeof(buf), f);
+			sscanf(buf, "%lf %lf",
+						&net->week_stats[0].rx, &net->week_stats[0].tx);
+			fgets(buf, sizeof(buf), f);		/* month */
+			fgets(buf, sizeof(buf), f);
+			sscanf(buf, "%lf %lf",
+						&net->month_stats[0].rx, &net->month_stats[0].tx);
+
+			net_stat_set_date_string(&net->day_stats[0], DAY_STAT, &tmx);
+			net_stat_set_date_string(&net->week_stats[0], WEEK_STAT, &tmx);
+			net_stat_set_date_string(&net->month_stats[0], MONTH_STAT, &tmx);
+
+			if (   net == net_timed
+				&& fgets(buf, sizeof(buf), f)	/* connect day*/
+			   )
+				{
+				fgets(buf, sizeof(buf), f);
+				sscanf(buf, "%d %d %d",
+						&net->day_stats[0].connect_time,
+						&net->week_stats[0].connect_time,
+						&net->month_stats[0].connect_time);
+				}
+			
 			}
+		else if (version == NET_DATA_VERSION)
+			{
+			while (fgets(buf, sizeof(buf), f) != NULL)
+				{
+				if (!strncmp(buf, "[daily]", 7))
+					{
+					ns = &net->day_stats[0];
+					n_stats = N_DAY_STATS;
+					}
+				else if (!strncmp(buf, "[weekly]", 8))
+					{
+					ns = &net->week_stats[0];
+					n_stats = N_WEEK_STATS;
+					}
+				else if (!strncmp(buf, "[monthly]", 9))
+					{
+					ns = &net->month_stats[0];
+					n_stats = N_MONTH_STATS;
+					}
+				else if (   ns && n_stats > 0
+				         && sscanf(buf, "\"%31[^\"]\" %lf %lf %d",
+				               date, &ns->rx, &ns->tx, &ns->connect_time) == 4
+				        )
+					{
+					gkrellm_dup_string(&ns->date, date);
+					++ns;
+					--n_stats;
+					}
+				}
+			}
+		else
+			{
+			fclose(f);
+			continue;
+			}
+
 		fclose(f);
 
 		month_delta = (tm->tm_year * 12 + tm->tm_mon) - (year * 12 + month);
@@ -1570,15 +1897,11 @@ load_net_data(void)
 			day_delta = 7;		/* max compared to is 6 */
 
 		if (day_delta > 0)
-			{
-			net->rx_day = net->tx_day = 0.0;
-			net->time_day = 0;
-			}
+			net_stats_shift_down(&net->day_stats[0], N_DAY_STATS, DAY_STAT);
+
 		if (wday + day_delta != tm->tm_wday)
-			{
-			net->rx_week = net->tx_week = 0.0;
-			net->time_week = 0;
-			}
+			net_stats_shift_down(&net->week_stats[0], N_WEEK_STATS, WEEK_STAT);
+
 		if (   (   tm->tm_mday < reset_mday
 				&& (   (month_delta > 1)
 					|| (month_delta == 1 && mday < reset_mday)
@@ -1590,10 +1913,8 @@ load_net_data(void)
 				   )
 			   )
 		   )
-			{
-			net->rx_month = net->tx_month = 0.0;
-			net->time_month = 0;
-			}
+			net_stats_shift_down(&net->month_stats[0], N_MONTH_STATS,
+						MONTH_STAT);
 		}
 	}
 
@@ -1781,25 +2102,24 @@ update_net(void)
 			if (GK.day_tick)
 				{
 				tm = gkrellm_get_current_time();
-				net->rx_day = net->tx_day = 0.0;
-				net->time_day = 0;
+				net_stats_shift_down(&net->day_stats[0], N_DAY_STATS,
+							DAY_STAT);
+
 				if (tm->tm_wday == 0)
-					{
-					net->rx_week = net->tx_week = 0.0;
-					net->time_week = 0;
-					}
+					net_stats_shift_down(&net->week_stats[0], N_WEEK_STATS,
+								WEEK_STAT);
+
 				if (net_accounting_month_new(tm))
-					{
-					net->rx_month = net->tx_month = 0.0;
-					net->time_month = 0;
-					}
+					net_stats_shift_down(&net->month_stats[0], N_MONTH_STATS,
+								MONTH_STAT);
 				}
-			net->rx_day   += rxd;
-			net->rx_week  += rxd;
-			net->rx_month += rxd;
-			net->tx_day   += txd;
-			net->tx_week  += txd;
-			net->tx_month += txd;
+			net->day_stats[0].rx   += rxd;
+			net->week_stats[0].rx  += rxd;
+			net->month_stats[0].rx += rxd;
+
+			net->day_stats[0].tx   += txd;
+			net->week_stats[0].tx  += txd;
+			net->month_stats[0].tx += txd;
 
 			if (net->alert)
 				{
@@ -1862,6 +2182,9 @@ create_timed_monitor(void)
 	NetMon		*net;
 	TimerType	*tt;
 
+	if (_GK.client_mode)
+		return;
+
 	net_timed = NULL;
 	timer_button_type = TIMER_TYPE_NONE;
 	time(&net_timer0);
@@ -2113,7 +2436,9 @@ save_net_config(FILE *f)
 						net->alert_uses_rx, net->alert_uses_tx);
 			}
 		}
-	fprintf(f, "%s timer_enabled %d\n", NET_CONFIG_KEYWORD,
+
+	if (!_GK.client_mode || timer_button_type != TIMER_TYPE_SERVER)
+		fprintf(f, "%s timer_enabled %d\n", NET_CONFIG_KEYWORD,
 						timer_button_enabled);
 	fprintf(f, "%s timer_seconds %d\n", NET_CONFIG_KEYWORD, timer_seconds);
 	if (!_GK.client_mode)
@@ -2123,6 +2448,8 @@ save_net_config(FILE *f)
 	fprintf(f, "%s timer_off %s\n", NET_CONFIG_KEYWORD, timer_off_command);
 	fprintf(f, "%s text_format %s\n", NET_CONFIG_KEYWORD, text_format);
 	fprintf(f, "%s reset_mday %d\n", NET_CONFIG_KEYWORD, reset_mday);
+	fprintf(f, "%s net_stats_window_height %d\n", NET_CONFIG_KEYWORD,
+				net_stats_window_height);
 	}
 
 static void
@@ -2138,12 +2465,16 @@ load_net_config(gchar *arg)
 	if (n != 2)
 		return;
 
-	if (!strcmp(config, "timer_enabled") && ! _GK.demo)
-		sscanf(item, "%d", &timer_button_enabled);
-	else if (!strcmp(config, "timer_seconds"))
+	if (!_GK.client_mode || timer_button_type != TIMER_TYPE_SERVER)
+		if (!strcmp(config, "timer_enabled") && ! _GK.demo)
+			sscanf(item, "%d", &timer_button_enabled);
+
+	if (!strcmp(config, "timer_seconds"))
 		sscanf(item, "%d", &timer_seconds);
 	else if (!strcmp(config, "reset_mday"))
 		sscanf(item, "%d", &reset_mday);
+	else if (!strcmp(config, "net_stats_window_height"))
+		sscanf(item, "%d", &net_stats_window_height);
 	else if (!strcmp(config, "timer_iface") && !_GK.client_mode)
 		{
 		if (_GK.demo && !strcmp(item, "none"))
@@ -2473,7 +2804,48 @@ cb_poff_entry(GtkWidget *widget, gpointer data)
 static void
 cb_reset_mday(GtkWidget *widget, GtkSpinButton *spin)
 	{
+	GList		*list;
+	NetMon		*net;
+	NetStat		*ns;
+	struct tm	*tm;
+	gint		year, month, mday, month_delta;
+
 	reset_mday = gtk_spin_button_get_value_as_int(spin);
+
+	for (list = net_mon_list; list; list = list->next)
+		{
+		net = (NetMon *) list->data;
+		net->month_stats[0].rx = net->month_stats[0].tx = 0.0;
+		net->month_stats[0].connect_time = 0;
+		tm = gkrellm_get_current_time();
+		net_stat_set_date_string(&net->month_stats[0], MONTH_STAT, tm);
+
+		for (ns = &net->day_stats[0]; ns < &net->day_stats[N_DAY_STATS]; ++ns)
+			{
+			if (sscanf(ns->date, "%d-%d-%d", &year, &month, &mday) != 3)
+				continue;
+			year -= 1900;	/* Adjust to struct tm ranges */
+			month -= 1;
+			month_delta = (tm->tm_year * 12 + tm->tm_mon)
+								- (year * 12 + month);
+			if (   (   tm->tm_mday < reset_mday
+			        && (   (month_delta > 1)
+			            || (month_delta == 1 && mday < reset_mday)
+			           )
+			       )
+			    || (   tm->tm_mday >= reset_mday
+			        && (   (month_delta > 0)
+			            || (month_delta == 0 && mday < reset_mday)
+			           )
+			       )
+			   )
+				continue;
+
+			net->month_stats[0].rx += ns->rx;
+			net->month_stats[0].tx += ns->tx;
+			net->month_stats[0].connect_time += ns->connect_time;
+			}
+		}
 	}
 
 #if !defined(WIN32)
@@ -2554,10 +2926,12 @@ create_net_tab(GtkWidget *tab_vbox)
 
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Timer Button"));
 
-	gkrellm_gtk_check_button_connected(vbox, &enable_net_timer_button,
-				timer_button_enabled, FALSE, FALSE, 10,
-				cb_timer_enable, NULL,
-				_("Enable timer button"));
+	if (timer_button_type != TIMER_TYPE_SERVER) /* enabled by gkrellmd.conf? */
+		gkrellm_gtk_check_button_connected(vbox, &enable_net_timer_button,
+					timer_button_enabled, FALSE, FALSE, 10,
+					cb_timer_enable, NULL,
+					_("Enable timer button"));
+
 	gkrellm_gtk_check_button_connected(vbox, &net_timer_seconds_button,
 				timer_seconds, FALSE, FALSE, 0,
 				cb_timer_seconds, NULL,
@@ -2652,7 +3026,7 @@ create_net_tab(GtkWidget *tab_vbox)
 		gtk_entry_set_text(GTK_ENTRY(net->label_entry), net->label);
 		gtk_box_pack_start (GTK_BOX (hbox), net->label_entry, FALSE, TRUE, 2);
 		label = gtk_label_new(_("Optional label for this interface."));
-		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 4);
+		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 4);
 		g_signal_connect(G_OBJECT(net->label_entry), "changed",
 					G_CALLBACK(cb_label_entry), net);
 
@@ -2753,16 +3127,21 @@ gkrellm_init_net_monitor(void)
 		}
 	else
 		{
-		timer_button_type = TIMER_TYPE_NONE;
 		timer_button_iface = g_strdup("none");
 		}
-	timer_button_enabled = !_GK.client_mode;
+	if (!_GK.client_mode)
+		timer_button_enabled = TRUE;
+	else
+		timer_button_enabled = (timer_button_type == TIMER_TYPE_SERVER);
+
 	timer_seconds = TRUE;
 	timer_on_command = g_strdup("");
 	timer_off_command = g_strdup("");
 	gkrellm_locale_dup_string(&text_format, DEFAULT_TEXT_FORMAT,
 					&text_format_locale);
 
+	net_stats_window_height = 200;
+
     monitor_net.name = _(monitor_net.name);
 	net_style_id = gkrellm_add_chart_style(&monitor_net, NET_STYLE_NAME);
 	mon_net = &monitor_net;
