@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -79,7 +79,8 @@ static GList	*temp_order_list,	/* For ordering from the config. */
 				*fan_order_list,
 				*volt_order_list;
 
-static gboolean	using_new_config;
+static gboolean	using_new_config,
+				need_disk_temperature_update;
 
 static gboolean	(*get_temperature)(gchar *name, gint id,
 			gint iodev, gint inter, gfloat *t);
@@ -161,8 +162,6 @@ gkrellm_sensors_add_sensor(gint type, gchar *sensor_path, gchar *id_name,
 	return (gpointer) sensor;
 	}
 
-
-
 /* ======================================================================== */
 static gboolean	use_threads,
 				thread_data_valid,
@@ -179,6 +178,8 @@ read_sensors_thread(void *data)
 	for (list = sensor_list; list; list = list->next)
 		{
 		sensor = (Sensor *) list->data;
+		if (!sensor->enabled)
+			continue;
 		if (sensor->type == SENSOR_TEMPERATURE && get_temperature)
 			(*get_temperature)(sensor->path, sensor->id,
 				sensor->iodev, sensor->inter, &sensor->raw_value);
@@ -725,6 +726,57 @@ sensor_relocation_error(gchar *pname)
 	g_free(msg);
 	}
 
+  /* When moving off some other panel, reset that panel.
+  */
+static void
+sensor_reset_location(Sensor *sr)
+	{
+	if (sr->group == SENSOR_GROUP_MAINBOARD)
+		{
+		if (sr->location == PROC_PANEL_LOCATION)
+			gkrellm_proc_set_sensor(NULL, sr->type);
+		else if (sr->location >= CPU_PANEL_LOCATION)
+			gkrellm_cpu_set_sensor(NULL, sr->type,
+						sr->location - CPU_PANEL_LOCATION);
+		}
+	else if (sr->group == SENSOR_GROUP_DISK)
+		{
+		if (sr->location == DISK_PANEL_LOCATION)
+			gkrellm_disk_temperature_remove(sr->id_name);
+		}
+	}
+
+void
+gkrellm_sensors_interface_remove(gint interface)
+	{
+	GList		*list;
+	Sensor		*sensor;
+	gboolean	removed_one;
+
+	do
+		{
+		removed_one = FALSE;
+		for (list = sensor_list; list; list = list->next)
+			{
+			sensor = (Sensor *) list->data;
+			if (sensor->inter == interface)
+				{
+				sensor_reset_location(sensor);
+				g_free(sensor->id_name);
+				g_free(sensor->path);
+				g_free(sensor->name);
+				g_free(sensor->default_label);
+				g_free(sensor->vref_name);
+				sensor_list = g_list_remove(sensor_list, sensor);
+				g_free(sensor);
+				removed_one = TRUE;
+				break;
+				}
+			}
+		}
+	while (removed_one);		
+	}
+
 static void
 add_sensor_monitor(Sensor *sr, GkrellmPanel **p, GList **smon_list)
 	{
@@ -1039,6 +1091,49 @@ update_disk_temperatures(void)
 		gkrellm_sensors_rebuild(TRUE, FALSE, FALSE);
 	}
 
+  /* Squeeze name decal text into a smaller font if it would overlap the
+  |  sensor decal.  With smaller fonts, the y_ink value may be smaller which
+  |  would bump the text upward.  So adjust decal offset by difference in
+  |  y_ink value.  (GKrellM text decal heights don't include the y_ink
+  |  space).
+  */
+static gchar *
+name_text_fit(Sensor *s, GkrellmDecal *dn, GkrellmDecal *ds)
+	{
+	gchar	*string;
+	gint	x_limit, w0, w1, y_ink0, y_ink1, h0, h1;
+
+	x_limit = ds->x;
+
+	/* Check for '<' in case user is doing his own markup
+	*/
+	if (*(s->name_locale) != '<' && dn->x + dn->w > x_limit)
+		{
+		gkrellm_text_markup_extents(dn->text_style.font, s->name_locale,
+				strlen(s->name_locale), &w0, NULL, NULL, &y_ink0);
+		string = g_strdup_printf("<small>%s</small>", s->name_locale);
+		gkrellm_text_markup_extents(dn->text_style.font, string,
+				strlen(string), &w1, &h0, NULL, &y_ink1);
+		h1 = h0;
+		if (dn->x + w1 > x_limit)
+			{
+			g_free(string);
+			string = g_strdup_printf("<small><small>%s</small></small>",
+						s->name_locale);
+			gkrellm_text_markup_extents(dn->text_style.font, string,
+					strlen(string), &w1, &h1, NULL, &y_ink1);
+			}
+		gkrellm_decal_text_set_offset(dn, 0,
+					y_ink0 - y_ink1 + (h0 - h1 + 1) / 2);
+		}
+	else
+		{
+		gkrellm_decal_text_set_offset(dn, 0, 0);
+		string = g_strdup(s->name_locale);
+		}
+	return string;
+	}
+
 
 static void
 draw_temperatures(gboolean draw_name)
@@ -1047,7 +1142,7 @@ draw_temperatures(gboolean draw_name)
 	SensorMon	*smon;
 	Sensor		*sensor;
 	gfloat		t;
-	gchar		units;
+	gchar		*name, units;
 
 	if (!pTemp)
 		return;
@@ -1057,8 +1152,11 @@ draw_temperatures(gboolean draw_name)
 		sensor = smon->sensor;
 
 		if (draw_name && smon->name_decal)
-			gkrellm_draw_decal_text(pTemp, smon->name_decal,
-						sensor->name_locale, -1);
+			{
+			name = name_text_fit(sensor, smon->name_decal, smon->sensor_decal);
+			gkrellm_draw_decal_markup(pTemp, smon->name_decal, name);
+			g_free(name);
+			}
 		if (smon->sensor_decal)
 			{
 			sensor_read_temperature(sensor, &t, &units);
@@ -1075,6 +1173,7 @@ draw_fans(gboolean draw_name)
 	GList		*list;
 	SensorMon	*smon;
 	Sensor		*sensor;
+	gchar		*name;
 	gfloat		f;
 
 	if (!pFan)
@@ -1085,8 +1184,11 @@ draw_fans(gboolean draw_name)
 		sensor = smon->sensor;
 
 		if (draw_name && smon->name_decal)
-			gkrellm_draw_decal_text(pFan, smon->name_decal,
-						sensor->name_locale, -1);
+			{
+			name = name_text_fit(sensor, smon->name_decal, smon->sensor_decal);
+			gkrellm_draw_decal_markup(pFan, smon->name_decal, name);
+			g_free(name);
+			}
 		if (smon->sensor_decal)
 			{
 			sensor_read_fan(sensor, &f);
@@ -1104,8 +1206,8 @@ draw_voltages(Sensor *s, gint do_names)
 	GList			*list;
 	SensorMon		*volt;
 	Sensor			*sensor;
-	GkrellmDecal	*d;
-	gchar			*fmt, buf[32];
+	GkrellmDecal	*ds, *dn;
+	gchar			*name, *fmt, buf[32];
 	gfloat			v;
 
 	if (!pVolt)
@@ -1117,10 +1219,15 @@ draw_voltages(Sensor *s, gint do_names)
 		if (s && s != sensor)
 			continue;
 		sensor->value_valid = FALSE;	/* In case vref monitoring stops */
-		if (do_names && volt->name_decal)
-			gkrellm_draw_decal_text(pVolt, volt->name_decal,
-						sensor->name_locale, -1);
-		if ((d = volt->sensor_decal) != NULL)
+		dn = volt->name_decal;
+		ds = volt->sensor_decal;
+		if (do_names && dn)
+			{
+			name = name_text_fit(sensor, dn, ds);
+			gkrellm_draw_decal_markup(pVolt, dn, name);
+			g_free(name);
+			}
+		if (ds)
 			{
 			if (sensor->vref && !sensor->vref->value_valid)
 				sensor_read_voltage(sensor->vref, NULL);
@@ -1131,8 +1238,8 @@ draw_voltages(Sensor *s, gint do_names)
 			else
 				fmt = "%.1f";
 			snprintf(buf, sizeof(buf), fmt, v);
-			d->x_off = (v < 0.0) ? 0 : minus_width;
-			gkrellm_draw_decal_text(pVolt, d, buf, -1);
+			ds->x_off = (v < 0.0) ? 0 : minus_width;
+			gkrellm_draw_decal_text(pVolt, ds, buf, -1);
 			}
 		}
 	gkrellm_draw_panel_layers(pVolt);
@@ -1144,7 +1251,12 @@ update_sensors(void)
 	static gboolean		first_time_done;
 
 	if (!GK.five_second_tick && first_time_done)
+		{
+		if (need_disk_temperature_update)	/* delayed until disks created */
+			update_disk_temperatures();
+		need_disk_temperature_update = FALSE;
 		return;
+		}
 	if (use_threads)
 		{
 		thread_data_valid = TRUE;
@@ -1461,6 +1573,9 @@ create_sensors(GtkWidget *vbox, gint first_create)
 
 		bezel_style = gkrellm_style_new0();
 		}
+	else		/* To be done after disk panels created */
+		need_disk_temperature_update = TRUE;
+
 	config_loaded = TRUE;
 
 	/* Here is where I define the volt panel theme image extensions.  I ask
@@ -1619,6 +1734,9 @@ save_sensors_config(FILE *f_not_used)
 				units_fahrenheit);
 	fprintf(f, "%s volt_display_mode %d\n", SENSOR_CONFIG_KEYWORD,
 				display_mode);
+	/* _GK.mbmon_port is handled in config.c so that the port can be
+	|  loaded early for sensor initialization.
+	*/
 	fclose(f);
 	}
 
@@ -1760,6 +1878,7 @@ enum
 	N_COLUMNS
 	};
 
+static GtkTreeModel			*sensor_model;
 static GtkTreeView			*treeview;
 static GtkTreeRowReference	*row_reference;
 static GtkTreeSelection		*selection;
@@ -1771,7 +1890,8 @@ static GtkWidget	*optionmenu,
 static GtkWidget	*display_mode_button[2];
 static GtkWidget	*factor_spin_button,
 					*offset_spin_button;
-static GtkWidget	*alert_button;
+static GtkWidget	*alert_button,
+					*mbmon_port_entry;
 
 static Sensor		*dragged_sensor;
 
@@ -1854,6 +1974,20 @@ create_model(void)
 	return GTK_TREE_MODEL(tree);
 	}
 
+void
+gkrellm_sensors_model_update(void)
+	{
+	GtkTreeModel	*model;
+
+	if (!gkrellm_config_window_shown())
+		return;
+	model = sensor_model;
+	sensor_model = create_model();
+	gtk_tree_view_set_model(treeview, sensor_model);
+	if (model)
+		g_object_unref(G_OBJECT(model));
+	}
+
 static void
 change_row_reference(GtkTreeModel *model, GtkTreePath *path)
 	{
@@ -2085,19 +2219,7 @@ cb_location_menu(GtkOptionMenu *om, gpointer data)
 
 	/* If moving off some other panel, reset that panel.
 	*/
-	if (sr->group == SENSOR_GROUP_MAINBOARD)
-		{
-		if (sr->location == PROC_PANEL_LOCATION)
-			gkrellm_proc_set_sensor(NULL, sr->type);
-		else if (sr->location >= CPU_PANEL_LOCATION)
-			gkrellm_cpu_set_sensor(NULL, sr->type,
-						sr->location - CPU_PANEL_LOCATION);
-		}
-	else if (sr->group == SENSOR_GROUP_DISK)
-		{
-		if (sr->location == DISK_PANEL_LOCATION)
-			gkrellm_disk_temperature_remove(sr->id_name);
-		}
+	sensor_reset_location(sr);
 
 	/* For mainboard sensor group, if relocating to a panel with some other
 	|  sensor of same type on it, auto restore the other sensor to the sensor
@@ -2279,7 +2401,7 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	}
 
 static void
-cb_label_edited(GtkCellRendererText *cell, gchar *path_string,
+label_edited_cb(GtkCellRendererText *cell, gchar *path_string,
 			gchar *new_label, gpointer data)
 	{
 	GtkTreeModel	*model;
@@ -2287,7 +2409,7 @@ cb_label_edited(GtkCellRendererText *cell, gchar *path_string,
 	GtkTreePath		*path;
 	Sensor			*s;
 
-	model = GTK_TREE_MODEL(data);
+	model = sensor_model;
 	path = gtk_tree_path_new_from_string(path_string);
 	gtk_tree_model_get_iter(model, &iter, path);
 	gtk_tree_path_free(path);
@@ -2315,7 +2437,7 @@ cb_label_edited(GtkCellRendererText *cell, gchar *path_string,
 	}
 
 static void
-cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
+enable_cb(GtkCellRendererText *cell, gchar *path_string, gpointer data)
 	{
 	GtkTreeModel	*model;
 	GtkTreeIter		iter;
@@ -2323,7 +2445,7 @@ cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
 	Sensor			*s;
 	gboolean		enabled;
 
-	model = GTK_TREE_MODEL(data);
+	model = sensor_model;
 	path = gtk_tree_path_new_from_string(path_string);
 	gtk_tree_model_get_iter(model, &iter, path);
 
@@ -2412,6 +2534,36 @@ cb_set_alert(GtkWidget *widget, gpointer data)
 	}
 
 
+static void
+sensors_apply(void)
+	{
+	gchar	*str;
+	gint	port;
+
+	if (mbmon_port_entry)
+		{
+		str = gkrellm_gtk_entry_get_text(&mbmon_port_entry);
+		if (isdigit(*str))
+			{
+			port = atoi(str);
+			if (_GK.mbmon_port != port)
+				{
+				if (!gkrellm_sys_sensors_mbmon_port_change(port) && port > 0)
+					gkrellm_message_dialog(NULL,
+				_("Can't read sensor data from mbmon daemon.\n"
+				  "Check mbmon port number and '-r' option.\n"
+				  "Run gkrellm -d 0x80 for debug output.\n"));
+				}
+			}
+		}
+	}
+
+static void
+mbmon_port_entry_activate_cb(GtkWidget *widget, gpointer data)
+	{
+	sensors_apply();
+	}
+
 static void
 cb_config_deleted(gpointer data)
 	{
@@ -2456,9 +2608,10 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	GtkWidget				*vbox, *vbox1, *hbox, *box;
 	GtkWidget				*scrolled;
 	GtkWidget				*image;
-	GtkWidget				*label;
+	GtkWidget				*label, *entry;
 	GtkTreeModel			*model;
 	GtkCellRenderer			*renderer;
+	gchar					buf[32];
 	gint					i;
 
 	row_reference = NULL;
@@ -2484,6 +2637,7 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(hbox), scrolled, TRUE, TRUE, 0);
 
 	model = create_model();
+	sensor_model = model;
 
 	treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	gtk_tree_view_set_rules_hint(treeview, TRUE);
@@ -2508,7 +2662,7 @@ create_sensors_tab(GtkWidget *tab_vbox)
 				"visible", VISIBLE_COLUMN,
 				NULL);
 	g_signal_connect(G_OBJECT(renderer), "toggled",
-				G_CALLBACK(cb_enable), model);
+				G_CALLBACK(enable_cb), NULL);
 
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_insert_column_with_attributes(treeview, -1, _("Label"),
@@ -2518,7 +2672,7 @@ create_sensors_tab(GtkWidget *tab_vbox)
 				"visible", VISIBLE_COLUMN,
 				NULL);
 	g_signal_connect(G_OBJECT(renderer), "edited",
-				G_CALLBACK(cb_label_edited), model);
+				G_CALLBACK(label_edited_cb), NULL);
 
 	renderer = gtk_cell_renderer_pixbuf_new();
 	gtk_tree_view_insert_column_with_attributes(treeview, -1, "",
@@ -2534,8 +2688,6 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	g_signal_connect(G_OBJECT(selection), "changed",
 				G_CALLBACK(cb_tree_selection_changed), NULL);
 
-	g_object_unref(G_OBJECT(model));
-
 	box = gkrellm_gtk_framed_vbox(vbox1, _("Factor"), 4, FALSE, 0, 2);
 	gkrellm_gtk_spin_button(box, &factor_spin_button, 1.0,
 				-1000.0, 1000.0, 0.01, 1.0, 4, 60,
@@ -2573,7 +2725,8 @@ create_sensors_tab(GtkWidget *tab_vbox)
 
 /* -- Options tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Options"));
-	box = gkrellm_gtk_framed_vbox(vbox, _("Temperatures"), 4, FALSE, 0, 2);
+//	box = gkrellm_gtk_framed_vbox(vbox, _("Temperatures"), 4, FALSE, 0, 2);
+	box = gkrellm_gtk_category_vbox(vbox, _("Temperatures"), 4, 0, TRUE);
 	gkrellm_gtk_check_button_connected(box, &button,
 				units_fahrenheit, FALSE, FALSE, 0,
 				cb_temperature_units, NULL,
@@ -2581,7 +2734,8 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	if (!sensor_list)
 		gtk_widget_set_sensitive(button, FALSE);
 
-	box = gkrellm_gtk_framed_vbox(vbox, _("Voltages"), 6, FALSE, 0, 2);
+//	box = gkrellm_gtk_framed_vbox(vbox, _("Voltages"), 6, FALSE, 0, 2);
+	box = gkrellm_gtk_category_vbox(vbox, _("Voltages"), 4, 0, TRUE);
 	button = gtk_radio_button_new_with_label(NULL,
 				_("Normal with labels"));
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, TRUE, 0);
@@ -2602,6 +2756,28 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	button = display_mode_button[display_mode];
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
+	if (gkrellm_sys_sensors_mbmon_supported())
+		{
+		box = gkrellm_gtk_category_vbox(vbox, _("MBmon Daemon Port"),
+					4, 0, TRUE);
+		label = gtk_label_new("");
+		gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+		gtk_label_set_markup(GTK_LABEL(label),
+			_("<small>Daemon command must be: <b>mbmon -r -P port</b>\n"
+			  "where 'port' must match the port number entered here:</small>"));
+		gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
+		entry = gtk_entry_new();
+		mbmon_port_entry = entry;
+		if (_GK.mbmon_port > 0)
+			{
+			snprintf(buf, sizeof(buf), "%d", _GK.mbmon_port);
+			gtk_entry_set_text(GTK_ENTRY(entry), buf);
+			}
+		gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 4);
+		g_signal_connect(G_OBJECT(mbmon_port_entry), "activate",
+	                G_CALLBACK(mbmon_port_entry_activate_cb), NULL);
+		}
+
 /* --Info tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
 	text = gkrellm_gtk_scrolled_text_view(vbox, NULL,
@@ -2613,6 +2789,10 @@ create_sensors_tab(GtkWidget *tab_vbox)
 
 	for (i = 0; i < sizeof(sensor_info_text1) / sizeof(gchar *); ++i)
 		gkrellm_gtk_text_view_append(text, _(sensor_info_text1[i]));
+
+	/* Present as instant apply, but still get our apply function called.
+	*/
+	gkrellm_config_instant_apply(mon_config_sensors);
 	}
 
 GkrellmMonitor *
@@ -2628,7 +2808,7 @@ static GkrellmMonitor	monitor_config_sensors =
 	NULL,				/* The create function		*/
 	NULL,				/* The update function		*/
 	create_sensors_tab,	/* The config tab create function	*/
-	NULL,				/* Sensors now do instant apply */
+	sensors_apply,
 
 	save_sensors_config, /* Save user conifg			*/
 	load_sensors_config, /* Load user config			*/
