@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -23,8 +23,13 @@
 #include "gkrellm-private.h"
 #include "gkrellm-sysdeps.h"
 
+
+static gboolean (*mbmon_check_func)();
+
+
 #if defined(__linux__)
 #include "sysdeps/linux.c"
+#include "sysdeps/sensors-common.c"
 #endif
 
 #if defined(__APPLE__)
@@ -35,6 +40,7 @@
 #if defined(__FreeBSD__)
 #include "sysdeps/freebsd.c"
 #include "sysdeps/bsd-common.c"
+#include "sysdeps/sensors-common.c"
 #endif
 
 #if defined(__NetBSD__)
@@ -84,3 +90,28 @@ gkrellm_sys_get_system_name(void)
 	return sname;
 	}
 
+gboolean
+gkrellm_sys_sensors_mbmon_port_change(gint port)
+	{
+	gboolean	result = FALSE;
+
+	_GK.mbmon_port = port;
+
+	/* mbmon_check_func will be set if sysdep code has included
+	|  sensors_common.c and has run gkrellm_sys_sensors_mbmon_check()
+	*/
+	if (mbmon_check_func)
+		{
+		gkrellm_sensors_interface_remove(MBMON_INTERFACE);
+		result = (*mbmon_check_func)(TRUE);
+		gkrellm_sensors_model_update();
+		gkrellm_sensors_rebuild(TRUE, TRUE, TRUE);
+		}
+	return result;
+	}
+
+gboolean
+gkrellm_sys_sensors_mbmon_supported(void)
+	{
+	return mbmon_check_func ? TRUE : FALSE;
+	}
