@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -189,7 +189,8 @@ linux_read_proc_stat(void)
 			for (i = 0; i < 4; ++ i)
 				{
 				wblk[i] = strtoul(s, &s, 0);
-				gkrellm_disk_assign_data_nth(i, 512 * rblk[i], 512 * wblk[i]);
+				gkrellm_disk_assign_data_nth(i,
+							512 * rblk[i], 512 * wblk[i], FALSE);
 				}
 			}
 		else if (!strncmp("disk_io:", buf, 8))		/* Kernel 2.4 only	*/
@@ -226,11 +227,14 @@ linux_read_proc_stat(void)
 						sprintf(name, "scd%d", i_disk);
 					if (name[0])
 						gkrellm_disk_assign_data_by_name(name,
-								512 * rblk, 512 * wblk);
+								512 * rblk, 512 * wblk, FALSE);
 					}
 				else
+					{
 					gkrellm_disk_assign_data_by_device(major, i_disk,
-								512 * rblk, 512 * wblk);
+								512 * rblk, 512 * wblk,
+								(major == 9) ? TRUE : FALSE);
+					}
 				s = strtok(NULL, " \t\n");
 				}
 			}
@@ -350,37 +354,37 @@ static struct _disk_name_map
 	{"hd",	IDE8_MAJOR,					64,	'q' },	/* 90: hdq, hdr */
 	{"hd",	IDE9_MAJOR,					64,	's' },	/* 91: hds, hdt */
 	{"sd",	SCSI_DISK0_MAJOR,			16,	'a' },	/* 8:  sda-sdh */
-	{"sg",	SCSI_GENERIC_MAJOR,			0,	'0' },	/* 21: sg0-sg16 */
-	{"scd",	SCSI_CDROM_MAJOR,			0,	'0' },	/* 11: scd0-scd16 */
-	{"sr",	SCSI_CDROM_MAJOR,			0,	'0' },	/* 11: sr0-sr16 */
+	{"sg",	SCSI_GENERIC_MAJOR,			16,	'0' },	/* 21: sg0-sg16 */
+	{"scd",	SCSI_CDROM_MAJOR,			16,	'0' },	/* 11: scd0-scd16 */
+	{"sr",	SCSI_CDROM_MAJOR,			16,	'0' },	/* 11: sr0-sr16 */
 	{"md",	MD_MAJOR,					0,	'0' },	/* 9:  md0-md3 */
 
-	{"c0d",	DAC960_MAJOR,				0,	'0' },	/* 48:  c0d0-c0d31 */
-	{"c1d",	DAC960_MAJOR + 1,			0,	'0' },	/* 49:  c1d0-c1d31 */
-	{"c2d",	DAC960_MAJOR + 2,			0,	'0' },	/* 50:  c2d0-c2d31 */
-	{"c3d",	DAC960_MAJOR + 3,			0,	'0' },	/* 51:  c3d0-c3d31 */
-	{"c4d",	DAC960_MAJOR + 4,			0,	'0' },	/* 52:  c4d0-c4d31 */
-	{"c5d",	DAC960_MAJOR + 5,			0,	'0' },	/* 53:  c5d0-c5d31 */
-	{"c6d",	DAC960_MAJOR + 6,			0,	'0' },	/* 54:  c6d0-c6d31 */
-	{"c7d",	DAC960_MAJOR + 7,			0,	'0' },	/* 55:  c7d0-c7d31 */
+	{"c0d",	DAC960_MAJOR,				32,	'0' },	/* 48:  c0d0-c0d31 */
+	{"c1d",	DAC960_MAJOR + 1,			32,	'0' },	/* 49:  c1d0-c1d31 */
+	{"c2d",	DAC960_MAJOR + 2,			32,	'0' },	/* 50:  c2d0-c2d31 */
+	{"c3d",	DAC960_MAJOR + 3,			32,	'0' },	/* 51:  c3d0-c3d31 */
+	{"c4d",	DAC960_MAJOR + 4,			32,	'0' },	/* 52:  c4d0-c4d31 */
+	{"c5d",	DAC960_MAJOR + 5,			32,	'0' },	/* 53:  c5d0-c5d31 */
+	{"c6d",	DAC960_MAJOR + 6,			32,	'0' },	/* 54:  c6d0-c6d31 */
+	{"c7d",	DAC960_MAJOR + 7,			32,	'0' },	/* 55:  c7d0-c7d31 */
 
-	{"cs0d", COMPAQ_SMART2_MAJOR,		0,	'0' },	/* 72:  c0d0-c0d15 */
-	{"cs1d", COMPAQ_SMART2_MAJOR + 1,	0,	'0' },	/* 73:  c1d0-c1d15 */
-	{"cs2d", COMPAQ_SMART2_MAJOR + 2,	0,	'0' },	/* 74:  c2d0-c2d15 */
-	{"cs3d", COMPAQ_SMART2_MAJOR + 3,	0,	'0' },	/* 75:  c3d0-c3d15 */
-	{"cs4d", COMPAQ_SMART2_MAJOR + 4,	0,	'0' },	/* 76:  c4d0-c4d15 */
-	{"cs5d", COMPAQ_SMART2_MAJOR + 5,	0,	'0' },	/* 77:  c5d0-c5d15 */
-	{"cs6d", COMPAQ_SMART2_MAJOR + 6,	0,	'0' },	/* 78:  c6d0-c6d15 */
-	{"cs7d", COMPAQ_SMART2_MAJOR + 7,	0,	'0' },	/* 79:  c7d0-c7d15 */
+	{"cs0d", COMPAQ_SMART2_MAJOR,		16,	'0' },	/* 72:  c0d0-c0d15 */
+	{"cs1d", COMPAQ_SMART2_MAJOR + 1,	16,	'0' },	/* 73:  c1d0-c1d15 */
+	{"cs2d", COMPAQ_SMART2_MAJOR + 2,	16,	'0' },	/* 74:  c2d0-c2d15 */
+	{"cs3d", COMPAQ_SMART2_MAJOR + 3,	16,	'0' },	/* 75:  c3d0-c3d15 */
+	{"cs4d", COMPAQ_SMART2_MAJOR + 4,	16,	'0' },	/* 76:  c4d0-c4d15 */
+	{"cs5d", COMPAQ_SMART2_MAJOR + 5,	16,	'0' },	/* 77:  c5d0-c5d15 */
+	{"cs6d", COMPAQ_SMART2_MAJOR + 6,	16,	'0' },	/* 78:  c6d0-c6d15 */
+	{"cs7d", COMPAQ_SMART2_MAJOR + 7,	16,	'0' },	/* 79:  c7d0-c7d15 */
 
-	{"cc0d", COMPAQ_CISS_MAJOR,			0,	'0' },	/* 104:  c0d0-c0d15 */
-	{"cc1d", COMPAQ_CISS_MAJOR + 1,		0,	'0' },	/* 105:  c1d0-c1d15 */
-	{"cc2d", COMPAQ_CISS_MAJOR + 2,		0,	'0' },	/* 106:  c2d0-c2d15 */
-	{"cc3d", COMPAQ_CISS_MAJOR + 3,		0,	'0' },	/* 107:  c3d0-c3d15 */
-	{"cc4d", COMPAQ_CISS_MAJOR + 4,		0,	'0' },	/* 108:  c4d0-c4d15 */
-	{"cc5d", COMPAQ_CISS_MAJOR + 5,		0,	'0' },	/* 109:  c5d0-c5d15 */
-	{"cc6d", COMPAQ_CISS_MAJOR + 6,		0,	'0' },	/* 110:  c6d0-c6d15 */
-	{"cc7d", COMPAQ_CISS_MAJOR + 7,		0,	'0' },	/* 111:  c7d0-c7d15 */
+	{"cc0d", COMPAQ_CISS_MAJOR,			16,	'0' },	/* 104:  c0d0-c0d15 */
+	{"cc1d", COMPAQ_CISS_MAJOR + 1,		16,	'0' },	/* 105:  c1d0-c1d15 */
+	{"cc2d", COMPAQ_CISS_MAJOR + 2,		16,	'0' },	/* 106:  c2d0-c2d15 */
+	{"cc3d", COMPAQ_CISS_MAJOR + 3,		16,	'0' },	/* 107:  c3d0-c3d15 */
+	{"cc4d", COMPAQ_CISS_MAJOR + 4,		16,	'0' },	/* 108:  c4d0-c4d15 */
+	{"cc5d", COMPAQ_CISS_MAJOR + 5,		16,	'0' },	/* 109:  c5d0-c5d15 */
+	{"cc6d", COMPAQ_CISS_MAJOR + 6,		16,	'0' },	/* 110:  c6d0-c6d15 */
+	{"cc7d", COMPAQ_CISS_MAJOR + 7,		16,	'0' },	/* 111:  c7d0-c7d15 */
 
 	{"fd",	FLOPPY_MAJOR,				0,	'0' }	/* 2:  fd0-fd3  */
 	};
@@ -521,7 +525,7 @@ linux_read_proc_diskstats(void)
 	gchar			buf[1024], part[64], disk[64];
 	gint			major, minor, n;
 	gulong			rd, wr, rd1, wr1;
-	gboolean		inactivity_override;
+	gboolean		inactivity_override, is_MD;
 	static gboolean	initial_read = TRUE;
 
 	if (!f && (f = fopen(PROC_DISKSTATS_FILE, "r")) == NULL)
@@ -562,8 +566,9 @@ linux_read_proc_diskstats(void)
 			|| !disk_get_device_name(major, minor, disk, part)
 		   )
 			continue;
+		is_MD = (major == MD_MAJOR);
 		if (part[0] == '\0')
-			gkrellm_disk_assign_data_by_name(disk, 512 * rd, 512 * wr);
+			gkrellm_disk_assign_data_by_name(disk, 512 * rd, 512 * wr, is_MD);
 		else
 			gkrellm_disk_subdisk_assign_data_by_name(part, disk,
 						512 * rd, 512 * wr);
@@ -584,6 +589,7 @@ linux_read_proc_partitions_or_sysfs(void)
 	gchar		buf[1024], part[64], disk[64], sysfspath[128];
 	gint		major, minor, n;
 	gulong		sectors, rd, wr;
+	gboolean	is_MD;
 
 	if ((f = fopen(PROC_PARTITIONS_FILE, "r")) != NULL)
 		{
@@ -591,9 +597,10 @@ linux_read_proc_partitions_or_sysfs(void)
 		fgets(buf, sizeof(buf), f); 
 		disk[0] = '\0';
 		part[0] = '\0';
-		
+
 		while ((fgets(buf, sizeof(buf), f)) != NULL)
 			{
+			major = 0;
 			if (have_partition_stats)
 				{
 				/* major minor  #blocks  name
@@ -630,8 +637,10 @@ linux_read_proc_partitions_or_sysfs(void)
 						continue;
 					}
 				}
+			is_MD = (major == MD_MAJOR);
 			if (part[0] == '\0')
-				gkrellm_disk_assign_data_by_name(disk, 512 * rd, 512 * wr);
+				gkrellm_disk_assign_data_by_name(disk,
+							512 * rd, 512 * wr, is_MD);
 			else
 				gkrellm_disk_subdisk_assign_data_by_name(part, disk,
 							512 * rd, 512 * wr);
@@ -1646,6 +1655,9 @@ acpi_battery_data(BatteryFile *bf)
 	if (!have_charging_state && on_line)	/* Another buggy ACPI */
 		charging = TRUE;
 
+	if (charging)
+		bf->got_full_cap = FALSE;			/* reread full_cap */
+
 	percent = cur_cap * 100 / bf->full_cap;
 	if (percent > 100)
 		{
@@ -1657,7 +1669,12 @@ acpi_battery_data(BatteryFile *bf)
 		}
 
 	if (cur_rate > 0)
-		time_left = 60 * cur_cap / cur_rate;
+		{
+		if (charging)
+			time_left = 60 * (bf->full_cap - cur_cap) / cur_rate;
+		else
+			time_left = 60 * cur_cap / cur_rate;
+		}
 
 	if (_GK.debug_level & DEBUG_BATTERY)
 		{
@@ -1869,10 +1886,25 @@ gkrellm_sys_uptime_init(void)
 #define	SENSORS_DIR			"/proc/sys/dev/sensors"
 #define SYSFS_I2C_DIR		"/sys/bus/i2c/devices"
 
-#define	LM_SENSOR				0
-#define	THERMAL_SENSOR			1
-#define	THERMAL_ZONE_SENSOR		2
-#define	HDDTEMP_SENSOR			3
+  /* mbmon and hddtemp sensor interfaces are handled in sensors-common.c
+  */
+#define	LM_SENSOR_INTERFACE			0
+#define	THERMAL_INTERFACE			1
+#define	THERMAL_ZONE_INTERFACE		2
+#define	NVIDIA_SETTINGS_INTERFACE	3
+#define IBM_ACPI_INTERFACE		4
+
+#define IBM_ACPI_FAN_FILE	"/proc/acpi/ibm/fan"
+#define IBM_ACPI_THERMAL	"/proc/acpi/ibm/thermal"
+#define IBM_ACPI_CPU_TEMP        0
+#define IBM_ACPI_PCI_TEMP        1
+#define IBM_ACPI_HDD_TEMP        2
+#define IBM_ACPI_GPU_TEMP        3
+#define IBM_ACPI_BAT1_TEMP       4
+#define IBM_ACPI_NA1_TEMP        5
+#define IBM_ACPI_BAT2_TEMP       6
+#define IBM_ACPI_NA2_TEMP        7
+#define IBM_ACPI_FAN		 8
 
 typedef struct
 	{
@@ -2000,143 +2032,20 @@ static VoltDefault	voltdefault9[] =
 	};
 
 
-typedef struct
-	{
-	gchar	*device;
-	gfloat	value;
-	gchar	unit;
-	}
-	Hddtemp;
-
-GList	*hddtemp_list;
-
-static Hddtemp *
-hddtemp_lookup(gchar *device)
-	{
-	GList	*list;
-	Hddtemp	*hdd;
-
-	for (list = hddtemp_list; list; list = list->next)
-		{
-		hdd = (Hddtemp *) list->data;
-		if (!strcmp(device, hdd->device))
-			return hdd;
-		}
-	return NULL;
-	}
-
-static void
-hddtemp_daemon_read(void)
-	{
-	struct sockaddr_in	sin;
-	struct hostent		*he;
-	gpointer			sr;
-	Hddtemp				*hdd;
-	gchar				**argv, **info, *id_name, *default_label;
-	gchar				buf[256], sep;
-	gint				fd, n, j;
-	static GString		*hddtemp_gstring;
-
-	if (!hddtemp_gstring)
-		hddtemp_gstring = g_string_new("");
-	hddtemp_gstring = g_string_truncate(hddtemp_gstring, 0);
-
-	if ((he = gethostbyname("127.0.0.1")) == NULL)
-		return;
-	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
-		return;
-
-	memset(&sin, 0, sizeof(struct sockaddr_in));
-	memcpy(&(sin.sin_addr.s_addr), he->h_addr, he->h_length);
-	sin.sin_family = AF_INET;
-	sin.sin_port = htons(7634);
-
-	if (connect(fd, (struct sockaddr *) &sin, (socklen_t) sizeof(sin)) == -1)
-		{
-		close(fd);
-		return;
-		}
-
-	while ((n = read(fd, buf, sizeof(buf) - 1)) > 0)
-		{
-		buf[n] = '\0';
-		hddtemp_gstring = g_string_append(hddtemp_gstring, buf);
-		}
-	close(fd);
-
-	if (_GK.debug_level & DEBUG_SENSORS)
-		printf("hddtemp_daemon_read: %s\n", hddtemp_gstring->str);
-
-	sep = hddtemp_gstring->str[0];
-	if (sep == '\0')
-		return;
-	sprintf(buf, "%c%c", sep, sep);
-
-	argv = g_strsplit(hddtemp_gstring->str + 1, buf, 20);
-	buf[1] = '\0';
-	for (n = 0; argv[n] != NULL; ++n)
-		{
-		info = g_strsplit(argv[n], buf, 4);
-		for (j = 0; info[j] != NULL; ++j)
-			;
-		if (j < 4)
-			{
-			g_strfreev(info);
-			continue;
-			}
-		if ((hdd = hddtemp_lookup(info[0])) == NULL)
-			{
-			hdd = g_new0(Hddtemp, 1);
-			hddtemp_list = g_list_append(hddtemp_list, hdd);
-			hdd->device = g_strdup(info[0]);
-			default_label = strrchr(hdd->device, '/');
-			if (default_label)
-				++default_label;
-			else
-				default_label = hdd->device;
-			id_name = g_strdup_printf("hddtemp/%s", default_label);
-			sr = gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
-						hdd->device, id_name,
-						0, 0, HDDTEMP_SENSOR,
-						1.0, 0.0, NULL, default_label);
-			gkrellm_sensors_set_group(sr, SENSOR_GROUP_DISK);
-			g_free(id_name);
-			}
-		hdd->value = atof(info[2]);
-		if (*info[3] == 'F')
-			hdd->value = (hdd->value - 32.0) / 1.8;
-		g_strfreev(info);
-		}
-	g_strfreev(argv);
-	}
-
-static void
-hddtemp_check(void)
-	{
-	static gint	check_time = -1;
-
-	/* hddtemp docs say shouldn't check more than once per minute.
-	*/
-	if (check_time < _GK.time_now)
-		{
-		hddtemp_daemon_read();
-		check_time = _GK.time_now + 60;
-		}
-	}
-
 
 gboolean
 gkrellm_sys_sensors_get_temperature(gchar *sensor_path, gint id,
 		gint iodev, gint interface, gfloat *temp)
 	{
 	FILE		*f;
-	Hddtemp		*hdd;
 	gchar		buf[64], units[32];
 	gint		n;
-	gfloat		T, t[5];
+	gfloat		T, t[5], ibm_acpi_temp[8];
 	gboolean	result = FALSE;
 
-	if (interface == THERMAL_SENSOR || interface == THERMAL_ZONE_SENSOR)
+	if (   interface == THERMAL_INTERFACE
+	    || interface == THERMAL_ZONE_INTERFACE
+	   )
 		{
 		f = fopen(sensor_path, "r");
 		if (f)
@@ -2158,15 +2067,63 @@ gkrellm_sys_sensors_get_temperature(gchar *sensor_path, gint id,
 		return result;
 		}
 
-	if (interface == HDDTEMP_SENSOR)
+	if (interface == IBM_ACPI_INTERFACE)
 		{
-		hddtemp_check();
-		if ((hdd = hddtemp_lookup(sensor_path)) != NULL)
+		f = fopen(sensor_path, "r");
+		if (f) 
 			{
-			*temp = hdd->value;
+			fgets(buf, sizeof(buf), f);
+			sscanf(buf, "temperatures: %f %f %f %f %f %f %f %f",
+						&ibm_acpi_temp[IBM_ACPI_CPU_TEMP],
+						&ibm_acpi_temp[IBM_ACPI_PCI_TEMP],
+						&ibm_acpi_temp[IBM_ACPI_HDD_TEMP],
+						&ibm_acpi_temp[IBM_ACPI_GPU_TEMP],
+						&ibm_acpi_temp[IBM_ACPI_BAT1_TEMP],
+						&ibm_acpi_temp[IBM_ACPI_NA1_TEMP],
+						&ibm_acpi_temp[IBM_ACPI_BAT2_TEMP],
+						&ibm_acpi_temp[IBM_ACPI_NA2_TEMP]);
+
+			*temp = ibm_acpi_temp[iodev];	
 			result = TRUE;
+			fclose(f);
 			}
+		return result;	
+		}
+
+	if (interface == HDDTEMP_INTERFACE)
+		{
+		gkrellm_sys_sensors_hddtemp_check();
+		return gkrellm_sys_sensors_hddtemp_get_value(sensor_path, temp);
+		}
+
+	if (interface == MBMON_INTERFACE)
+		{
+		gkrellm_sys_sensors_mbmon_check(FALSE);
+		return gkrellm_sys_sensors_mbmon_get_value(sensor_path, temp);
+		}
+
+	if (interface == NVIDIA_SETTINGS_INTERFACE)
+		{
+#if GLIB_CHECK_VERSION(2,0,0)
+		char *args[] = { "nvidia-settings", "-q", sensor_path, NULL };
+		char *output = NULL;
+
+		result = g_spawn_sync(NULL, args, NULL,
+					G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+					NULL, NULL, &output, NULL, NULL, NULL);
+
+		if(result && output)
+			{
+			gfloat dummy;
+			if(!temp) temp = &dummy;
+			result = (sscanf(output, " Attribute %*s %*s %f", temp) == 1);
+			}
+
+		g_free(output);
 		return result;
+#else
+		return FALSE;
+#endif
 		}
 
 	if ((f = fopen(sensor_path, "r")) == NULL)
@@ -2195,10 +2152,33 @@ gboolean
 gkrellm_sys_sensors_get_fan(gchar *sensor_path, gint id,
 		gint iodev, gint interface, gfloat *fan)
 	{
-	FILE	*f;
-	gchar	buf[64];
-	gint	n;
-	gfloat	t[4];
+	FILE		*f;
+	gchar		buf[64];
+	gint		n;
+	gfloat		t[4], T;
+	gboolean	result = FALSE;
+
+	if (interface == IBM_ACPI_INTERFACE)
+		{
+		f = fopen(sensor_path, "r");
+		if (f) 
+			{
+			fgets(buf, sizeof(buf), f);
+			fgets(buf, sizeof(buf), f);
+			sscanf(buf, "speed: %f", &T); 
+			fclose(f);
+			*fan = T;
+			result = TRUE;
+			}
+		return result;
+		}
+
+
+	if (interface == MBMON_INTERFACE)
+		{
+		gkrellm_sys_sensors_mbmon_check(FALSE);
+		return gkrellm_sys_sensors_mbmon_get_value(sensor_path, fan);
+		}
 
 	if ((f = fopen(sensor_path, "r")) == NULL)
 		return FALSE;
@@ -2225,6 +2205,12 @@ gkrellm_sys_sensors_get_voltage(gchar *sensor_path, gint id,
 	gfloat	V, t[3];
 	gint	n;
 
+	if (interface == MBMON_INTERFACE)
+		{
+		gkrellm_sys_sensors_mbmon_check(FALSE);
+		return gkrellm_sys_sensors_mbmon_get_value(sensor_path, volt);
+		}
+
 	if ((f = fopen(sensor_path, "r")) == NULL)
 		return FALSE;
 	fgets(buf, sizeof(buf), f);
@@ -2445,6 +2431,7 @@ extern void		gkrellm_sensors_linux_name_fix(gchar *chip_name);
 gboolean
 gkrellm_sys_sensors_init(void)
 	{
+	FILE			*f;
 	GDir			*dir, *chip_dir;
 	VoltDefault		*voltdefault;
 	gint			id = 0;
@@ -2470,7 +2457,7 @@ gkrellm_sys_sensors_init(void)
 				snprintf(id_name, sizeof(id_name), "thermal_zone/%s", name);
 				gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
 							path, id_name,
-							id, 0, THERMAL_ZONE_SENSOR,
+							id, 0, THERMAL_ZONE_INTERFACE,
 							1.0, 0.0, NULL, name);
 				}
 			g_free(path);
@@ -2488,7 +2475,7 @@ gkrellm_sys_sensors_init(void)
 				snprintf(id_name, sizeof(id_name), "thermal/%s", name);
 				gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
 							path, id_name,
-							id, 0, THERMAL_SENSOR,
+							id, 0, THERMAL_INTERFACE,
 							1.0, 0.0, NULL, name);
 				}
 			g_free(path);
@@ -2496,7 +2483,76 @@ gkrellm_sys_sensors_init(void)
 		g_dir_close(dir);
 		}
 
-	hddtemp_check();
+	/* Do intial daemon reads to get sensors loaded into sensors.c
+	*/
+	gkrellm_sys_sensors_hddtemp_check();
+	gkrellm_sys_sensors_mbmon_check(TRUE);
+
+	/* 
+	 * IBM ACPI Temperature Sensors
+	 */
+	if ((f = fopen(IBM_ACPI_THERMAL, "r")) != NULL)
+		{
+		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+				IBM_ACPI_THERMAL, "IBM ACPI CPU",
+				id, IBM_ACPI_CPU_TEMP, IBM_ACPI_INTERFACE,
+				1.0, 0.0, NULL, "CPU");
+	
+		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+				IBM_ACPI_THERMAL, "IBM ACPI Mini PCI Module",
+				id, IBM_ACPI_PCI_TEMP, IBM_ACPI_INTERFACE,
+				1.0, 0.0, NULL, "PCI");
+	
+		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+				IBM_ACPI_THERMAL, "IBM ACPI HDD",
+				id, IBM_ACPI_HDD_TEMP, IBM_ACPI_INTERFACE,
+				1.0, 0.0, NULL, "HDD");
+	
+		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+				IBM_ACPI_THERMAL, "IBM ACPI GPU",
+				id, IBM_ACPI_GPU_TEMP, IBM_ACPI_INTERFACE,
+				1.0, 0.0, NULL, "GPU");
+	
+		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+				IBM_ACPI_THERMAL, "IBM ACPI Battery 1",
+				id, IBM_ACPI_BAT1_TEMP, IBM_ACPI_INTERFACE,
+				1.0, 0.0, NULL, "BAT1");
+						
+		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+				IBM_ACPI_THERMAL, "IBM ACPI Battery 2",
+				id, IBM_ACPI_BAT2_TEMP, IBM_ACPI_INTERFACE,
+				1.0, 0.0, NULL, "BAT2");
+		fclose(f);
+		}
+
+	/* 
+	 * IBM ACPI Cooling Fan Sensor
+	 */
+	if ((f = fopen(IBM_ACPI_FAN_FILE, "r")) != NULL)
+		{
+		gkrellm_sensors_add_sensor(SENSOR_FAN,
+				IBM_ACPI_FAN_FILE, "IBM ACPI Fan Sensor",
+				id, IBM_ACPI_FAN, IBM_ACPI_INTERFACE,
+				1.0, 0.0, NULL, "Fan");
+		fclose(f);
+		}
+	
+	if (gkrellm_sys_sensors_get_temperature("GPUCoreTemp", id, 0,
+					NVIDIA_SETTINGS_INTERFACE, NULL))
+		{
+		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+					"GPUCoreTemp", "nVidia GPU Core",
+					id, 0, NVIDIA_SETTINGS_INTERFACE,
+					1.0, 0.0, NULL, "GPU C");
+		}
+	if (gkrellm_sys_sensors_get_temperature("GPUAmbientTemp", id, 0,
+				NVIDIA_SETTINGS_INTERFACE, NULL))
+		{
+		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE,
+					"GPUAmbientTemp", "nVidia GPU Ambient",
+					id, 0, NVIDIA_SETTINGS_INTERFACE,
+					1.0, 0.0, NULL, "GPU A");
+		}
 
 	if ((dir = g_dir_open(SENSORS_DIR, 0, NULL)) == NULL)
 		{
