@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -294,7 +294,7 @@ gkrellm_sys_disk_read_data(void)
             kstat_read(kc, ksp, &kios);
 
 	    gkrellm_disk_assign_data_by_name(drive->name,
-						kios.nread, kios.nwritten);
+						kios.nread, kios.nwritten, FALSE);
 	}
     }
 }
