@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -280,24 +280,30 @@ gkrellm_gdk_text_width(PangoFontDescription *font_desc,
 				const gchar *string, gint len)
 	{
 	PangoLayout				*layout;
-	PangoFontDescription	*fd;
 	gint					w, h;
 
-	if (font_desc == _GK.large_font)
-		fd = _GK.large_font;
-	else if (font_desc == _GK.small_font)
-		fd = _GK.small_font;
-	else
-		fd = _GK.normal_font;
-
 	layout = gtk_widget_create_pango_layout(gkrellm_get_top_window(), NULL);
-	pango_layout_set_font_description(layout, fd);
+	pango_layout_set_font_description(layout, font_desc);
 	pango_layout_set_text(layout, string, len);
 	pango_layout_get_pixel_size(layout, &w, &h);
 	g_object_unref(layout);
 	return w;	
 	}
 
+gint
+gkrellm_gdk_text_markup_width(PangoFontDescription *font_desc,
+				const gchar *string, gint len)
+	{
+	PangoLayout				*layout;
+	gint					w, h;
+
+	layout = gtk_widget_create_pango_layout(gkrellm_get_top_window(), NULL);
+	pango_layout_set_font_description(layout, font_desc);
+	pango_layout_set_markup(layout, string, len);
+	pango_layout_get_pixel_size(layout, &w, &h);
+	g_object_unref(layout);
+	return w;	
+	}
 
 gint
 gkrellm_gdk_string_width(PangoFontDescription *font_desc, gchar *string)
@@ -313,6 +319,20 @@ gkrellm_gdk_string_width(PangoFontDescription *font_desc, gchar *string)
 	return w;
 	}
 
+gint
+gkrellm_gdk_string_markup_width(PangoFontDescription *font_desc, gchar *string)
+	{
+	PangoLayout	*layout;
+	gint		w, h;
+
+	layout = gtk_widget_create_pango_layout(gkrellm_get_top_window(), NULL);
+	pango_layout_set_font_description(layout, font_desc);
+	pango_layout_set_markup(layout, string, strlen(string));
+	pango_layout_get_pixel_size(layout, &w, &h);
+	g_object_unref(layout);
+	return w;
+	}
+
 
 void
 gkrellm_text_extents(PangoFontDescription *font_desc, gchar *text,
@@ -350,6 +370,42 @@ gkrellm_text_extents(PangoFontDescription *font_desc, gchar *text,
 		*y_ink = ink.y - logical.y;
 	}
 
+void
+gkrellm_text_markup_extents(PangoFontDescription *font_desc, gchar *text,
+		gint len, gint *width, gint *height, gint *baseline, gint *y_ink)
+	{
+	PangoLayout		*layout;
+	PangoLayoutIter	*iter;
+	PangoRectangle	ink, logical;
+	gchar			*utf8;
+	gint			base;
+
+	layout = gtk_widget_create_pango_layout(gkrellm_get_top_window(), NULL);
+	pango_layout_set_font_description(layout, font_desc);
+	if (g_utf8_validate(text, -1, NULL))
+		pango_layout_set_markup(layout, text, len);
+	else
+		{
+		utf8 = g_locale_to_utf8(text, -1, NULL, NULL, NULL);
+		pango_layout_set_markup(layout, utf8, len);
+		g_free(utf8);
+		}
+	iter = pango_layout_get_iter(layout);
+	base = pango_layout_iter_get_baseline(iter) / PANGO_SCALE;
+	pango_layout_get_pixel_extents(layout, &ink, &logical);
+	pango_layout_iter_free(iter);
+	g_object_unref(layout);
+
+	if (width)
+		*width = logical.width;
+	if (height)
+		*height = ink.height;
+	if (baseline)
+		*baseline = base;
+	if (y_ink)
+		*y_ink = ink.y - logical.y;
+	}
+
 void
 gkrellm_gdk_draw_string(GdkDrawable *drawable, PangoFontDescription *font_desc,
 			GdkGC *gc, gint x, gint y, gchar *string)
@@ -363,7 +419,20 @@ gkrellm_gdk_draw_string(GdkDrawable *drawable, PangoFontDescription *font_desc,
 	g_object_unref(layout);
 	}
 
-  /* Deprecated as of 2.2.0 */
+void
+gkrellm_gdk_draw_string_markup(GdkDrawable *drawable,
+			PangoFontDescription *font_desc,
+			GdkGC *gc, gint x, gint y, gchar *string)
+	{
+	PangoLayout	*layout;
+
+	layout = gtk_widget_create_pango_layout(gkrellm_get_top_window(), NULL);
+	pango_layout_set_font_description(layout, font_desc);
+	pango_layout_set_markup(layout, string, strlen(string));
+	gdk_draw_layout(drawable, gc, x, y, layout);
+	g_object_unref(layout);
+	}
+
 void
 gkrellm_gdk_draw_text(GdkDrawable *drawable, PangoFontDescription *font_desc,
 			GdkGC *gc, gint x, gint y, gchar *string, gint len)
@@ -377,6 +446,20 @@ gkrellm_gdk_draw_text(GdkDrawable *drawable, PangoFontDescription *font_desc,
 	g_object_unref(layout);
 	}
 
+void
+gkrellm_gdk_draw_text_markup(GdkDrawable *drawable,
+			PangoFontDescription *font_desc,
+			GdkGC *gc, gint x, gint y, gchar *string, gint len)
+	{
+	PangoLayout	*layout;
+
+	layout = gtk_widget_create_pango_layout(gkrellm_get_top_window(), NULL);
+	pango_layout_set_font_description(layout, font_desc);
+	pango_layout_set_markup(layout, string, len);
+	gdk_draw_layout(drawable, gc, x, y, layout);
+	g_object_unref(layout);
+	}
+
   /* Gtk config widgets work with utf8, so as long as I'm using gdk_draw
   |  functions, both utf8 and current locale versions of strings drawn on
   |  GKrellM must be maintained.  If src is not utf8, *dst is converted
