@@ -473,7 +473,6 @@ typedef struct {
 
       /* Functions new for 2.1.9
       */
-#if 0
 	GkrellmPiximage *(*gkrellm_piximage_new_from_inline)(const guint8 *data,
 						gboolean copy_pixels);
     gboolean	(*gkrellm_load_piximage_from_inline)(gchar *name,
@@ -482,10 +481,9 @@ typedef struct {
 	void		(*gkrellm_alert_commands_config)(GkrellmAlert *alert,
 						gboolean alarm, gboolean warn);
     void		(*gkrellm_reset_alert_soft)(GkrellmAlert *);
-#endif
+
       /* Functions new for 2.1.11
       */
-#if 0
 	void		(*gkrellm_decal_text_clear)(GkrellmDecal *d);
 	void		(*gkrellm_decal_text_insert)(GkrellmDecal *d, gchar *s,
 						GkrellmTextstyle *ts, gint x_off, gint y_off);
@@ -495,36 +493,32 @@ typedef struct {
 	void		(*gkrellm_chartconfig_callback_block)(GkrellmChartconfig *,
 						gboolean);
 
-#endif
-
       /* Functions new for 2.1.16
       */
-#if 0
-	void		gkrellm_alert_get_alert_state(GkrellmAlert *alert,
+	void		(*gkrellm_alert_get_alert_state)(GkrellmAlert *alert,
 						gboolean *alarm_state, gboolean *warn_state);
-	GkrellmAlertPlugin	*gkrellm_alert_plugin_add(GkrellmMonitor *mon,
+	GkrellmAlertPlugin	*(*gkrellm_alert_plugin_add)(GkrellmMonitor *mon,
 						gchar *name);
-	void		gkrellm_alert_plugin_connect(GkrellmAlertPlugin *gap,
+	void		(*gkrellm_alert_plugin_alert_connect)(GkrellmAlertPlugin *gap,
             			void (*alarm_func)(), void (*warn_func)(),
-						void (*update_func)(), void (*destroy_func)());
-	void		gkrellm_alert_plugin_config_connect(GkrellmAlertPlugin *gap,
+						void (*update_func)(), void (*check_func)(),
+						void (*destroy_func)());
+	void		(*gkrellm_alert_plugin_config_connect)(GkrellmAlertPlugin *gap,
 						gchar *tab_name,
             			void (*config_create_func)(), void (*config_done_func),
             			void (*config_save_func)(),void (*config_load_func)());
-	gchar		*gkrellm_alert_plugin_config_get_id_string(GkrellmAlert *alert);
-	void		gkrellm_alert_plugin_alert_attach(GkrellmAlertPlugin *gap,
+	gchar		*(*gkrellm_alert_plugin_config_get_id_string)(GkrellmAlert *alert);
+	void		(*gkrellm_alert_plugin_alert_attach)(GkrellmAlertPlugin *gap,
             			GkrellmAlert *alert, gpointer data);
-	void		gkrellm_alert_plugin_alert_detach(GkrellmAlertPlugin *gap,
+	void		(*gkrellm_alert_plugin_alert_detach)(GkrellmAlertPlugin *gap,
 						GkrellmAlert *alert);
-	gpointer	gkrellm_alert_plugin_get_data(GkrellmAlertPlugin *gap,
+	gpointer	(*gkrellm_alert_plugin_get_data)(GkrellmAlertPlugin *gap,
 						GkrellmAlert *alert);
-	void		gkrellm_alert_plugin_command_process(GkrellmAlert *alert,
+	void		(*gkrellm_alert_plugin_command_process)(GkrellmAlert *alert,
 						gchar *src, gchar *dst, gint dst_size);
-	GtkWidget	*gkrellm_gtk_category_vbox(GtkWidget *box, gchar *category_header,
+	GtkWidget	*(*gkrellm_gtk_category_vbox)(GtkWidget *box, gchar *category_header,
         				gint header_pad, gint box_pad, gboolean pack_start);
-	void		gkrellm_remove_launcher(GkrellmLauncher *launch);
-
-#endif
+	void		(*gkrellm_remove_launcher)(GkrellmLauncher *launch);
 
 } win32_plugin_callbacks;
 
