@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2003 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -267,7 +267,8 @@ gkrellm_winop_options(gint argc, gchar **argv)
             gdk_window_move(gkrellm_get_top_window()->window, 0, 0);
 			_GK.x_position = 0;
 			_GK.y_position = 0;
-            
+			_GK.position_valid = TRUE;
+    	    
             SetWindowLong(GDK_WINDOW_HWND(gkrellm_get_top_window()->window), GWL_USERDATA, 0x49474541);
     	    
             PostMessage(hwnd, BM_SLITMESSAGE, BSM_UPDATEPOSITIONS, 0);
@@ -276,6 +277,11 @@ gkrellm_winop_options(gint argc, gchar **argv)
     } 
 }
 
+void
+gkrellm_winop_withdrawn(void)
+    {
+	}
+
 void
 gkrellm_winop_place_gkrellm(gchar *geom)
     {
@@ -354,6 +360,7 @@ gkrellm_winop_place_gkrellm(gchar *geom)
 			_GK.y_position = newY;
 			_GK.x_position = newX;
 		}
+	_GK.position_valid = TRUE;
 	}
 
 void
@@ -367,6 +374,10 @@ gkrellm_winop_updated_background(void)
 	return FALSE;
 	}
 
+void  gkrellm_winop_update_struts(void) 
+	{
+	}
+
 gboolean
 gkrellm_winop_draw_rootpixmap_onto_transparent_chart(GkrellmChart *cp)
 	{
