@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2005 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -322,8 +322,7 @@ gkrellm_winop_update_struts(void)
 		net_wm_strut = XInternAtom(display, "_NET_WM_STRUT", False);
 		}
 
-	width = _GK.chart_width + _GK.frame_left_width + _GK.frame_right_width;
-	height = _GK.monitor_height +  _GK.total_frame_height;
+	gtk_window_get_size(GTK_WINDOW(gkrellm_get_top_window()), &width, &height);
 
 	if (_GK.x_position == 0)
 		{
